@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2017 the xine project
+ * Copyright (C) 2000-2019 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -53,7 +53,7 @@ typedef union {         /* CLUT == Color LookUp Table */
   uint32_t u32;
 } __attribute__ ((packed)) clut_t;
 
-static const clut_t textpalettes_color[] = {
+static const clut_t textpalettes_color[OVL_PALETTE_SIZE] = {
   /* white, no border, translucid */
     CLUT_Y_CR_CB_INIT(0x00, 0x00, 0x00), //0
     CLUT_Y_CR_CB_INIT(0x60, 0x80, 0x80), //1
@@ -80,7 +80,7 @@ static const clut_t textpalettes_color[] = {
     CLUT_Y_CR_CB_INIT(0xff, 0x90, 0x00), //10
 };
 
-static const uint8_t textpalettes_trans[] = {
+static const uint8_t textpalettes_trans[OVL_PALETTE_SIZE] = {
   /* white, no border, translucid */
   0, 8, 9, 10, 11, 12, 13, 14, 15, 15, 15,
   /* yellow, black border, transparent */
@@ -572,7 +572,7 @@ void osd_stream_position(void) {
 }
 
 void osd_display_spu_lang(void) {
-  char   buffer[32];
+  char   buffer[XINE_LANG_MAX + 32];
   char   lang_buffer[XINE_LANG_MAX];
   int    channel;
   const char *lang = NULL;
@@ -601,7 +601,7 @@ void osd_display_spu_lang(void) {
 }
 
 void osd_display_audio_lang(void) {
-  char   buffer[32];
+  char   buffer[XINE_LANG_MAX + 32];
   char   lang_buffer[XINE_LANG_MAX];
   int    channel;
   const char *lang = NULL;
