@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2009 the xine project
+ * Copyright (C) 2000-2017 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -36,8 +36,8 @@ void osd_display_audio_lang(void);
 void osd_display_zoom(void);
 void osd_update_status(void);
 void osd_stream_position(void);
-void osd_display_info(char *info, ...) __attribute__ ((format (printf, 1, 2)));
+void osd_display_info(const char *info, ...) __attribute__ ((format (printf, 1, 2)));
   /* see OSD_BAR_* */
 void osd_stream_infos(void);
-void osd_draw_bar(char *title, int min, int max, int val, int type);
+void osd_draw_bar(const char *title, int min, int max, int val, int type);
 #endif
