@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2017 the xine project
+ * Copyright (C) 2000-2019 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -33,6 +33,8 @@
 
 #include "Imlib-light/Imlib.h"
 
+typedef struct gGui_st gGui_t;
+
 #include "xitk.h"
 
 #include "kbindings.h"
@@ -152,16 +154,21 @@
 /* Our default location for skin downloads */
 #define SKIN_SERVER_URL         "http://xine.sourceforge.net/skins/skins.slx"
 
-#define fontname       "-*-helvetica-medium-r-*-*-10-*-*-*-*-*-*-*"
-#define boldfontname   "-*-helvetica-bold-r-*-*-10-*-*-*-*-*-*-*"
+#define fontname       "-*-helvetica-medium-r-*-*-11-*-*-*-*-*-*-*"
+#define boldfontname   "-*-helvetica-bold-r-*-*-11-*-*-*-*-*-*-*"
 #define hboldfontname  "-*-helvetica-bold-r-*-*-12-*-*-*-*-*-*-*"
-#define lfontname      "-*-helvetica-bold-r-*-*-10-*-*-*-*-*-*-*"
+#define lfontname      "-*-helvetica-bold-r-*-*-11-*-*-*-*-*-*-*"
 #define tabsfontname   "-*-helvetica-bold-r-*-*-12-*-*-*-*-*-*-*"
-#define br_fontname    "-misc-fixed-medium-r-normal-*-10-*-*-*-*-*-*-*"
+#define br_fontname    "-misc-fixed-medium-r-normal-*-11-*-*-*-*-*-*-*"
 #define btnfontname    "-*-helvetica-bold-r-*-*-12-*-*-*-*-*-*-*"
 
+/* easy pseudo widgets for *_toggle_* calls. not all windows support them yet. */
+#define XUI_W_TOGGLE NULL
+#define XUI_W_ON ((xitk_widget_t *)1)
+#define XUI_W_OFF ((xitk_widget_t *)2)
 
-typedef struct {
+struct gGui_st {
+  xine_t                   *xine;
   xine_video_port_t        *vo_port;
   xine_video_port_t        *vo_none;
   xine_audio_port_t        *ao_none;
@@ -180,16 +187,6 @@ typedef struct {
   int                       deinterlace_elements_num;
   int                       deinterlace_enable;
 
-  struct {
-    int                     hue, default_hue;
-    int                     brightness, default_brightness;
-    int                     saturation, default_saturation;
-    int                     contrast, default_contrast;
-    int                     gamma, default_gamma;
-    int                     sharpness, default_sharpness;
-    int                     noise_reduction, default_noise_reduction;
-  } video_settings;
-
   xine_audio_port_t        *ao_port;
 
   xine_stream_t            *stream;
@@ -205,8 +202,12 @@ typedef struct {
 
   int                       broadcast_port;
 
-  pthread_mutex_t           xe_mutex;
-  int                       new_pos;
+  /* Seek through xine_play () may be slow. We defer it to a separate thread. */
+  pthread_mutex_t           seek_mutex;
+  /* 0 = no, 1 = yes, 2 = shutdown */
+  int                       seek_running;
+  int                       seek_pos;
+  int                       seek_timestep;
 
   xine_event_queue_t       *event_queue;
 
@@ -255,6 +256,8 @@ typedef struct {
   char                     *autoscan_plugin;
 
   /* basic X11 stuff */
+  void                    (*x_lock_display) (Display *display);
+  void                    (*x_unlock_display) (Display *display);
   Display                  *display;
   int                       screen;
   int		            depth;
@@ -289,7 +292,7 @@ typedef struct {
 
   /* Current mediamark */
   mediamark_t               mmk;
-
+  /* Recursive mutex, protecting .mmk and .playlist. */
   pthread_mutex_t           mmk_mutex;
   
   /* playlist */
@@ -384,16 +387,36 @@ typedef struct {
   FILE                      *orig_stdout; /* original stdout at startup        */
                                           /* before an evtl. later redirection */
 
-  /* xine_open() may wait for our event handler */
-  int                        suppress_messages;
-} gGui_t;
+  /* suspend event messages */
+  struct {
+    pthread_mutex_t          mutex;
+    struct timeval           until;
+    int                      level;
+  } no_messages;
+
+  xui_panel_t               *panel;
+  xui_vwin_t                *vwin;
+  xui_setup_t               *setup;
+  xui_mrlb_t                *mrlb;
+  xui_vctrl_t               *vctrl;
+
+  /* actions.c */
+  filebrowser_t             *load_stream;
+  filebrowser_t             *load_sub;
+  int                        last_playback_speed;
+
+  /* main.c */
+  char                     **video_driver_ids;
+  char                     **audio_driver_ids;
+  window_attributes_t        window_attribute;
+};
 
 extern gGui_t *gGui;
 
 void set_window_states_start(Window window);
 #define set_window_states_start(window)                                   \
   do {                                                                    \
-    if(!video_window_is_visible())                                        \
+    if(!video_window_is_visible(gGui->vwin))                              \
       xitk_set_wm_window_type((window), WINDOW_TYPE_NORMAL);              \
     else                                                                  \
       xitk_unset_wm_window_type((window), WINDOW_TYPE_NORMAL);            \
@@ -404,13 +427,13 @@ void set_window_states_start(Window window);
 void reparent_window(Window window);
 #define reparent_window(window)                                                          \
   do {                                                                                   \
-    if((!(video_window_get_fullscreen_mode() & WINDOWED_MODE)) && !wm_not_ewmh_only()) { \
+    if((!(video_window_get_fullscreen_mode(gGui->vwin) & WINDOWED_MODE)) && !wm_not_ewmh_only()) { \
       XLockDisplay(gGui->display);                                                       \
       /* Don't unmap this window, because on re-mapping, it will not be visible until    \
 	 its ancestor, the video window, is visible. That's not what's intended.         \
          XUnmapWindow(gGui->display, (window));                                          \
       */								                 \
-      if(!video_window_is_visible())                                                     \
+      if(!video_window_is_visible(gGui->vwin))                                           \
         xitk_set_wm_window_type((window), WINDOW_TYPE_NORMAL);                           \
       else                                                                               \
         xitk_unset_wm_window_type((window), WINDOW_TYPE_NORMAL);                         \
