@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2017 the xine project
+ * Copyright (C) 2000-2019 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -50,8 +50,6 @@
 /*
  * global variables
  */
-extern _panel_t        *panel;
-
 static pid_t            xine_pid;
 
 /* Icon data */
@@ -111,7 +109,7 @@ int hidden_file_cb(int action, int value) {
   xine_cfg_entry_t  cfg_entry;
   int               retval = 0;
   
-  if(xine_config_lookup_entry(__xineui_global_xine_instance, "media.files.show_hidden_files", &cfg_entry)) {
+  if(xine_config_lookup_entry (gGui->xine, "media.files.show_hidden_files", &cfg_entry)) {
     if(action)
       config_update_bool("media.files.show_hidden_files", value);
     else
@@ -123,27 +121,29 @@ int hidden_file_cb(int action, int value) {
 
 void dummy_config_cb(void *data, xine_cfg_entry_t *cfg) {
   /* It exist to avoid "restart" window message in setup window */
+  (void)data;
+  (void)cfg;
 }
 static void auto_vo_visibility_cb(void *data, xine_cfg_entry_t *cfg) {
-  gGui_t *gui = gGui;
+  gGui_t *gui = data;
   gui->auto_vo_visibility = cfg->num_value;
 }
 static void auto_panel_visibility_cb(void *data, xine_cfg_entry_t *cfg) {
-  gGui_t *gui = gGui;
+  gGui_t *gui = data;
   gui->auto_panel_visibility = cfg->num_value;
 }
 static void skip_by_chapter_cb(void *data, xine_cfg_entry_t *cfg) {
-  gGui_t *gui = gGui;
+  gGui_t *gui = data;
   gui->skip_by_chapter = cfg->num_value;
-  panel_update_nextprev_tips();
+  panel_update_nextprev_tips (gui->panel);
 }
 static void ssaver_timeout_cb(void *data, xine_cfg_entry_t *cfg) {
-  gGui_t *gui = gGui;
+  gGui_t *gui = data;
   gui->ssaver_timeout = cfg->num_value;
 }
 
 static void visual_anim_cb(void *data, xine_cfg_entry_t *cfg) {
-  gGui_t *gui = gGui;
+  gGui_t *gui = data;
   
   if(gui->visual_anim.enabled == cfg->num_value)
     return;
@@ -186,7 +186,7 @@ static void visual_anim_cb(void *data, xine_cfg_entry_t *cfg) {
 }
 
 static void stream_info_auto_update_cb(void *data, xine_cfg_entry_t *cfg) {
-  gGui_t *gui = gGui;
+  gGui_t *gui = data;
   gui->stream_info_auto_update = cfg->num_value;
   stream_infos_toggle_auto_update();
 }
@@ -195,11 +195,11 @@ static void stream_info_auto_update_cb(void *data, xine_cfg_entry_t *cfg) {
  * Layer above callbacks
  */
 static void layer_above_cb(void *data, xine_cfg_entry_t *cfg) {
-  gGui_t *gui = gGui;
+  gGui_t *gui = data;
   gui->layer_above = cfg->num_value;
 }
 static void always_layer_above_cb(void *data, xine_cfg_entry_t *cfg) {
-  gGui_t *gui = gGui;
+  gGui_t *gui = data;
   gui->always_layer_above = cfg->num_value;
 }
 
@@ -207,7 +207,7 @@ static void always_layer_above_cb(void *data, xine_cfg_entry_t *cfg) {
  * Callback for snapshots saving location.
  */
 static void snapshot_loc_cb(void *data, xine_cfg_entry_t *cfg) {
-  gGui_t *gui = gGui;
+  gGui_t *gui = data;
   gui->snapshot_location = cfg->str_value;
 }
 
@@ -215,7 +215,7 @@ static void snapshot_loc_cb(void *data, xine_cfg_entry_t *cfg) {
  * Callback for skin server
  */
 static void skin_server_url_cb(void *data, xine_cfg_entry_t *cfg) {
-  gGui_t *gui = gGui;
+  gGui_t *gui = data;
   gui->skin_server_url = cfg->str_value;
 }
 
@@ -223,55 +223,41 @@ static void skin_server_url_cb(void *data, xine_cfg_entry_t *cfg) {
  * OSD cbs
  */
 static void osd_enabled_cb(void *data, xine_cfg_entry_t *cfg) {
-  gGui_t *gui = gGui;
+  gGui_t *gui = data;
   gui->osd.enabled = cfg->num_value;
 }
 static void osd_use_unscaled_cb(void *data, xine_cfg_entry_t *cfg) {
-  gGui_t *gui = gGui;
+  gGui_t *gui = data;
   gui->osd.use_unscaled = cfg->num_value;
 }
 static void osd_timeout_cb(void *data, xine_cfg_entry_t *cfg) {
-  gGui_t *gui = gGui;
+  gGui_t *gui = data;
   gui->osd.timeout = cfg->num_value;
 }
 
 static void smart_mode_cb(void *data, xine_cfg_entry_t *cfg) {
-  gGui_t *gui = gGui;
+  gGui_t *gui = data;
   gui->smart_mode = cfg->num_value;
 }
 
 static void play_anyway_cb(void *data, xine_cfg_entry_t *cfg) {
-  gGui_t *gui = gGui;
+  gGui_t *gui = data;
   gui->play_anyway = cfg->num_value;
 }
 
 static void exp_level_cb(void *data, xine_cfg_entry_t *cfg) {
-  gGui_t *gui = gGui;
+  gGui_t *gui = data;
   gui->experience_level = (cfg->num_value * 10);
 }
 
 static void audio_mixer_method_cb(void *data, xine_cfg_entry_t *cfg) {
-  gGui_t *gui = gGui;
-  int max = 100, vol = 0;
-  
+  gGui_t *gui = data;
   gui->mixer.method = cfg->num_value;
-
-  switch(gui->mixer.method) {
-  case SOUND_CARD_MIXER:
-    max = 100;
-    vol = gui->mixer.volume_level;
-    break;
-  case SOFTWARE_MIXER:
-    max = 200;
-    vol = gui->mixer.amp_level;
-    break;
-  }
-
-  xitk_slider_set_max(panel->mixer.slider, max);
-  xitk_slider_set_pos(panel->mixer.slider, vol);
+  panel_update_mixer_display (gui->panel);
 }
+
 static void shortcut_style_cb(void *data, xine_cfg_entry_t *cfg) {
-  gGui_t *gui = gGui;
+  gGui_t *gui = data;
   gui->shortcut_style = cfg->num_value;
 }
 
@@ -300,17 +286,20 @@ int actions_on_start(action_id_t actions[], action_id_t a) {
  *
  */
 static void dummy_sighandler(int dummy) {
+  (void)dummy;
 }
+
 static void gui_signal_handler (int sig, void *data) {
   pid_t     cur_pid = getppid();
-  
+
+  (void)data;
   switch (sig) {
 
   case SIGHUP:
     if(cur_pid == xine_pid) {
       printf("SIGHUP received: re-read config file\n");
-      xine_config_reset(__xineui_global_xine_instance);
-      xine_config_load(__xineui_global_xine_instance, __xineui_global_config_file);
+      xine_config_reset (gGui->xine);
+      xine_config_load (gGui->xine, __xineui_global_config_file);
     }
     break;
 
@@ -331,7 +320,7 @@ static void gui_signal_handler (int sig, void *data) {
     if(cur_pid == xine_pid) {
       struct sigaction action;
 
-      xine_config_save(__xineui_global_xine_instance, __xineui_global_config_file);
+      xine_config_save (gGui->xine, __xineui_global_config_file);
       
       action.sa_handler = dummy_sighandler;
       sigemptyset(&(action.sa_mask));
@@ -379,8 +368,7 @@ static const char *pts2str(int64_t pts) {
 /*
  *
  */
-void gui_execute_action_id(action_id_t action) {
-  gGui_t *gui = gGui;
+void gui_execute_action_id (gGui_t *gui, action_id_t action) {
   xine_event_t   xine_event;
   int narg = -1;
   char *sarg = NULL;
@@ -487,12 +475,12 @@ void gui_execute_action_id(action_id_t action) {
       /* Limit size to a practical minimum.                         */
       /* Too small window is hard to view or to hit with the mouse. */
 
-      video_window_get_output_size(&output_width, &output_height);
+      video_window_get_output_size (gui->vwin, &output_width, &output_height);
       if(output_width > 50 && output_height > 50) {
 	float	xmag, ymag;
 
-	video_window_get_mag(&xmag, &ymag);
-	video_window_set_mag(xmag * (1/1.2f), ymag * (1/1.2f));
+        video_window_get_mag (gui->vwin, &xmag, &ymag);
+        video_window_set_mag (gui->vwin, xmag * (1/1.2f), ymag * (1/1.2f));
       }
     }
     break;
@@ -508,32 +496,32 @@ void gui_execute_action_id(action_id_t action) {
       /* size (no further ConfigureNotify after size limit is exceeded),           */
       /* so we'll not end up in endless magnification.                             */
 
-      video_window_get_output_size(&output_width, &output_height);
+      video_window_get_output_size (gui->vwin, &output_width, &output_height);
       xitk_get_window_position(gui->video_display, gui->video_window,
 			       NULL, NULL, &window_width, &window_height);
       if(output_width < 5000 && output_height < 5000 &&
 	 output_width <= window_width && output_height <= window_height) {
 	float	xmag, ymag;
 
-	video_window_get_mag(&xmag, &ymag);
-	video_window_set_mag(xmag * 1.2f, ymag * 1.2f);
+        video_window_get_mag (gui->vwin, &xmag, &ymag);
+        video_window_set_mag (gui->vwin, xmag * 1.2f, ymag * 1.2f);
       }
     }
     break;
 
   case ACTID_ZOOM_1_1:
   case ACTID_WINDOW100:
-    if(video_window_set_mag (1.0f, 1.0f))
+    if (video_window_set_mag (gui->vwin, 1.0f, 1.0f))
       osd_display_info(_("Zoom: 1:1"));
     break;
 
   case ACTID_WINDOW200:
-    if(video_window_set_mag (2.0f, 2.0f))
+    if (video_window_set_mag (gui->vwin, 2.0f, 2.0f))
       osd_display_info(_("Zoom: 200%%"));
     break;
 
   case ACTID_WINDOW50:
-    if(video_window_set_mag (0.5f, 0.5f))
+    if (video_window_set_mag (gui->vwin, 0.5f, 0.5f))
       osd_display_info(_("Zoom: 50%%"));
     break;
 
@@ -552,33 +540,33 @@ void gui_execute_action_id(action_id_t action) {
     break;
     
   case ACTID_CONTROLSHOW:
-    gui_control_show(NULL, NULL);
+    gui_control_show (NULL, gui->vctrl);
     break;
     
   case ACTID_TOGGLE_WINOUT_VISIBLITY:
-    gui_toggle_visibility(NULL, NULL);
+    gui_toggle_visibility (NULL, gui);
     break;
     
   case ACTID_TOGGLE_WINOUT_BORDER:
-    gui_toggle_border(NULL, NULL);
+    gui_toggle_border (NULL, gui);
     break;
 
   case ACTID_AUDIOCHAN_NEXT:
     if (narg < 0)
       gui_change_audio_channel(NULL, (void*)GUI_NEXT);
     else
-      gui_direct_change_audio_channel (NULL, (void*)GUI_NEXT, narg);
+      gui_direct_change_audio_channel (NULL, gui, narg);
     break;
     
   case ACTID_AUDIOCHAN_PRIOR:
     if (narg < 0)
       gui_change_audio_channel(NULL, (void*)GUI_PREV);
     else
-      gui_direct_change_audio_channel (NULL, (void*)GUI_PREV, narg);
+      gui_direct_change_audio_channel (NULL, gui, narg);
     break;
     
   case ACTID_PAUSE:
-    gui_pause (NULL, (void*)(1), 0); 
+    gui_pause (NULL, gui, 0);
     break;
 
   case ACTID_PLAYLIST:
@@ -586,16 +574,16 @@ void gui_execute_action_id(action_id_t action) {
     break;
       
   case ACTID_TOGGLE_VISIBLITY:
-    panel_toggle_visibility (NULL, NULL);
+    panel_toggle_visibility (NULL, gui->panel);
     break;
 
   case ACTID_TOGGLE_FULLSCREEN:
     if (narg >= 0) {
-      int fullscreen = video_window_get_fullscreen_mode() & FULLSCR_MODE;
+      int fullscreen = video_window_get_fullscreen_mode (gui->vwin) & FULLSCR_MODE;
       if ((narg && !fullscreen) || (!narg && fullscreen))
-        gui_set_fullscreen_mode(NULL, NULL);
+        gui_set_fullscreen_mode (NULL, gui);
     } else
-      gui_set_fullscreen_mode(NULL, NULL);
+      gui_set_fullscreen_mode (NULL, gui);
     break;
 
 #ifdef HAVE_XINERAMA
@@ -625,19 +613,19 @@ void gui_execute_action_id(action_id_t action) {
     if ((gui->event_pending <= 0) && gui->event_reject)
       pthread_cond_signal (&gui->event_safe);
     pthread_mutex_unlock (&gui->event_mutex);
-    gui_exit(NULL, NULL);
+    gui_exit (NULL, gui);
     break;
 
   case ACTID_PLAY:
-    gui_play(NULL, NULL);
+    gui_play (NULL, gui);
     break;
 
   case ACTID_STOP:
-    gui_stop(NULL, NULL);
+    gui_stop (NULL, gui);
     break;
 
   case ACTID_CLOSE:
-    gui_close(NULL, NULL);
+    gui_close (NULL, gui);
     break;
 
   case ACTID_EVENT_SENDER:
@@ -660,9 +648,9 @@ void gui_execute_action_id(action_id_t action) {
 
   case ACTID_MRL_SELECT:
     if (narg < 0)
-      gui_playlist_play(0);
+      gui_playlist_play (gui, 0);
     else
-      gui_playlist_play (narg);
+      gui_playlist_play (gui, narg);
     break;
       
   case ACTID_SETUP:
@@ -670,7 +658,7 @@ void gui_execute_action_id(action_id_t action) {
     break;
 
   case ACTID_EJECT:
-    gui_eject(NULL, NULL);
+    gui_eject (NULL, gui);
     break;
 
   case ACTID_SET_CURPOS:
@@ -774,7 +762,7 @@ void gui_execute_action_id(action_id_t action) {
     break;
     
   case ACTID_MUTE:
-    panel_toggle_audio_mute(NULL, NULL, !gui->mixer.mute);
+    panel_toggle_audio_mute (NULL, gui->panel, !gui->mixer.mute);
     break;
     
   case ACTID_AV_SYNC_m3600:
@@ -887,7 +875,7 @@ void gui_execute_action_id(action_id_t action) {
     break;
 
   case ACTID_SNAPSHOT:
-    panel_snapshot(NULL, NULL);
+    panel_snapshot (NULL, gui->panel);
     break;
     
   case ACTID_ZOOM_IN:
@@ -920,19 +908,19 @@ void gui_execute_action_id(action_id_t action) {
     
   case ACTID_GRAB_POINTER:
     if(!gui->cursor_grabbed) {
-      if(!panel_is_visible()) {
-	XLockDisplay(gui->video_display);
+      if (!panel_is_visible (gui->panel)) {
+	gui->x_lock_display (gui->video_display);
 	XGrabPointer(gui->video_display, gui->video_window, 1, None, 
 		     GrabModeAsync, GrabModeAsync, gui->video_window, None, CurrentTime);
-	XUnlockDisplay(gui->video_display);
+	gui->x_unlock_display (gui->video_display);
       }
       
       gui->cursor_grabbed = 1;
     }
     else {
-      XLockDisplay(gui->display);
+      gui->x_lock_display (gui->display);
       XUngrabPointer(gui->display, CurrentTime);
-      XUnlockDisplay(gui->display);
+      gui->x_unlock_display (gui->display);
       gui->cursor_grabbed = 0;
     }
     break;
@@ -966,7 +954,7 @@ void gui_execute_action_id(action_id_t action) {
 
   case ACTID_MRLIDENTTOGGLE:
     gui->is_display_mrl = !gui->is_display_mrl;
-    panel_update_mrl_display();
+    panel_update_mrl_display (gui->panel);
     playlist_mrlident_toggle();
     break;
     
@@ -1041,63 +1029,63 @@ void gui_execute_action_id(action_id_t action) {
     break;
 
   case ACTID_HUECONTROLp:
-    control_inc_image_prop(XINE_PARAM_VO_HUE);
+    control_inc_image_prop (gui->vctrl, XINE_PARAM_VO_HUE);
     break;
   case ACTID_HUECONTROLm:
-    control_dec_image_prop(XINE_PARAM_VO_HUE);
+    control_dec_image_prop (gui->vctrl, XINE_PARAM_VO_HUE);
     break;
 
   case ACTID_SATURATIONCONTROLp:
-    control_inc_image_prop(XINE_PARAM_VO_SATURATION);
+    control_inc_image_prop (gui->vctrl, XINE_PARAM_VO_SATURATION);
     break;
   case ACTID_SATURATIONCONTROLm:
-    control_dec_image_prop(XINE_PARAM_VO_SATURATION);
+    control_dec_image_prop (gui->vctrl, XINE_PARAM_VO_SATURATION);
     break;
 
   case ACTID_BRIGHTNESSCONTROLp:
-    control_inc_image_prop(XINE_PARAM_VO_BRIGHTNESS);
+    control_inc_image_prop (gui->vctrl, XINE_PARAM_VO_BRIGHTNESS);
     break;
   case ACTID_BRIGHTNESSCONTROLm:
-    control_dec_image_prop(XINE_PARAM_VO_BRIGHTNESS);
+    control_dec_image_prop (gui->vctrl, XINE_PARAM_VO_BRIGHTNESS);
     break;
 
   case ACTID_CONTRASTCONTROLp:
-    control_inc_image_prop(XINE_PARAM_VO_CONTRAST);
+    control_inc_image_prop (gui->vctrl, XINE_PARAM_VO_CONTRAST);
     break;
   case ACTID_CONTRASTCONTROLm:
-    control_dec_image_prop(XINE_PARAM_VO_CONTRAST);
+    control_dec_image_prop (gui->vctrl, XINE_PARAM_VO_CONTRAST);
     break;
 
   case ACTID_GAMMACONTROLp:
 #ifdef XINE_PARAM_VO_GAMMA
-    control_inc_image_prop(XINE_PARAM_VO_GAMMA);
+    control_inc_image_prop (gui->vctrl, XINE_PARAM_VO_GAMMA);
 #endif
     break;
   case ACTID_GAMMACONTROLm:
 #ifdef XINE_PARAM_VO_GAMMA
-    control_dec_image_prop(XINE_PARAM_VO_GAMMA);
+    control_dec_image_prop (gui->vctrl, XINE_PARAM_VO_GAMMA);
 #endif
     break;
 
   case ACTID_SHARPNESSCONTROLp:
 #ifdef XINE_PARAM_VO_SHARPNESS
-    control_inc_image_prop(XINE_PARAM_VO_SHARPNESS);
+    control_inc_image_prop (gui->vctrl, XINE_PARAM_VO_SHARPNESS);
 #endif
     break;
   case ACTID_SHARPNESSCONTROLm:
 #ifdef XINE_PARAM_VO_SHARPNESS
-    control_dec_image_prop(XINE_PARAM_VO_SHARPNESS);
+    control_dec_image_prop (gui->vctrl, XINE_PARAM_VO_SHARPNESS);
 #endif
     break;
 
   case ACTID_NOISEREDUCTIONCONTROLp:
 #ifdef XINE_PARAM_VO_NOISE_REDUCTION
-    control_inc_image_prop(XINE_PARAM_VO_NOISE_REDUCTION);
+    control_inc_image_prop (gui->vctrl, XINE_PARAM_VO_NOISE_REDUCTION);
 #endif
     break;
   case ACTID_NOISEREDUCTIONCONTROLm:
 #ifdef XINE_PARAM_VO_NOISE_REDUCTION
-    control_dec_image_prop(XINE_PARAM_VO_NOISE_REDUCTION);
+    control_dec_image_prop (gui->vctrl, XINE_PARAM_VO_NOISE_REDUCTION);
 #endif
     break;
 
@@ -1285,8 +1273,8 @@ void gui_execute_action_id(action_id_t action) {
         mediamark_load_mediamarks (sarg);
         gui_set_current_mmk(mediamark_get_current_mmk());
         playlist_update_playlist();
-        if((!is_playback_widgets_enabled()) && gui->playlist.num)
-        	enable_playback_controls(1);
+        if ((!is_playback_widgets_enabled (gui->panel)) && gui->playlist.num)
+          enable_playback_controls (gui->panel, 1);
     }
     break;
 
@@ -1306,20 +1294,20 @@ void gui_execute_action_id(action_id_t action) {
  * top-level event handler
  */
 void gui_handle_event (XEvent *event, void *data) {
-  gGui_t *gui = gGui;
+  gGui_t *gui = data;
 
   switch(event->type) {
 
   case MappingNotify:
-    XLockDisplay(gui->display);
+    gui->x_lock_display (gui->display);
     XRefreshKeyboardMapping((XMappingEvent *) event);
-    XUnlockDisplay(gui->display);
+    gui->x_unlock_display (gui->display);
     break;
 
   case DestroyNotify:
     if(event->xany.window == gui->panel_window
        || event->xany.window == gui->video_window) {
-      xine_exit (__xineui_global_xine_instance);
+      xine_exit (gui->xine);
       gui->running = 0;
     }
     break;
@@ -1331,7 +1319,7 @@ void gui_handle_event (XEvent *event, void *data) {
     /* printf ("button: %d\n",bevent->button); */
 
     if ((bevent->button == 3) && (bevent->window == gui->video_window))
-      panel_toggle_visibility (NULL, NULL);
+      panel_toggle_visibility (NULL, gui->panel);
     
   }
   break;
@@ -1358,12 +1346,11 @@ void gui_handle_event (XEvent *event, void *data) {
 /*
  * Start playback of an entry in playlist
  */
-int gui_playlist_play(int idx) {
-  gGui_t *gui = gGui;
+int gui_playlist_play (gGui_t *gui, int idx) {
   int ret = 1;
 
   osd_hide();
-  panel_reset_slider ();
+  panel_reset_slider (gui->panel);
 
   if(idx >= gui->playlist.num)
     return 0;
@@ -1383,14 +1370,13 @@ int gui_playlist_play(int idx) {
 /*
  * Start playback to next entry in playlist (or stop the engine, then display logo).
  */
-void gui_playlist_start_next() {
-  gGui_t *gui = gGui;
+void gui_playlist_start_next (gGui_t *gui) {
 
   if (gui->ignore_next)
     return;
 
   osd_hide();
-  panel_reset_slider ();
+  panel_reset_slider (gui->panel);
 
   if(gui->playlist.control & PLAYLIST_CONTROL_STOP) {
     if(!(gui->playlist.control & PLAYLIST_CONTROL_STOP_PERSIST))
@@ -1399,9 +1385,9 @@ void gui_playlist_start_next() {
     return;
   }
 
-  if(is_playback_widgets_enabled() && (!gui->playlist.num)) {
+  if (is_playback_widgets_enabled (gui->panel) && (!gui->playlist.num)) {
     gui_set_current_mmk(NULL);
-    enable_playback_controls(0);
+    enable_playback_controls (gui->panel, 0);
     gui_display_logo();
     return;
   }
@@ -1418,7 +1404,7 @@ void gui_playlist_start_next() {
 	mediamark_reset_played_state();
 
 	if(gui->actions_on_start[0] == ACTID_QUIT)
-	  gui_exit(NULL, NULL);
+          gui_exit (NULL, gui);
 	else
 	  gui_display_logo();
 	return;
@@ -1445,7 +1431,7 @@ void gui_playlist_start_next() {
       if(gui->playlist.loop == PLAYLIST_LOOP_SHUF_PLUS)
 	goto __shuffle_restart;
       else if(gui->actions_on_start[0] == ACTID_QUIT)
-      	gui_exit(NULL, NULL);
+        gui_exit (NULL, gui);
       else
 	gui_display_logo();
       return;    
@@ -1453,14 +1439,14 @@ void gui_playlist_start_next() {
     break;
   }
 
-  if(gui_playlist_play(gui->playlist.cur))
+  if (gui_playlist_play (gui, gui->playlist.cur))
     return;
 
   switch(gui->playlist.loop) {
 
   case PLAYLIST_LOOP_NO_LOOP:
     if(mediamark_all_played() && (gui->actions_on_start[0] == ACTID_QUIT)) {
-      gui_exit(NULL, NULL);
+      gui_exit (NULL, gui);
       return;
     }
     break;
@@ -1566,8 +1552,10 @@ static void gui_find_visual (Visual **visual_return, int *depth_return) {
     *visual_return = visual;
 }
 
-void gui_deinit(void) {
-  gGui_t *gui = gGui;
+void gui_deinit (gGui_t *gui) {
+#ifdef HAVE_XINE_CONFIG_UNREGISTER_CALLBACKS
+  xine_config_unregister_callbacks (gui->xine, NULL, NULL, gui, sizeof (*gui));
+#endif
   pthread_mutex_lock (&gui->event_mutex);
   gui->event_reject = 1;
   while (gui->event_pending > 0)
@@ -1580,8 +1568,12 @@ void gui_deinit(void) {
 /*
  * Initialize the GUI
  */
-void gui_init (int nfiles, char *filenames[], window_attributes_t *window_attribute) {
-  gGui_t *gui = gGui;
+
+static void gui_dummy_un_lock_display (Display *display) {
+  (void)display;
+}
+
+void gui_init (gGui_t *gui, int nfiles, char *filenames[], window_attributes_t *window_attribute) {
   int    i;
   char  *server;
   char  *video_display_name;
@@ -1638,17 +1630,28 @@ void gui_init (int nfiles, char *filenames[], window_attributes_t *window_attrib
   
   gui->is_display_mrl = 0;
   gui->mrl_overrided  = 0;
-  gui->new_pos        = -1;
+  /* gui->new_pos        = -1; */
 
   /*
    * X / imlib stuff
    */
 
-  if(!XInitThreads()) {
-    printf (_("\nXInitThreads failed - looks like you don't have a "
-	    "thread-safe xlib.\n"));
+  if (xine_config_register_bool (gui->xine, "gui.use_XLockDisplay", 1,
+        _("Enable extra XLib thread safety."),
+        _("This is needed for some very old XLib/XCB versions.\n"
+          "Otherwise, it may boost or brake performance - just try out."),
+        CONFIG_LEVEL_ADV, NULL, NULL)) {
+    gui->x_lock_display = XLockDisplay;
+    gui->x_unlock_display = XUnlockDisplay;
+  } else {
+    gui->x_lock_display =
+    gui->x_unlock_display = gui_dummy_un_lock_display;
+  }
+
+  if (!XInitThreads ()) {
+    printf (_("\nXInitThreads failed - looks like you don't have a thread-safe xlib.\n"));
     exit(1);
-  } 
+  }
   
   if((gui->display = XOpenDisplay((getenv("DISPLAY")))) == NULL) {
     fprintf(stderr, _("Cannot open display\n"));
@@ -1656,7 +1659,7 @@ void gui_init (int nfiles, char *filenames[], window_attributes_t *window_attrib
   }
 
   video_display_name = 
-    (char *)xine_config_register_string (__xineui_global_xine_instance, "gui.video_display", 
+    (char *)xine_config_register_string (gui->xine, "gui.video_display", 
 					 "",
 					 _("Name of video display"),
 					 _("Use this setting to configure to which "
@@ -1680,7 +1683,7 @@ void gui_init (int nfiles, char *filenames[], window_attributes_t *window_attrib
     gui->video_display = gui->display;
   }
 
-  if (xine_config_register_bool (__xineui_global_xine_instance, "gui.xsynchronize", 
+  if (xine_config_register_bool (gui->xine, "gui.xsynchronize", 
 				 0,
 				 _("Synchronized X protocol (debug)"), 
 				 _("Do X transactions in synchronous mode. "
@@ -1689,82 +1692,82 @@ void gui_init (int nfiles, char *filenames[], window_attributes_t *window_attrib
 				 CONFIG_NO_CB,
 				 CONFIG_NO_DATA)) {
 
-    XLockDisplay(gui->display);
+    gui->x_lock_display (gui->display);
     XSynchronize (gui->display, True);
-    XUnlockDisplay(gui->display);
+    gui->x_unlock_display (gui->display);
     fprintf (stderr, _("Warning! Synchronized X activated - this is very slow...\n"));
   }
 
   gui->layer_above = 
-    xine_config_register_bool (__xineui_global_xine_instance, "gui.layer_above", 0,
+    xine_config_register_bool (gui->xine, "gui.layer_above", 0,
 			       _("Windows stacking"),
 			       _("Use WM layer property to place windows on top\n"
 				 "except for the video window in non-fullscreen mode."), 
 			       CONFIG_LEVEL_ADV,
 			       layer_above_cb,
-			       CONFIG_NO_DATA);
+			       gGui);
 
   gui->always_layer_above = 
-    xine_config_register_bool (__xineui_global_xine_instance, "gui.always_layer_above", 0,
+    xine_config_register_bool (gui->xine, "gui.always_layer_above", 0,
 			       _("Windows stacking (more)"),
 			       _("Use WM layer property to place windows on top\n"
 				 "for all windows (surpasses the 'layer_above' setting)."), 
 			       CONFIG_LEVEL_ADV,
 			       always_layer_above_cb,
-			       CONFIG_NO_DATA);
+			       gGui);
 
   gui->snapshot_location = 
-    (char *)xine_config_register_string (__xineui_global_xine_instance, "gui.snapshotdir", 
+    (char *)xine_config_register_string (gui->xine, "gui.snapshotdir", 
 					 (char *) (xine_get_homedir()),
 					 _("Snapshot location"),
 					 _("Where snapshots will be saved."),
 					 CONFIG_LEVEL_BEG,
 					 snapshot_loc_cb,
-					 CONFIG_NO_DATA);
+					 gGui);
   
   gui->ssaver_timeout =
-    xine_config_register_num (__xineui_global_xine_instance, "gui.screensaver_timeout", 10,
+    xine_config_register_num (gui->xine, "gui.screensaver_timeout", 10,
 			      _("Screensaver reset interval (s)"),
 			      _("Time, in seconds, between two faked events to keep a screensaver quiet, 0 to disable."),
 			      CONFIG_LEVEL_ADV,
 			      ssaver_timeout_cb,
-			      CONFIG_NO_DATA);
+			      gGui);
   
   gui->skip_by_chapter = 
-    xine_config_register_bool (__xineui_global_xine_instance, "gui.skip_by_chapter", 1,
+    xine_config_register_bool (gui->xine, "gui.skip_by_chapter", 1,
 			       _("Chapter hopping"),
 			       _("Play next|previous chapter instead of mrl (dvdnav)"), 
 			       CONFIG_LEVEL_ADV,
 			       skip_by_chapter_cb, 
-			       CONFIG_NO_DATA);
+			       gGui);
   
   gui->auto_vo_visibility = 
-    xine_config_register_bool (__xineui_global_xine_instance, "gui.auto_video_output_visibility", 0,
+    xine_config_register_bool (gui->xine, "gui.auto_video_output_visibility", 0,
 			       _("Visibility behavior of output window"),
 			       _("Hide video output window if there is no video in the stream"), 
 			       CONFIG_LEVEL_ADV,
 			       auto_vo_visibility_cb, 
-			       CONFIG_NO_DATA);
+			       gGui);
 
   gui->auto_panel_visibility = 
-    xine_config_register_bool (__xineui_global_xine_instance, "gui.auto_panel_visibility", 0,
+    xine_config_register_bool (gui->xine, "gui.auto_panel_visibility", 0,
 			       _("Visiblility behavior of panel"),
 			       _("Automatically show/hide panel window, according to auto_video_output_visibility"), 
 			       CONFIG_LEVEL_ADV,
 			       auto_panel_visibility_cb,
-			       CONFIG_NO_DATA); 
+			       gGui); 
  
   gui->eventer_sticky = 
-    xine_config_register_bool(__xineui_global_xine_instance, "gui.eventer_sticky", 
+    xine_config_register_bool (gui->xine, "gui.eventer_sticky", 
 			      1,
 			      _("Event sender behavior"),
 			      _("Event sender window stick to main panel"), 
 			      CONFIG_LEVEL_ADV,
 			      event_sender_sticky_cb,
-			      CONFIG_NO_DATA);
+			      gGui);
 
   gui->visual_anim.enabled = 
-    xine_config_register_enum(__xineui_global_xine_instance, "gui.visual_anim", 
+    xine_config_register_enum (gui->xine, "gui.visual_anim", 
 			      1, /* Post plugin */
 			      (char**)visual_anim_style,
 			      _("Visual animation style"),
@@ -1772,135 +1775,135 @@ void gui_init (int nfiles, char *filenames[], window_attributes_t *window_attrib
 				"current stream is audio only (eg: mp3)."), 
 			      CONFIG_LEVEL_BEG,
 			      visual_anim_cb,
-			      CONFIG_NO_DATA);
+			      gGui);
 
   gui->stream_info_auto_update = 
-    xine_config_register_bool(__xineui_global_xine_instance, "gui.sinfo_auto_update", 
+    xine_config_register_bool (gui->xine, "gui.sinfo_auto_update", 
 			      0,
 			      _("Stream information"),
 			      _("Update stream information (in stream info window) "
 				"each half seconds."), 
 			      CONFIG_LEVEL_ADV,
 			      stream_info_auto_update_cb,
-			      CONFIG_NO_DATA);
+			      gGui);
   
 
   server = 
-    (char *)xine_config_register_string (__xineui_global_xine_instance, "gui.skin_server_url", 
+    (char *)xine_config_register_string (gui->xine, "gui.skin_server_url", 
 					 SKIN_SERVER_URL,
 					 _("Skin Server Url"),
 					 _("From where we can get skins."),
 					 CONFIG_LEVEL_ADV,
 					 skin_server_url_cb,
-					 CONFIG_NO_DATA);
+					 gGui);
   
   config_update_string("gui.skin_server_url", 
 		       gui->skin_server_url ? gui->skin_server_url : server);
   
   gui->osd.enabled = 
-    xine_config_register_bool(__xineui_global_xine_instance, "gui.osd_enabled", 
+    xine_config_register_bool (gui->xine, "gui.osd_enabled", 
 			      1,
 			      _("Enable OSD support"),
 			      _("Enabling OSD permit to display some status/informations "
 				"in output window."), 
 			      CONFIG_LEVEL_BEG,
 			      osd_enabled_cb,
-			      CONFIG_NO_DATA);
+			      gGui);
 
   gui->osd.use_unscaled = 
-    xine_config_register_bool(__xineui_global_xine_instance, "gui.osd_use_unscaled", 
+    xine_config_register_bool (gui->xine, "gui.osd_use_unscaled", 
 			      1,
 			      _("Use unscaled OSD"),
 			      _("Use unscaled (full screen resolution) OSD if possible"), 
 			      CONFIG_LEVEL_ADV,
 			      osd_use_unscaled_cb,
-			      CONFIG_NO_DATA);
+			      gGui);
 
   gui->osd.timeout = 
-    xine_config_register_num(__xineui_global_xine_instance, "gui.osd_timeout", 
+    xine_config_register_num (gui->xine, "gui.osd_timeout", 
 			      3,
 			      _("Dismiss OSD time (s)"),
 			      _("Persistence time of OSD visual, in seconds."),
 			      CONFIG_LEVEL_BEG,
 			      osd_timeout_cb,
-			      CONFIG_NO_DATA);
+			      gGui);
 
   gui->smart_mode = 
-    xine_config_register_bool(__xineui_global_xine_instance, "gui.smart_mode", 
+    xine_config_register_bool (gui->xine, "gui.smart_mode", 
 			      1,
 			      _("Change xine's behavior for unexperienced user"), 
 			      _("In this mode, xine take some decisions to simplify user's life."),
 			      CONFIG_LEVEL_BEG,
 			      smart_mode_cb,
-			      CONFIG_NO_DATA);
+			      gGui);
 
   gui->play_anyway = 
-    xine_config_register_bool(__xineui_global_xine_instance, "gui.play_anyway", 
+    xine_config_register_bool (gui->xine, "gui.play_anyway", 
 			      0,
 			      _("Ask user for playback with unsupported codec"),
 			      _("If xine don't support audio or video codec of current stream "
 				"the user will be asked if the stream should be played or not"), 
 			      CONFIG_LEVEL_BEG,
 			      play_anyway_cb,
-			      CONFIG_NO_DATA);
+			      gGui);
 
   gui->experience_level =
-    (xine_config_register_enum(__xineui_global_xine_instance, "gui.experience_level", 
+    (xine_config_register_enum (gui->xine, "gui.experience_level", 
 			       0, (char**)exp_levels,
 			       _("Configuration experience level"),
 			       _("Level of user's experience, this will show more or less "
 				 "configuration options."), 
 			       CONFIG_LEVEL_BEG,
 			       exp_level_cb, 
-			       CONFIG_NO_DATA)) * 10;
+			       gGui)) * 10;
 
-  gui->mixer.amp_level = xine_config_register_range(__xineui_global_xine_instance, "gui.amp_level", 
+  gui->mixer.amp_level = xine_config_register_range (gui->xine, "gui.amp_level", 
 						     100, 0, 200,
 						     _("Amplification level"),
 						     NULL,
 						     CONFIG_LEVEL_DEB,
 						     dummy_config_cb, 
-						     CONFIG_NO_DATA);
+						     gGui);
 
   gui->splash = 
-    gui->splash ? (xine_config_register_bool(__xineui_global_xine_instance, "gui.splash", 
+    gui->splash ? (xine_config_register_bool (gui->xine, "gui.splash", 
 					      1,
 					      _("Display splash screen"),
 					      _("If enabled, xine will display its splash screen"), 
 					      CONFIG_LEVEL_BEG,
 					      dummy_config_cb,
-					      CONFIG_NO_DATA)) : 0;
+					      gGui)) : 0;
   
   gui->mixer.method = 
-    xine_config_register_enum(__xineui_global_xine_instance, "gui.audio_mixer_method", 
+    xine_config_register_enum (gui->xine, "gui.audio_mixer_method", 
 			      SOUND_CARD_MIXER, (char**)mixer_control_method,
 			      _("Audio mixer control method"),
 			      _("Which method used to control audio volume."), 
 			      CONFIG_LEVEL_ADV,
 			      audio_mixer_method_cb,
-			      CONFIG_NO_DATA);
+			      gGui);
 
   gui->shortcut_style = 
-    xine_config_register_enum(__xineui_global_xine_instance, "gui.shortcut_style", 
+    xine_config_register_enum (gui->xine, "gui.shortcut_style", 
 			      0, (char**)shortcut_style,
 			      _("Menu shortcut style"),
 			      _("Shortcut representation in menu, 'Ctrl,Alt' or 'C,M'."), 
 			      CONFIG_LEVEL_ADV,
 			      shortcut_style_cb,
-			      CONFIG_NO_DATA);
+			      gGui);
 
   gui->numeric.set = 0;
   gui->numeric.arg = 0;
   gui->alphanum.set = 0;
   gui->alphanum.arg = "";
   
-  XLockDisplay (gui->display);
+  gui->x_lock_display (gui->display);
 
   gui->screen = DefaultScreen(gui->display);
   
-  XLockDisplay (gui->video_display);
+  gui->x_lock_display (gui->video_display);
   gui->video_screen = DefaultScreen(gui->video_display);
-  XUnlockDisplay (gui->video_display);
+  gui->x_unlock_display (gui->video_display);
 
   
 
@@ -1913,16 +1916,16 @@ void gui_init (int nfiles, char *filenames[], window_attributes_t *window_attrib
 
   gui_find_visual(&gui->visual, &gui->depth);
 
-  gui_init_imlib (gui->visual);
+  gui_init_imlib (gui, gui->visual);
 
-  XUnlockDisplay (gui->display);
+  gui->x_unlock_display (gui->display);
 
   /*
    * create and map panel and video window
    */
   xine_pid = getppid();
   
-  xitk_init(gui->display, gui->black, (__xineui_global_verbosity) ? 1 : 0);
+  xitk_init (gui->display, gui->black, gui->x_lock_display, gui->x_unlock_display, (__xineui_global_verbosity) ? 1 : 0);
   
   preinit_skins_support();
   
@@ -1934,9 +1937,8 @@ void gui_init (int nfiles, char *filenames[], window_attributes_t *window_attrib
   gui->on_quit = 0;
   gui->running = 1;
   
-  video_window_init (window_attribute, 
-		     ((actions_on_start(gui->actions_on_start, 
-				       ACTID_TOGGLE_WINOUT_VISIBLITY)) ? 1 : 0));
+  video_window_init (gui, window_attribute,
+    ((actions_on_start(gui->actions_on_start, ACTID_TOGGLE_WINOUT_VISIBLITY)) ? 1 : 0));
 
   /* kbinding might open an error dialog (double keymapping), which produces a segfault,
    * when done before the video_window_init(). */
@@ -1944,12 +1946,11 @@ void gui_init (int nfiles, char *filenames[], window_attributes_t *window_attrib
 
   gui_set_current_mmk(mediamark_get_current_mmk());
 
-  panel_init();
+  panel_init (gui);
   gui->event_reject = 0;
 }
 
-void gui_init_imlib (Visual *vis) {
-  gGui_t *gui = gGui;
+void gui_init_imlib (gGui_t *gui, Visual *vis) {
   XColor                dummy;
   ImlibInitParams	imlib_init;
 
@@ -1982,6 +1983,9 @@ void gui_init_imlib (Visual *vis) {
     exit(1);
   }
 
+  gui->imlib_data->x.x_lock_display = gui->x_lock_display;
+  gui->imlib_data->x.x_unlock_display = gui->x_unlock_display;
+
   gui->colormap = Imlib_get_colormap (gui->imlib_data);
 
   XAllocNamedColor(gui->display, gui->colormap,
@@ -2030,7 +2034,7 @@ static void on_start(void *data) {
   }
   
   if(startup->start)
-    gui_execute_action_id(ACTID_PLAY);
+    gui_execute_action_id (gui, ACTID_PLAY);
   
 }
 
@@ -2039,20 +2043,20 @@ void gui_run(char **session_opts) {
   _startup_t  startup;
   int         i, auto_start = 0;
   
-  video_window_change_skins(0);
-  panel_add_autoplay_buttons();
-  panel_show_tips();
-  panel_add_mixer_control();
-  panel_update_channel_display () ;
-  panel_update_mrl_display ();
-  panel_update_runtime_display();
+  video_window_change_skins (gui->vwin, 0);
+  panel_add_autoplay_buttons (gui->panel);
+  panel_show_tips (gui->panel);
+  panel_add_mixer_control (gui->panel);
+  panel_update_channel_display (gui->panel);
+  panel_update_mrl_display (gui->panel);
+  panel_update_runtime_display (gui->panel);
 
   /* Register config entries related to video control settings */
-  control_config_register();
+  control_init (gui);
 
   /* autoscan playlist  */
   if(gui->autoscan_plugin != NULL) {
-    const char *const *autoscan_plugins = xine_get_autoplay_input_plugin_ids(__xineui_global_xine_instance);
+    const char *const *autoscan_plugins = xine_get_autoplay_input_plugin_ids (gui->xine);
     int         i;
 
     if(autoscan_plugins) {
@@ -2061,7 +2065,7 @@ void gui_run(char **session_opts) {
 
 	if(!strcasecmp(autoscan_plugins[i], gui->autoscan_plugin)) {
 	  int    num_mrls, j;
-	  const char * const *autoplay_mrls = xine_get_autoplay_mrls (__xineui_global_xine_instance,
+	  const char * const *autoplay_mrls = xine_get_autoplay_mrls (gui->xine,
 							 gui->autoscan_plugin,
 							 &num_mrls);
 	  
@@ -2078,7 +2082,7 @@ void gui_run(char **session_opts) {
     }
   }  
   
-  enable_playback_controls((gui->playlist.num > 0));
+  enable_playback_controls (gui->panel, (gui->playlist.num > 0));
 
   /* We can't handle signals here, xitk handle this, so
    * give a function callback for this.
@@ -2100,7 +2104,7 @@ void gui_run(char **session_opts) {
   /*  global event handler */
   gui->widget_key = xitk_register_event_handler("NO WINDOW", None,
 						 gui_handle_event, 
-						 NULL,
+						 gui,
 						 gui_dndcallback, 
 						 NULL, NULL);
   
@@ -2112,27 +2116,27 @@ void gui_run(char **session_opts) {
   if(gui->tvout) {
     int w, h;
     
-    video_window_get_visible_size(&w, &h);
-    tvout_set_fullscreen_mode(gui->tvout, 
-			      ((video_window_get_fullscreen_mode() & WINDOWED_MODE) ? 0 : 1), w, h);
+    video_window_get_visible_size (gui->vwin, &w, &h);
+    tvout_set_fullscreen_mode (gui->tvout, 
+      ((video_window_get_fullscreen_mode (gui->vwin) & WINDOWED_MODE) ? 0 : 1), w, h);
   }
   
   if(gui->actions_on_start[0] != ACTID_NOKEY) {
 
     /* Popup setup window if there is no config file */
     if(actions_on_start(gui->actions_on_start, ACTID_SETUP)) {
-      xine_config_save(__xineui_global_xine_instance, __xineui_global_config_file);
-      gui_execute_action_id(ACTID_SETUP);
+      xine_config_save (gui->xine, __xineui_global_config_file);
+      gui_execute_action_id (gui, ACTID_SETUP);
     }
     
     /*  The user wants to hide video window  */
     if(actions_on_start(gui->actions_on_start, ACTID_TOGGLE_WINOUT_VISIBLITY)) {
-      if(!panel_is_visible())
-	gui_execute_action_id(ACTID_TOGGLE_VISIBLITY);
+      if (!panel_is_visible (gui->panel))
+        gui_execute_action_id (gui, ACTID_TOGGLE_VISIBLITY);
       
       /* DXR3 case */
-      if(video_window_is_visible())
-	video_window_set_visibility(!(video_window_is_visible()));
+      if (video_window_is_visible (gui->vwin))
+        video_window_set_visibility (gui->vwin, 0);
       else
 	xine_port_send_gui_data(gui->vo_port,
 			      XINE_GUI_SEND_VIDEOWIN_VISIBLE,
@@ -2142,12 +2146,12 @@ void gui_run(char **session_opts) {
 
     /* The user wants to see in fullscreen mode  */
     for (i = actions_on_start(gui->actions_on_start, ACTID_TOGGLE_FULLSCREEN); i > 0; i--)
-      gui_execute_action_id(ACTID_TOGGLE_FULLSCREEN);
+      gui_execute_action_id (gui, ACTID_TOGGLE_FULLSCREEN);
 
 #ifdef HAVE_XINERAMA
     /* The user wants to see in xinerama fullscreen mode  */
     for (i = actions_on_start(gui->actions_on_start, ACTID_TOGGLE_XINERAMA_FULLSCREEN); i > 0; i--)
-      gui_execute_action_id(ACTID_TOGGLE_XINERAMA_FULLSCREEN);
+      gui_execute_action_id (gui, ACTID_TOGGLE_XINERAMA_FULLSCREEN);
 #endif
 
     /* User load a playlist on startup */
@@ -2156,8 +2160,8 @@ void gui_run(char **session_opts) {
 
       if(gui->playlist.num) {
 	gui->playlist.cur = 0;
-	if(!is_playback_widgets_enabled())
-	  enable_playback_controls(1);
+        if (!is_playback_widgets_enabled (gui->panel))
+          enable_playback_controls (gui->panel, 1);
       }
     }
 
