@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2017 the xine project
+ * Copyright (C) 2000-2019 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -79,21 +79,22 @@ typedef struct {
 static _eventer_t    *eventer = NULL;
 
 void event_sender_sticky_cb(void *data, xine_cfg_entry_t *cfg) {
-  int old_sticky_value = gGui->eventer_sticky;
+  gGui_t *gui = data;
+  int old_sticky_value = gui->eventer_sticky;
   
-  gGui->eventer_sticky = cfg->num_value;
+  gui->eventer_sticky = cfg->num_value;
   if(eventer) {
-    if((!old_sticky_value) && gGui->eventer_sticky) {
+    if((!old_sticky_value) && gui->eventer_sticky) {
       int  px, py, pw;
-      xitk_get_window_position(gGui->display, gGui->panel_window, &px, &py, &pw, NULL);
+      xitk_get_window_position (gui->display, gui->panel_window, &px, &py, &pw, NULL);
       eventer->x = px + pw;
       eventer->y = py;
-      xitk_window_move_window(gGui->imlib_data, eventer->xwin, eventer->x, eventer->y);
+      xitk_window_move_window (gui->imlib_data, eventer->xwin, eventer->x, eventer->y);
     }
   }
 }
 
-static void event_sender_store_new_position(int x, int y, int w, int h) {
+static void event_sender_store_new_position (void *data, int x, int y, int w, int h) {
 
   if(eventer && !gGui->eventer_sticky) {
     eventer->x = x;
@@ -157,7 +158,7 @@ static void event_sender_handle_event(XEvent *event, void *data) {
      * If we tried to move sticky window, move it back to stored position.
      */
     if(eventer && gGui->eventer_sticky) {
-      if(panel_is_visible()) {
+      if (panel_is_visible (gGui->panel)) {
 	int  x, y;
 	xitk_window_get_window_position(gGui->imlib_data, eventer->xwin, &x, &y, NULL, NULL);
 	if((x != eventer->x) || (y != eventer->y))
@@ -174,9 +175,9 @@ static void event_sender_handle_event(XEvent *event, void *data) {
 
       mykeyevent = event->xkey;
       
-      XLockDisplay(gGui->display);
+      gGui->x_lock_display (gGui->display);
       XLookupString(&mykeyevent, kbuf, sizeof(kbuf), &key, NULL);
-      XUnlockDisplay(gGui->display);
+      gGui->x_unlock_display (gGui->display);
 
       switch(key) {
       case XK_Up:
@@ -239,7 +240,7 @@ static void event_sender_handle_event(XEvent *event, void *data) {
 	return;
 
       default:
-	gui_handle_event(event, data);
+        gui_handle_event (event, gGui);
 	return;
       }
     }
@@ -308,11 +309,11 @@ static void event_sender_exit(xitk_widget_t *w, void *data) {
     xitk_window_destroy_window(gGui->imlib_data, eventer->xwin);
     
     eventer->xwin = NULL;
-    xitk_list_free((XITK_WIDGET_LIST_LIST(eventer->widget_list)));
+    /* xitk_dlist_init (&eventer->widget_list->list); */
     
-    XLockDisplay(gGui->display);
+    gGui->x_lock_display (gGui->display);
     XFreeGC(gGui->display, (XITK_WIDGET_LIST_GC(eventer->widget_list)));
-    XUnlockDisplay(gGui->display);
+    gGui->x_unlock_display (gGui->display);
     
     XITK_WIDGET_LIST_FREE(eventer->widget_list);
     
@@ -399,7 +400,7 @@ void event_sender_panel(void) {
 					 CONFIG_NO_CB,
 					 CONFIG_NO_DATA);
   
-  if(gGui->eventer_sticky && panel_is_visible()) {
+  if (gGui->eventer_sticky && panel_is_visible (gGui->panel)) {
     int  px, py, pw;
     xitk_get_window_position(gGui->display, gGui->panel_window, &px, &py, &pw, NULL);
     eventer->x = px + pw;
@@ -411,14 +412,13 @@ void event_sender_panel(void) {
 
   set_window_states_start((xitk_window_get_window(eventer->xwin)));
 
-  XLockDisplay (gGui->display);
+  gGui->x_lock_display (gGui->display);
   XStoreName(gGui->display, (xitk_window_get_window(eventer->xwin)), _("xine Event Sender"));
   gc = XCreateGC(gGui->display, 
 		 (xitk_window_get_window(eventer->xwin)), None, None);
-  XUnlockDisplay (gGui->display);
+  gGui->x_unlock_display (gGui->display);
   
   eventer->widget_list = xitk_widget_list_new();
-  xitk_widget_list_set(eventer->widget_list, WIDGET_LIST_LIST, (xitk_list_new()));
   xitk_widget_list_set(eventer->widget_list, 
 		       WIDGET_LIST_WINDOW, (void *) (xitk_window_get_window(eventer->xwin)));
   xitk_widget_list_set(eventer->widget_list, WIDGET_LIST_GC, gc);
@@ -438,12 +438,10 @@ void event_sender_panel(void) {
 
   lb.label             = _("Up");
   lb.userdata          = I2PTR (XINE_EVENT_INPUT_UP);
-  xitk_list_append_content((XITK_WIDGET_LIST_LIST(eventer->widget_list)),
-	   (eventer->navigator.up = 
-	    xitk_noskin_labelbutton_create(eventer->widget_list, 
-					   &lb, x, y, 70, 40,
-					   "Black", "Black", "White", 
-					   hboldfontname)));
+  eventer->navigator.up = xitk_noskin_labelbutton_create (eventer->widget_list,
+    &lb, x, y, 70, 40, "Black", "Black", "White", hboldfontname);
+  xitk_add_widget (eventer->widget_list, eventer->navigator.up);
+
   xitk_enable_and_show_widget(eventer->navigator.up);
 
   x -= 70;
@@ -451,12 +449,10 @@ void event_sender_panel(void) {
 
   lb.label             = _("Left");
   lb.userdata          = I2PTR (XINE_EVENT_INPUT_LEFT);
-  xitk_list_append_content((XITK_WIDGET_LIST_LIST(eventer->widget_list)), 
-	   (eventer->navigator.left = 
-	    xitk_noskin_labelbutton_create(eventer->widget_list, 
-					   &lb, x, y, 70, 40,
-					   "Black", "Black", "White", 
-					   hboldfontname)));
+  eventer->navigator.left = xitk_noskin_labelbutton_create (eventer->widget_list,
+    &lb, x, y, 70, 40, "Black", "Black", "White", hboldfontname);
+  xitk_add_widget (eventer->widget_list, eventer->navigator.left);
+
   xitk_enable_and_show_widget(eventer->navigator.left);
 
   x += 75;
@@ -464,12 +460,10 @@ void event_sender_panel(void) {
 
   lb.label             = _("Select");
   lb.userdata          = I2PTR (XINE_EVENT_INPUT_SELECT);
-  xitk_list_append_content((XITK_WIDGET_LIST_LIST(eventer->widget_list)), 
-	   (eventer->navigator.select = 
-	    xitk_noskin_labelbutton_create(eventer->widget_list, 
-					   &lb, x, y, 60, 30,
-					   "Black", "Black", "White", 
-					   hboldfontname)));
+  eventer->navigator.select = xitk_noskin_labelbutton_create (eventer->widget_list,
+    &lb, x, y, 60, 30, "Black", "Black", "White", hboldfontname);
+  xitk_add_widget (eventer->widget_list, eventer->navigator.select);
+
   xitk_enable_and_show_widget(eventer->navigator.select);
 
   x += 65;
@@ -477,12 +471,10 @@ void event_sender_panel(void) {
 
   lb.label             = _("Right");
   lb.userdata          = I2PTR (XINE_EVENT_INPUT_RIGHT);
-  xitk_list_append_content((XITK_WIDGET_LIST_LIST(eventer->widget_list)), 
-	   (eventer->navigator.right = 
-	    xitk_noskin_labelbutton_create(eventer->widget_list, 
-					   &lb, x, y, 70, 40,
-					   "Black", "Black", "White", 
-					   hboldfontname)));
+  eventer->navigator.right = xitk_noskin_labelbutton_create (eventer->widget_list,
+    &lb, x, y, 70, 40, "Black", "Black", "White", hboldfontname);
+  xitk_add_widget (eventer->widget_list, eventer->navigator.right);
+
   xitk_enable_and_show_widget(eventer->navigator.right);
 
   x -= 70;
@@ -490,19 +482,17 @@ void event_sender_panel(void) {
  
   lb.label             = _("Down");
   lb.userdata          = I2PTR (XINE_EVENT_INPUT_DOWN);
-  xitk_list_append_content((XITK_WIDGET_LIST_LIST(eventer->widget_list)), 
-	   (eventer->navigator.down = 
-	    xitk_noskin_labelbutton_create(eventer->widget_list, 
-					   &lb, x, y, 70, 40,
-					   "Black", "Black", "White", 
-					   hboldfontname)));
+  eventer->navigator.down = xitk_noskin_labelbutton_create (eventer->widget_list,
+    &lb, x, y, 70, 40, "Black", "Black", "White", hboldfontname);
+  xitk_add_widget (eventer->widget_list, eventer->navigator.down);
+
   xitk_enable_and_show_widget(eventer->navigator.down);
 
   x = 23 * 2 + 5 + (80 - 23 * 3) / 2 + 1; /* (+1 to round up) */
   y = 5 + 23 * 3 + 5 + 40 + 5;
 
   for(i = 9; i >= 0; i--) {
-    char number[2];
+    char number[3];
     
     snprintf(number, sizeof(number), "%d", i);
 
@@ -512,11 +502,10 @@ void event_sender_panel(void) {
     if(!i)
       x -= 46;
 
-    xitk_list_append_content((XITK_WIDGET_LIST_LIST(eventer->widget_list)), 
-	     (eventer->numbers.number[i] = 
-	      xitk_noskin_labelbutton_create(eventer->widget_list, 
-					     &lb, x, y, 23, 23,
-					     "Black", "Black", "White", hboldfontname)));
+    eventer->numbers.number[i] = xitk_noskin_labelbutton_create (eventer->widget_list,
+      &lb, x, y, 23, 23, "Black", "Black", "White", hboldfontname);
+    xitk_add_widget (eventer->widget_list, eventer->numbers.number[i]);
+
     xitk_enable_and_show_widget(eventer->numbers.number[i]);
   
     if(!((i - 1) % 3)) {
@@ -540,11 +529,10 @@ void event_sender_panel(void) {
     lb.label             = number;
     lb.userdata          = I2PTR (XINE_EVENT_INPUT_NUMBER_10_ADD);
     
-    xitk_list_append_content((XITK_WIDGET_LIST_LIST(eventer->widget_list)), 
-	     (eventer->numbers.number[i] = 
-	      xitk_noskin_labelbutton_create(eventer->widget_list, 
-					     &lb, x, y, 46, 23,
-					     "Black", "Black", "White", hboldfontname)));
+    eventer->numbers.number[i] = xitk_noskin_labelbutton_create (eventer->widget_list,
+      &lb, x, y, 46, 23, "Black", "Black", "White", hboldfontname);
+    xitk_add_widget (eventer->widget_list, eventer->numbers.number[i]);
+
     xitk_enable_and_show_widget(eventer->numbers.number[i]);
   }
 
@@ -553,22 +541,20 @@ void event_sender_panel(void) {
 
   lb.label             = _("Angle +");
   lb.userdata          = I2PTR (XINE_EVENT_INPUT_ANGLE_NEXT);
-  xitk_list_append_content((XITK_WIDGET_LIST_LIST(eventer->widget_list)), 
-	   (eventer->angles.next = 
-	    xitk_noskin_labelbutton_create(eventer->widget_list, 
-					   &lb, x, y, 80, 23,
-					   "Black", "Black", "White", hboldfontname)));
+  eventer->angles.next = xitk_noskin_labelbutton_create (eventer->widget_list,
+    &lb, x, y, 80, 23, "Black", "Black", "White", hboldfontname);
+  xitk_add_widget (eventer->widget_list, eventer->angles.next);
+
   xitk_enable_and_show_widget(eventer->angles.next);
 
   y += 23;
 
   lb.label             = _("Angle -");
   lb.userdata          = I2PTR (XINE_EVENT_INPUT_ANGLE_PREVIOUS);
-  xitk_list_append_content((XITK_WIDGET_LIST_LIST(eventer->widget_list)), 
-	   (eventer->angles.prev = 
-	    xitk_noskin_labelbutton_create(eventer->widget_list, 
-					   &lb, x, y, 80, 23,
-					   "Black", "Black", "White", hboldfontname)));
+  eventer->angles.prev = xitk_noskin_labelbutton_create (eventer->widget_list,
+    &lb, x, y, 80, 23, "Black", "Black", "White", hboldfontname);
+  xitk_add_widget (eventer->widget_list, eventer->angles.prev);
+
   xitk_enable_and_show_widget(eventer->angles.prev);
 
 
@@ -578,11 +564,10 @@ void event_sender_panel(void) {
   for(i = 0; i < 7; i++) {
     lb.label             = "";
     lb.userdata          = I2PTR (XINE_EVENT_INPUT_MENU1 + i);
-    xitk_list_append_content((XITK_WIDGET_LIST_LIST(eventer->widget_list)), 
-	     (eventer->menus.menu[i] = 
-	      xitk_noskin_labelbutton_create(eventer->widget_list, 
-					     &lb, x, y, 80, 23,
-					     "Black", "Black", "White", hboldfontname)));
+    eventer->menus.menu[i] = xitk_noskin_labelbutton_create (eventer->widget_list,
+      &lb, x, y, 80, 23, "Black", "Black", "White", hboldfontname);
+    xitk_add_widget (eventer->widget_list, eventer->menus.menu[i]);
+
     xitk_enable_and_show_widget(eventer->menus.menu[i]);
 
     if(i == 2) {
@@ -604,12 +589,12 @@ void event_sender_panel(void) {
   lb.label             = _("Close");
   lb.callback          = event_sender_exit;
   lb.userdata          = NULL;
-  xitk_list_append_content((XITK_WIDGET_LIST_LIST(eventer->widget_list)), 
-   (w = xitk_noskin_labelbutton_create(eventer->widget_list, 
-				       &lb, x, y, 70, 23,
-				       "Black", "Black", "White", hboldfontname)));
+  w = xitk_noskin_labelbutton_create (eventer->widget_list,
+    &lb, x, y, 70, 23, "Black", "Black", "White", hboldfontname);
+  xitk_add_widget (eventer->widget_list, w);
+
   xitk_enable_and_show_widget(w);
-  event_sender_show_tips(panel_get_tips_enable(), panel_get_tips_timeout());
+  event_sender_show_tips (panel_get_tips_enable (gGui->panel), panel_get_tips_timeout (gGui->panel));
 
   eventer->widget_key = xitk_register_event_handler("eventer", 
 						    (xitk_window_get_window(eventer->xwin)),
