@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2017 the xine project
+ * Copyright (C) 2000-2019 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -25,7 +25,15 @@
 /* 
  * Default key mapping table.
  */
-static const kbinding_entry_t default_binding_table[] = {
+static const struct {
+  const char       *comment;     /* Comment automatically added in xbinding_display*() outputs */
+  const char       *action;      /* Human readable action, used in config file too */
+  action_id_t       action_id;   /* The numerical action, handled in a case statement */
+  const char        key[9];      /* key binding */
+  uint8_t           modifier;    /* Modifier key of binding (can be OR'ed) */
+  uint8_t           is_alias : 1;/* is made from an alias entry ? */
+  uint8_t           is_gui : 1;
+}  default_binding_table[] = {
   { "start playback",
     "Play",                   ACTID_PLAY                    , "Return",   KEYMOD_NOMOD   , 0 , 0},
   { "playback pause toggle",
@@ -410,11 +418,11 @@ static const kbinding_entry_t default_binding_table[] = {
 #endif
 #endif
   { 0,
-    0,                        0                              , 0,          0              , 0 , 0}
+    0,                        0                              , "",         0              , 0 , 0}
 };
 
 
-static int _kbinding_get_is_gui_from_default(char *action) {
+static int _kbinding_get_is_gui_from_default(const char *action) {
   int i;
   
   for(i = 0; default_binding_table[i].action != NULL; i++) {
@@ -887,11 +895,11 @@ static void _kbindings_check_redundancy(kbinding_t *kbt) {
 						     _kbinding_reset_cb, _kbinding_editor_cb, NULL,
 						     (void *) kbt, 450, ALIGN_CENTER,
 						     "%s", kmsg);
-    XLockDisplay(gui->display);
+    gui->x_lock_display (gui->display);
     if(!gui->use_root_window && gui->video_display == gui->display)
       XSetTransientForHint(gui->display, xitk_window_get_window(xw), gui->video_window);
     XSync(gui->display, False);
-    XUnlockDisplay(gui->display);
+    gui->x_unlock_display (gui->display);
     layer_above_video(xitk_window_get_window(xw));
   }
 #endif
