@@ -1,5 +1,4 @@
-/*
- * Copyright (C) 2000-2017 the xine project
+/** Copyright (C) 2000-2019 the xine project
  *
  * This file is part of xine, a unix video player.
  * 
@@ -70,12 +69,7 @@
 /*
  * global variables
  */
-gGui_t                       *gGui;
-extern _panel_t              *panel;
-
-static char                 **video_driver_ids;
-static char                 **audio_driver_ids;
-static window_attributes_t    window_attribute;
+gGui_t *gGui;
 
 #define CONFIGFILE "config"
 #define CONFIGDIR  ".xine"
@@ -389,11 +383,11 @@ static void xrm_parse(void) {
   (void) XrmMergeDatabases(home_rmdb, &rmdb);
 
   if(XrmGetResource(rmdb, "xine.geometry", "xine.Geometry", &str_type, &value) == True) {
-    if(!parse_geometry(&window_attribute, (char *)value.addr))
+    if (!parse_geometry (&gui->window_attribute, (char *)value.addr))
       printf(_("Bad geometry '%s'\n"), (char *)value.addr);
   } 
   if(XrmGetResource(rmdb, "xine.border", "xine.Border", &str_type, &value) == True) {
-    window_attribute.borderless = !get_bool_value((char *) value.addr);
+    gui->window_attribute.borderless = !get_bool_value((char *) value.addr);
   }
   if(XrmGetResource(rmdb, "xine.visual", "xine.Visual", &str_type, &value) == True) {
     if(!parse_visual(&gui->prefered_visual_id, 
@@ -413,11 +407,11 @@ static void xrm_parse(void) {
 }
 
 static void main_change_logo_cb(void *data, xine_cfg_entry_t *cfg) {
-  gGui_t *gui = gGui;
+  gGui_t *gui = data;
   gui->logo_mrl = cfg->str_value;
 }
 static void sub_autoload_cb(void *data, xine_cfg_entry_t *cfg) {
-  gGui_t *gui = gGui;
+  gGui_t *gui = data;
   gui->subtitle_autoload = cfg->num_value;
 }
 
@@ -430,7 +424,7 @@ static void show_version(void) {
 #ifdef DEBUG
   printf("-[DEBUG]");
 #endif
-  printf(_(".\n(c) 2000-2014 The xine Team.\n"));
+  printf(_(".\n(c) %s The xine Team.\n"), "2000-2019");
 }
 
 /*
@@ -516,6 +510,7 @@ static void list_plugins(char *type) {
   
   xine = xine_new();
   xine_config_load(xine, configfile);
+  free(configfile);
   xine_init(xine);
 
   show_version();
@@ -580,6 +575,7 @@ static void show_usage (void) {
   
   xine = xine_new();
   xine_config_load(xine, configfile);
+  free(configfile);
   xine_init(xine);
   
   printf("\n");
@@ -751,12 +747,12 @@ static xine_video_port_t *load_video_out_driver(int driver_number) {
   vis.display           = gui->video_display;
   vis.screen            = gui->video_screen;
   vis.d                 = gui->video_window;
-  XLockDisplay(gui->video_display);
+  gui->x_lock_display (gui->video_display);
   res_h                 = (DisplayWidth  (gui->video_display, gui->video_screen)*1000 
 			   / DisplayWidthMM (gui->video_display, gui->video_screen));
   res_v                 = (DisplayHeight (gui->video_display, gui->video_screen)*1000
 			   / DisplayHeightMM (gui->video_display, gui->video_screen));
-  XUnlockDisplay(gui->video_display);
+  gui->x_unlock_display (gui->video_display);
   gui->pixel_aspect    = res_v / res_h;
   
 #ifdef HAVE_XINERAMA
@@ -778,14 +774,13 @@ static xine_video_port_t *load_video_out_driver(int driver_number) {
 
   vis.dest_size_cb      = video_window_dest_size_cb;
   vis.frame_output_cb   = video_window_frame_output_cb;
-  vis.user_data         = NULL;
+  vis.user_data         = gui->vwin;
   
   /*
    * Setting default (configfile stuff need registering before updating, etc...).
    */
   driver_num = 
-    xine_config_register_enum(__xineui_global_xine_instance, "video.driver", 
-			      0, video_driver_ids,
+    xine_config_register_enum (gui->xine, "video.driver", 0, gui->video_driver_ids,
 			      _("video driver to use"),
 			      _("Choose video driver. "
 				"NOTE: you may restart xine to use the new driver"),
@@ -798,30 +793,24 @@ static xine_video_port_t *load_video_out_driver(int driver_number) {
     const char *const *driver_ids;
     int                i;
     
-    if((!strcasecmp(video_driver_ids[driver_num], "none")) || 
-       (!strcasecmp(video_driver_ids[driver_num], "null"))) {
-      video_port = xine_open_video_driver(__xineui_global_xine_instance,
-					  video_driver_ids[driver_num],
-					  XINE_VISUAL_TYPE_NONE,
-					  (void *) &vis);
+    if ((!strcasecmp (gui->video_driver_ids[driver_num], "none")) ||
+        (!strcasecmp (gui->video_driver_ids[driver_num], "null"))) {
+      video_port = xine_open_video_driver (gui->xine, gui->video_driver_ids[driver_num],
+        XINE_VISUAL_TYPE_NONE, (void *) &vis);
       if (video_port)
 	return video_port;
       
     }
-    else if(!strcasecmp(video_driver_ids[driver_num], "fb")) {
-      video_port = xine_open_video_driver(__xineui_global_xine_instance,
-					  video_driver_ids[driver_num],
-					  XINE_VISUAL_TYPE_FB, NULL);
+    else if (!strcasecmp (gui->video_driver_ids[driver_num], "fb")) {
+      video_port = xine_open_video_driver (gui->xine, gui->video_driver_ids[driver_num],
+        XINE_VISUAL_TYPE_FB, NULL);
       if (video_port)
 	return video_port;
       
     }
-    else if(strcasecmp(video_driver_ids[driver_num], "auto")) {
-      
-      video_port = xine_open_video_driver(__xineui_global_xine_instance, 
-					  video_driver_ids[driver_num],
-					  XINE_VISUAL_TYPE_X11,
-					  (void *) &vis);
+    else if (strcasecmp (gui->video_driver_ids[driver_num], "auto")) {
+      video_port = xine_open_video_driver (gui->xine, gui->video_driver_ids[driver_num],
+        XINE_VISUAL_TYPE_X11, (void *) &vis);
       if (video_port)
 	return video_port;
     }
@@ -830,14 +819,14 @@ static xine_video_port_t *load_video_out_driver(int driver_number) {
      *       but doing it here should do no harm.
      */
     i = 0;
-    driver_ids = xine_list_video_output_plugins (__xineui_global_xine_instance);
+    driver_ids = xine_list_video_output_plugins (gui->xine);
 
     while (driver_ids[i]) {
       
       if(__xineui_global_verbosity)
 	printf (_("main: probing <%s> video output plugin\n"), driver_ids[i]);
       
-      video_port = xine_open_video_driver(__xineui_global_xine_instance, 
+      video_port = xine_open_video_driver (gui->xine, 
 					  driver_ids[i],
 					  XINE_VISUAL_TYPE_X11, 
 					  (void *) &vis);
@@ -857,30 +846,25 @@ static xine_video_port_t *load_video_out_driver(int driver_number) {
   else {
     
     /* 'none' plugin is a special case, just change the visual type */
-    if((!strcasecmp(video_driver_ids[driver_number], "none")) 
-       || (!strcasecmp(video_driver_ids[driver_number], "null"))) {
-      video_port = xine_open_video_driver(__xineui_global_xine_instance,
-					  video_driver_ids[driver_number],
-					  XINE_VISUAL_TYPE_NONE,
-					  (void *) &vis);
+    if ((!strcasecmp (gui->video_driver_ids[driver_number], "none"))
+      || (!strcasecmp (gui->video_driver_ids[driver_number], "null"))) {
+      video_port = xine_open_video_driver (gui->xine, gui->video_driver_ids[driver_number],
+        XINE_VISUAL_TYPE_NONE, (void *) &vis);
       
       /* do not save on config, otherwise user would never see images again... */
     }
-    else if(!strcasecmp(video_driver_ids[driver_number], "fb")) {
-      video_port = xine_open_video_driver(__xineui_global_xine_instance,
-					  video_driver_ids[driver_number],
-					  XINE_VISUAL_TYPE_FB, NULL);
+    else if (!strcasecmp (gui->video_driver_ids[driver_number], "fb")) {
+      video_port = xine_open_video_driver (gui->xine, gui->video_driver_ids[driver_number],
+        XINE_VISUAL_TYPE_FB, NULL);
 
     }
     else {
-      video_port = xine_open_video_driver(__xineui_global_xine_instance,
-					  video_driver_ids[driver_number],
-					  XINE_VISUAL_TYPE_X11, 
-					  (void *) &vis);
+      video_port = xine_open_video_driver (gui->xine, gui->video_driver_ids[driver_number],
+        XINE_VISUAL_TYPE_X11, (void *) &vis);
     }
     
     if(!video_port) {
-      printf (_("main: video driver <%s> failed\n"), video_driver_ids[driver_number]);
+      printf (_("main: video driver <%s> failed\n"), gui->video_driver_ids[driver_number]);
       exit (1);
     }
     
@@ -893,6 +877,7 @@ static xine_video_port_t *load_video_out_driver(int driver_number) {
  * Try to load audio output plugin, by stored name or probing
  */
 static xine_audio_port_t *load_audio_out_driver(int driver_number) {
+  gGui_t                 *gui = gGui;
   xine_audio_port_t      *audio_port = NULL;
   int                     driver_num;
   
@@ -900,8 +885,7 @@ static xine_audio_port_t *load_audio_out_driver(int driver_number) {
    * Setting default (configfile stuff need registering before updating, etc...).
    */
   driver_num = 
-    xine_config_register_enum(__xineui_global_xine_instance, "video.driver", 
-			      0, video_driver_ids,
+    xine_config_register_enum (gui->xine, "video.driver", 0, gui->video_driver_ids,
 			      _("video driver to use"),
 			      _("Choose video driver. "
 				"NOTE: you may restart xine to use the new driver"),
@@ -911,8 +895,7 @@ static xine_audio_port_t *load_audio_out_driver(int driver_number) {
   
 
   driver_num = 
-    xine_config_register_enum(__xineui_global_xine_instance, "audio.driver", 
-			      0, audio_driver_ids,
+    xine_config_register_enum (gui->xine, "audio.driver", 0, gui->audio_driver_ids,
 			      _("audio driver to use"),
 			      _("Choose audio driver. "
 				"NOTE: you may restart xine to use the new driver"),
@@ -924,18 +907,16 @@ static xine_audio_port_t *load_audio_out_driver(int driver_number) {
     const char *const *driver_ids;
     int    i;
     
-    if (strcasecmp(audio_driver_ids[driver_num], "auto")) {
+    if (strcasecmp (gui->audio_driver_ids[driver_num], "auto")) {
       
       /* don't want to load an audio driver ? */
-      if (!strncasecmp(audio_driver_ids[driver_num], "NULL", 4)) {
+      if (!strncasecmp (gui->audio_driver_ids[driver_num], "NULL", 4)) {
 	if(__xineui_global_verbosity)
 	  printf(_("main: not using any audio driver (as requested).\n"));
         return NULL;
       }
       
-      audio_port = xine_open_audio_driver(__xineui_global_xine_instance, 
-					  audio_driver_ids[driver_num],
-					  NULL);
+      audio_port = xine_open_audio_driver (gui->xine, gui->audio_driver_ids[driver_num], NULL);
       if (audio_port)
 	return audio_port;
     }
@@ -944,7 +925,7 @@ static xine_audio_port_t *load_audio_out_driver(int driver_number) {
      *       but doing it here should do no harm.
      */
     i = 0;
-    driver_ids = xine_list_audio_output_plugins (__xineui_global_xine_instance);
+    driver_ids = xine_list_audio_output_plugins (gui->xine);
 
     while (driver_ids[i]) {
       
@@ -959,7 +940,7 @@ static xine_audio_port_t *load_audio_out_driver(int driver_number) {
         continue;
       }
       
-      audio_port = xine_open_audio_driver(__xineui_global_xine_instance, 
+      audio_port = xine_open_audio_driver (gui->xine, 
 					  driver_ids[i],
 					  NULL);
       if (audio_port) {
@@ -974,7 +955,7 @@ static xine_audio_port_t *load_audio_out_driver(int driver_number) {
   else {
     
     /* don't want to load an audio driver ? */
-    if (!strncasecmp (audio_driver_ids[driver_number], "NULL", 4)) {
+    if (!strncasecmp (gui->audio_driver_ids[driver_number], "NULL", 4)) {
 
       if(__xineui_global_verbosity)
 	printf(_("main: not using any audio driver (as requested).\n"));
@@ -987,10 +968,10 @@ static xine_audio_port_t *load_audio_out_driver(int driver_number) {
     }
     else {
     
-      audio_port = xine_open_audio_driver(__xineui_global_xine_instance, audio_driver_ids[driver_number], NULL);
+      audio_port = xine_open_audio_driver (gui->xine, gui->audio_driver_ids[driver_number], NULL);
 
       if (!audio_port) {
-        printf (_("main: audio driver <%s> failed\n"), audio_driver_ids[driver_number]);
+        printf (_("main: audio driver <%s> failed\n"), gui->audio_driver_ids[driver_number]);
         exit(1);
       }
     
@@ -1004,8 +985,8 @@ static xine_audio_port_t *load_audio_out_driver(int driver_number) {
 /*
  *
  */
-static void event_listener(void *user_data, const xine_event_t *event) {
-  gGui_t *gui = gGui;
+static void event_listener (void *user_data, const xine_event_t *event) {
+  gGui_t *gui = user_data;
   struct timeval tv;
   static int mrl_ext = 0; /* set if we get an MRL_REFERENCE_EXT */ 
 
@@ -1035,7 +1016,7 @@ static void event_listener(void *user_data, const xine_event_t *event) {
       if( xine_check_version(1,1,1) )
         xine_set_param(gui->stream, XINE_PARAM_GAPLESS_SWITCH, 1);
 #endif
-      gui_playlist_start_next();
+      gui_playlist_start_next (gui);
     }
     else if(event->stream == gui->visual_anim.stream) {
       /* printf("xitk/main.c: restarting visual stream...\n"); */
@@ -1046,7 +1027,7 @@ static void event_listener(void *user_data, const xine_event_t *event) {
     /* inform ui that new channel info is available */
   case XINE_EVENT_UI_CHANNELS_CHANGED:
     if(event->stream == gui->stream)
-      panel_update_channel_display();
+      panel_update_channel_display (gui->panel);
     break;
     
     /* request title display change in ui */
@@ -1054,6 +1035,7 @@ static void event_listener(void *user_data, const xine_event_t *event) {
     if(event->stream == gui->stream) {
       xine_ui_data_t *uevent = (xine_ui_data_t *) event->data;
       
+      pthread_mutex_lock (&gui->mmk_mutex);
       if(strcmp(gui->mmk.ident, uevent->str)) {
 	
 	free(gui->mmk.ident);
@@ -1065,10 +1047,13 @@ static void event_listener(void *user_data, const xine_event_t *event) {
 	  gui->playlist.mmk[gui->playlist.cur]->ident = strdup(uevent->str);
 	}
 	gui->mmk.ident = strdup(uevent->str);
+        pthread_mutex_unlock (&gui->mmk_mutex);
 	
-	video_window_set_mrl(gui->mmk.ident);
+        video_window_set_mrl (gui->vwin, uevent->str);
 	playlist_mrlident_toggle();
-	panel_update_mrl_display();
+        panel_update_mrl_display (gui->panel);
+      } else {
+        pthread_mutex_unlock (&gui->mmk_mutex);
       }
     }
     break;
@@ -1189,12 +1174,17 @@ static void event_listener(void *user_data, const xine_event_t *event) {
 	/* none */
       case XINE_MSG_ENCRYPTED_SOURCE:
 	strlcpy(buffer, _("The source seems encrypted, and can't be read."), sizeof(buffer));
-	if(!strncasecmp(gui->mmk.mrl, "dvd:/", 5)) {
-	  strlcat(buffer, _("\nYour DVD is probably crypted. "
-			   "According to your country laws, you can or can't "
-			   "install/use libdvdcss to be able to read this disc, "
-			   "which you bought."), sizeof(buffer));
-	}
+        {
+          int i;
+          pthread_mutex_lock (&gui->mmk_mutex);
+          i = strncasecmp (gui->mmk.mrl, "dvd:/", 5);
+          pthread_mutex_unlock (&gui->mmk_mutex);
+          if (!i) {
+            strlcat (buffer,
+              _("\nYour DVD is probably crypted. According to your country laws, you can or can't "
+                "install/use libdvdcss to be able to read this disc, which you bought."), sizeof(buffer));
+          }
+        }
 	if(data->explanation)
 	  snprintf(buffer+strlen(buffer), sizeof(buffer)-strlen(buffer), " (%s)", (char *) data + data->parameters);
 	break;
@@ -1209,7 +1199,7 @@ static void event_listener(void *user_data, const xine_event_t *event) {
 	break;
 
       case XINE_MSG_AUDIO_OUT_UNAVAILABLE:
-	gui_stop(NULL, NULL);
+        gui_stop (NULL, gui);
 	strlcpy(buffer, _("The audio device is unavailable. "
 			  "Please verify if another program already uses it."), sizeof(buffer));
 	break;
@@ -1247,13 +1237,18 @@ static void event_listener(void *user_data, const xine_event_t *event) {
 	strlcat(buffer, "]", sizeof(buffer));
       }
       
-      if(strlen(buffer)) {
-	if(gui->suppress_messages) {
-	  if(__xineui_global_verbosity)
-	    printf("xine-ui: GUI temporarily unavailable. Message:\n%s\n", buffer);
-	} else {
-	  report("%s", buffer);
-	}
+      if (buffer[0]) {
+        struct timeval tv;
+        pthread_mutex_lock (&gui->no_messages.mutex);
+        tv = gui->no_messages.until;
+        pthread_mutex_unlock (&gui->no_messages.mutex);
+        if ((event->tv.tv_sec < tv.tv_sec)
+          || ((event->tv.tv_sec == tv.tv_sec) && (event->tv.tv_usec < tv.tv_usec))) {
+          if (__xineui_global_verbosity >= XINE_VERBOSITY_DEBUG)
+            printf ("xine-ui: suppressed message:\n%s\n", buffer);
+        } else {
+          report ("%s", buffer);
+        }
       }
       
     }
@@ -1271,10 +1266,7 @@ static void event_listener(void *user_data, const xine_event_t *event) {
       gui->mixer.volume_level = (aevent->left + aevent->right) / 2;
       if(gui->mixer.method == SOUND_CARD_MIXER) {
 	gui->mixer.mute = aevent->mute;
-	if (panel) {
-	  xitk_slider_set_pos(panel->mixer.slider, gui->mixer.volume_level);
-	  xitk_checkbox_set_state(panel->mixer.mute, gui->mixer.mute);
-	}
+        panel_update_mixer_display (gui->panel);
       }
     }
     break;
@@ -1288,10 +1280,7 @@ static void event_listener(void *user_data, const xine_event_t *event) {
       gui->mixer.amp_level = (aevent->left + aevent->right) / 2;
       if(gui->mixer.method == SOFTWARE_MIXER) {
 	gui->mixer.mute = aevent->mute;
-	if (panel) {
-	  xitk_slider_set_pos(panel->mixer.slider, gui->mixer.amp_level);
-	  xitk_checkbox_set_state(panel->mixer.mute, gui->mixer.mute);
-        }
+        panel_update_mixer_display (gui->panel);
       }
     }
     break;
@@ -1309,7 +1298,7 @@ static void event_listener(void *user_data, const xine_event_t *event) {
       
       snprintf(buffer, sizeof(buffer), "%s [%d%%]\n", pevent->description, pevent->percent);
       gui->mrl_overrided = 3;
-      panel_set_title(buffer);
+      panel_set_title (gui->panel, buffer);
       osd_display_info("%s", buffer);
     }
     break;
@@ -1326,12 +1315,14 @@ static void event_listener(void *user_data, const xine_event_t *event) {
         gui->playlist.ref_append++;
         mediamark_insert_entry(gui->playlist.ref_append, ref->mrl, ref->mrl, NULL, 0, -1, 0, 0);
       } else {
+        pthread_mutex_lock (&gui->mmk_mutex);
 	mediamark_t *mmk = mediamark_get_mmk_by_index(gui->playlist.ref_append);
 	
 	if(mmk) {
 	  mediamark_append_alternate_mrl(mmk, ref->mrl);
 	  mediamark_set_got_alternate(mmk);
 	}
+        pthread_mutex_unlock (&gui->mmk_mutex);
 
       }
 
@@ -1363,12 +1354,14 @@ typedef struct {
         mediamark_insert_entry(gui->playlist.ref_append, ref->mrl, *title ? title : ref->mrl, NULL, ref->start_time, ref->duration ? ref->start_time + ref->duration : -1, 0, 0);
       } else {
         /* FIXME: title? start? duration? */
+        pthread_mutex_lock (&gui->mmk_mutex);
 	mediamark_t *mmk = mediamark_get_mmk_by_index(gui->playlist.ref_append);
 
 	if(mmk) {
 	  mediamark_append_alternate_mrl(mmk, ref->mrl);
 	  mediamark_set_got_alternate(mmk);
 	}
+        pthread_mutex_unlock (&gui->mmk_mutex);
       }
     }
     break;
@@ -1381,9 +1374,9 @@ typedef struct {
       xine_spu_button_t *spubtn = (xine_spu_button_t *) event->data;
       
       if(spubtn->direction)
-	video_window_set_cursor(CURSOR_HAND);
+        video_window_set_cursor (gui->vwin, CURSOR_HAND);
       else
-	video_window_set_cursor(CURSOR_ARROW);
+        video_window_set_cursor (gui->vwin, CURSOR_ARROW);
     }
     break;
 
@@ -1503,16 +1496,35 @@ int main(int argc, char *argv[]) {
   gui->nongui_error_msg       = NULL;
   gui->orig_stdout            = stdout;
   
-  window_attribute.x     = window_attribute.y      = -8192;
-  window_attribute.width = window_attribute.height = -1;
-  window_attribute.borderless = 0;
+  gui->panel                  = NULL;
+  gui->vwin                   = NULL;
+  gui->setup                  = NULL;
+  gui->mrlb                   = NULL;
+  gui->vctrl                  = NULL;
+
+  gui->load_stream            = NULL;
+  gui->load_sub               = NULL;
+
+  gui->last_playback_speed    = XINE_FINE_SPEED_NORMAL;
+
+  gui->window_attribute.x     =
+  gui->window_attribute.y     = -8192;
+  gui->window_attribute.width =
+  gui->window_attribute.height = -1;
+  gui->window_attribute.borderless = 0;
 
   _argv = build_command_line_args(argc, argv, &_argc);
 #ifdef TRACE_RC
   _rc_file_check_args(_argc, _argv);
 #endif
 
-  pthread_mutex_init(&gui->mmk_mutex, NULL);
+  {
+    pthread_mutexattr_t attr;
+    pthread_mutexattr_init (&attr);
+    pthread_mutexattr_settype (&attr, PTHREAD_MUTEX_RECURSIVE);
+    pthread_mutex_init (&gui->mmk_mutex, &attr);
+    pthread_mutexattr_destroy (&attr);
+  }
 
   visual_anim_init();
 
@@ -1715,7 +1727,7 @@ int main(int argc, char *argv[]) {
 
     case 'G': /* Set geometry */
       if(optarg != NULL) {
-	if(!parse_geometry(&window_attribute, optarg)) {
+        if (!parse_geometry (&gui->window_attribute, optarg)) {
 	  fprintf(stderr, _("Bad geometry '%s', see xine --help\n"), optarg);
 	  exit(1);
 	}
@@ -1723,7 +1735,7 @@ int main(int argc, char *argv[]) {
       break;
 
     case 'B':
-      window_attribute.borderless = 1;
+      gui->window_attribute.borderless = 1;
       break;
 
     case 'N':
@@ -1736,11 +1748,13 @@ int main(int argc, char *argv[]) {
       if (optarg) {
       if((!actions_on_start(gui->actions_on_start, ACTID_PLAYLIST) && (aos < MAX_ACTIONS_ON_START)))
 	gui->actions_on_start[aos++] = ACTID_PLAYLIST;
-      
+
+      pthread_mutex_lock (&gui->mmk_mutex);
       if(!gui->playlist.mmk)
 	mediamark_load_mediamarks(optarg);
       else
 	mediamark_concat_mediamarks(optarg);
+      pthread_mutex_unlock (&gui->mmk_mutex);
 
       /* don't load original playlist when loading this one */
       no_old_playlist = 1;
@@ -1978,9 +1992,11 @@ int main(int argc, char *argv[]) {
    * window geometry, so, reset those params.
    */
   if(gui->use_root_window) {
-    window_attribute.x     = window_attribute.y      = -8192;
-    window_attribute.width = window_attribute.height = -1;
-    window_attribute.borderless = 0;
+    gui->window_attribute.x =
+    gui->window_attribute.y = -8192;
+    gui->window_attribute.width =
+    gui->window_attribute.height = -1;
+    gui->window_attribute.borderless = 0;
   }
   
   show_banner();
@@ -2052,24 +2068,32 @@ int main(int argc, char *argv[]) {
     sprintf(gui->keymap_file, "%s/%s/%s", xine_get_homedir(), cfgdir, keymap);
   }
   
-  pthread_mutex_init(&gui->xe_mutex, NULL);
+  pthread_mutex_init (&gui->seek_mutex, NULL);
+  gui->seek_running = 0;
+  gui->seek_pos = -1;
+  gui->seek_timestep = 0;
 
-  __xineui_global_xine_instance = xine_new();
-  xine_config_load(__xineui_global_xine_instance, __xineui_global_config_file);
-  xine_engine_set_param(__xineui_global_xine_instance, XINE_ENGINE_PARAM_VERBOSITY, __xineui_global_verbosity);
+  pthread_mutex_init (&gui->no_messages.mutex, NULL);
+  gui->no_messages.until.tv_sec = 0;
+  gui->no_messages.until.tv_usec = 0;
+  gui->no_messages.level = 0;
+
+  __xineui_global_xine_instance = gui->xine = xine_new ();
+  xine_config_load (gui->xine, __xineui_global_config_file);
+  xine_engine_set_param (gui->xine, XINE_ENGINE_PARAM_VERBOSITY, __xineui_global_verbosity);
   
   /* 
    * Playlist auto reload 
    */
   old_playlist_cfg = 
-    xine_config_register_bool(__xineui_global_xine_instance, "gui.playlist_auto_reload", 
+    xine_config_register_bool (gui->xine, "gui.playlist_auto_reload", 
 			      0,
 			      _("Automatically reload old playlist"),
 			      _("If it's enabled and if you don't specify any MRL in command "
 				"line, xine will automatically load previous playlist."), 
 			      CONFIG_LEVEL_BEG,
 			      dummy_config_cb,
-			      CONFIG_NO_DATA);
+			      gGui);
 
   if(old_playlist_cfg && (!(_argc - optind)) && (!no_old_playlist)) {
     char buffer[XITK_PATH_MAX + XITK_NAME_MAX + 2];
@@ -2079,16 +2103,16 @@ int main(int argc, char *argv[]) {
   }
 
   gui->subtitle_autoload = 
-    xine_config_register_bool(__xineui_global_xine_instance, "gui.subtitle_autoload", 1,
+    xine_config_register_bool (gui->xine, "gui.subtitle_autoload", 1,
 			      _("Subtitle autoloading"),
 			      _("Automatically load subtitles if they exist."), 
 			      CONFIG_LEVEL_BEG,
 			      sub_autoload_cb,
-			      CONFIG_NO_DATA);
+			      gGui);
 
 
   enable_deinterlace +=
-    xine_config_register_bool(__xineui_global_xine_instance, "gui.deinterlace_by_default", 0,
+    xine_config_register_bool (gui->xine, "gui.deinterlace_by_default", 0,
 			      _("Enable deinterlacing by default"),
 			      _("Deinterlace plugin will be enabled on "
 			        "startup. Progressive streams are automatically "
@@ -2105,7 +2129,7 @@ int main(int argc, char *argv[]) {
   /*
    * init gui
    */
-  gui_init(_argc - optind, &_argv[optind], &window_attribute);
+  gui_init (gui, _argc - optind, &_argv[optind], &gui->window_attribute);
 
   pthread_mutex_init(&gui->download_mutex, NULL);
 
@@ -2125,13 +2149,13 @@ int main(int argc, char *argv[]) {
   /*
    * xine init
    */
-  xine_init(__xineui_global_xine_instance);
+  xine_init (gui->xine);
 
   /* Get old working path from input plugin */
   {
     xine_cfg_entry_t  cfg_entry;
     
-    if(xine_config_lookup_entry(__xineui_global_xine_instance, "media.files.origin_path", &cfg_entry))
+    if(xine_config_lookup_entry (gui->xine, "media.files.origin_path", &cfg_entry))
       strlcpy(gui->curdir, cfg_entry.str_value, sizeof(gui->curdir));
     else
       getcwd(&(gui->curdir[0]), XITK_PATH_MAX);
@@ -2143,24 +2167,24 @@ int main(int argc, char *argv[]) {
   /* Video out plugin */
   driver_num = -1;
   {
-    const char *const *vids = xine_list_video_output_plugins(__xineui_global_xine_instance);
+    const char *const *vids = xine_list_video_output_plugins (gui->xine);
     int                i = 0;
     
     while(vids[i++]);
     
-    video_driver_ids = (char **) calloc((i + 1), sizeof(char *));
+    gui->video_driver_ids = (char **)calloc ((i + 1), sizeof (char *));
     i = 0;
-    video_driver_ids[i] = strdup("auto");
+    gui->video_driver_ids[i] = strdup ("auto");
     while(vids[i]) {
-      video_driver_ids[i + 1] = strdup(vids[i]);
+      gui->video_driver_ids[i + 1] = strdup (vids[i]);
       i++;
     }
     
-    video_driver_ids[i + 1] = NULL;
+    gui->video_driver_ids[i + 1] = NULL;
     
     if(video_driver_id) {
-      for(i = 0; video_driver_ids[i] != NULL; i++) {
-	if(!strcasecmp(video_driver_id, video_driver_ids[i])) {
+      for (i = 0; gui->video_driver_ids[i] != NULL; i++) {
+        if (!strcasecmp (video_driver_id, gui->video_driver_ids[i])) {
 	  driver_num = i;
 	  break;
 	}
@@ -2172,12 +2196,12 @@ int main(int argc, char *argv[]) {
   {
     xine_cfg_entry_t  cfg_vo_entry;
     
-    if(xine_config_lookup_entry(__xineui_global_xine_instance, "video.driver", &cfg_vo_entry)) {
+    if(xine_config_lookup_entry (gui->xine, "video.driver", &cfg_vo_entry)) {
 
-      if(!strcasecmp(video_driver_ids[cfg_vo_entry.num_value], "dxr3")) {
+      if (!strcasecmp (gui->video_driver_ids[cfg_vo_entry.num_value], "dxr3")) {
 	xine_cfg_entry_t  cfg_entry;
 	
-	if(xine_config_lookup_entry(__xineui_global_xine_instance, "dxr3.output.mode", &cfg_entry)) {
+	if(xine_config_lookup_entry (gui->xine, "dxr3.output.mode", &cfg_entry)) {
 	  if(((!strcmp(cfg_entry.enum_values[cfg_entry.num_value], "letterboxed tv")) ||
 	      (!strcmp(cfg_entry.enum_values[cfg_entry.num_value], "widescreen tv"))) && 
 	     (!(actions_on_start(gui->actions_on_start, ACTID_TOGGLE_WINOUT_VISIBLITY)))) {
@@ -2186,7 +2210,7 @@ int main(int argc, char *argv[]) {
 	  }
 	}
       }
-      else if(!strcasecmp(video_driver_ids[cfg_vo_entry.num_value], "fb")) {
+      else if (!strcasecmp (gui->video_driver_ids[cfg_vo_entry.num_value], "fb")) {
 	if(aos < MAX_ACTIONS_ON_START)
 	  gui->actions_on_start[aos++] = ACTID_TOGGLE_WINOUT_VISIBLITY;
       }
@@ -2198,25 +2222,25 @@ int main(int argc, char *argv[]) {
   /* Audio out plugin */
   driver_num = -1;
   {
-    const char *const *aids = xine_list_audio_output_plugins(__xineui_global_xine_instance);
+    const char *const *aids = xine_list_audio_output_plugins (gui->xine);
     int                i = 0;
     
     while(aids[i++]);
     
-    audio_driver_ids = (char **) calloc((i + 2), sizeof(char *));
+    gui->audio_driver_ids = (char **)calloc ((i + 2), sizeof (char *));
     i = 0;
-    audio_driver_ids[i] = strdup("auto");
-    audio_driver_ids[i + 1] = strdup("null");
+    gui->audio_driver_ids[i] = strdup ("auto");
+    gui->audio_driver_ids[i + 1] = strdup ("null");
     while(aids[i]) {
-      audio_driver_ids[i + 2] = strdup(aids[i]);
+      gui->audio_driver_ids[i + 2] = strdup (aids[i]);
       i++;
     }
     
-    audio_driver_ids[i + 2] = NULL;
+    gui->audio_driver_ids[i + 2] = NULL;
     
     if(audio_driver_id) {
-      for(i = 0; audio_driver_ids[i] != NULL; i++) {
-	if(!strcasecmp(audio_driver_id, audio_driver_ids[i])) {
+      for (i = 0; gui->audio_driver_ids[i] != NULL; i++) {
+        if (!strcasecmp (audio_driver_id, gui->audio_driver_ids[i])) {
 	  driver_num = i;
 	  break;
 	}
@@ -2229,31 +2253,32 @@ int main(int argc, char *argv[]) {
   post_deinterlace_init(pdeinterlace);
   post_init();
 
-  gui->stream = xine_stream_new(__xineui_global_xine_instance, gui->ao_port, gui->vo_port);
+  gui->stream = xine_stream_new (gui->xine, gui->ao_port, gui->vo_port);
 #ifdef XINE_PARAM_EARLY_FINISHED_EVENT
   if( xine_check_version(1,1,1) )
       xine_set_param(gui->stream, XINE_PARAM_EARLY_FINISHED_EVENT, 1);
 #endif
 
-  gui->vo_none = xine_open_video_driver(__xineui_global_xine_instance, "none", XINE_VISUAL_TYPE_NONE, NULL);
-  gui->ao_none = xine_open_audio_driver(__xineui_global_xine_instance, "none", NULL);
+  gui->vo_none = xine_open_video_driver (gui->xine, "none", XINE_VISUAL_TYPE_NONE, NULL);
+  gui->ao_none = xine_open_audio_driver (gui->xine, "none", NULL);
 
   osd_init();
 
   /*
    * Setup logo.
    */
+#ifdef XINE_LOGO2_MRL
+#  define USE_XINE_LOGO_MRL XINE_LOGO2_MRL
+#else
+#  define USE_XINE_LOGO_MRL XINE_LOGO_MRL
+#endif
   gui->logo_mode = 0;
   gui->logo_has_changed = 0;
-  gui->logo_mrl = xine_config_register_string (__xineui_global_xine_instance, "gui.logo_mrl", XINE_LOGO_MRL,
-						_("Logo MRL"),
-						CONFIG_NO_HELP, 
-						CONFIG_LEVEL_EXP,
-						main_change_logo_cb, 
-						CONFIG_NO_DATA);
+  gui->logo_mrl = xine_config_register_string (gui->xine, "gui.logo_mrl", USE_XINE_LOGO_MRL,
+    _("Logo MRL"), CONFIG_NO_HELP, CONFIG_LEVEL_EXP, main_change_logo_cb, gGui);
 
   gui->event_queue = xine_event_new_queue(gui->stream);
-  xine_event_create_listener_thread(gui->event_queue, event_listener, NULL);
+  xine_event_create_listener_thread (gui->event_queue, event_listener, gui);
 
   if(tvout && strlen(tvout)) {
     if((gui->tvout = tvout_init(gui->display, tvout)))
@@ -2269,20 +2294,20 @@ int main(int argc, char *argv[]) {
 #endif
 
   /* Visual animation stream init */
-  gui->visual_anim.stream = xine_stream_new(__xineui_global_xine_instance, NULL, gui->vo_port);
+  gui->visual_anim.stream = xine_stream_new (gui->xine, NULL, gui->vo_port);
   gui->visual_anim.event_queue = xine_event_new_queue(gui->visual_anim.stream);
   gui->visual_anim.current = 0;
-  xine_event_create_listener_thread(gui->visual_anim.event_queue, event_listener, NULL);
+  xine_event_create_listener_thread (gui->visual_anim.event_queue, event_listener, gui);
   xine_set_param(gui->visual_anim.stream, XINE_PARAM_AUDIO_CHANNEL_LOGICAL, -2);
   xine_set_param(gui->visual_anim.stream, XINE_PARAM_SPU_CHANNEL, -2);
   xine_set_param(gui->visual_anim.stream, XINE_PARAM_AUDIO_REPORT_LEVEL, 0);
 
   /* subtitle stream */
-  gui->spu_stream = xine_stream_new(__xineui_global_xine_instance, NULL, gui->vo_port);
+  gui->spu_stream = xine_stream_new (gui->xine, NULL, gui->vo_port);
   xine_set_param(gui->spu_stream, XINE_PARAM_AUDIO_REPORT_LEVEL, 0);
 
   /* init the video window */
-  video_window_select_visual();
+  video_window_select_visual (gui->vwin);
 
   xine_set_param(gui->stream, XINE_PARAM_VO_ASPECT_RATIO, aspect_ratio);
         
@@ -2317,16 +2342,16 @@ int main(int argc, char *argv[]) {
    * callback functions are called. If libGL.so has already been unloaded
    * with dlclose() this will cause a segmentation fault.
    */
-  XLockDisplay(gui->display);
-  XUnlockDisplay(gui->display);
+  gui->x_lock_display (gui->display);
+  gui->x_unlock_display (gui->display);
   XCloseDisplay(gui->display);
   if( gui->video_display != gui->display ) {
-    XLockDisplay(gui->video_display);
-    XUnlockDisplay(gui->video_display);
+    gui->x_lock_display (gui->video_display);
+    gui->x_unlock_display (gui->video_display);
     XCloseDisplay(gui->video_display);
   }
 
-  xine_exit(__xineui_global_xine_instance);
+  xine_exit (gui->xine);
 
   visual_anim_done();
   free(pplugins);
@@ -2341,7 +2366,8 @@ int main(int argc, char *argv[]) {
   }
 
   pthread_mutex_destroy(&gui->mmk_mutex);
-  pthread_mutex_destroy(&gui->xe_mutex);
+  pthread_mutex_destroy (&gui->no_messages.mutex);
+  pthread_mutex_destroy (&gui->seek_mutex);
   pthread_mutex_destroy(&gui->download_mutex);
   pthread_mutex_destroy(&gui->logo_mutex);
 
@@ -2354,12 +2380,13 @@ int main(int argc, char *argv[]) {
 
   {
     int ii;
-    for (ii = 0; video_driver_ids[ii]; ii++)
-      free(video_driver_ids[ii]);
-    free(video_driver_ids);
-    for (ii = 0; audio_driver_ids[ii]; ii++)
-      free(audio_driver_ids[ii]);
-    free(audio_driver_ids);
+    for (ii = 0; gui->video_driver_ids[ii]; ii++)
+      free (gui->video_driver_ids[ii]);
+    free (gui->video_driver_ids);
+    for (ii = 0; gui->audio_driver_ids[ii]; ii++)
+      free (gui->audio_driver_ids[ii]);
+    free (gui->audio_driver_ids);
   }
   return retval;
 }
+
