@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2017 the xine project
+ * Copyright (C) 2000-2019 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -91,8 +91,11 @@ static char *_download_file(const char *filename, int *size) {
   
   if((network_download(filename, download))) {
     *size = download->size;
-    buf = (char *) malloc(*size);
-    memcpy(buf, download->buf, *size);
+    buf = (char *) malloc(*size + 1);
+    if (buf) {
+      memcpy(buf, download->buf, *size);
+      buf[*size] = 0;
+    }
   }
   else
     xine_error("Unable to download '%s': %s", filename, download->error);
@@ -115,25 +118,40 @@ static int _file_exist(char *filename) {
 }
 #endif
 
+int mrl_look_like_playlist (const char *mrl) {
+  /* TJ. I dont know whether someone really needs to treat
+   * "foo/bar.m3under/the/table" as an m3u playlist.
+   * Lets keep this behaviour for now, but make sure that
+   * hls (.m3u8) goes to xine-lib verbatim. */
+  const char *extension = strrchr (mrl, '.');
+  if (extension) {
+    /* All known playlist ending */
+    if ((!strncasecmp (extension, ".asx", 4))  ||
+        (!strncasecmp (extension, ".smi", 4))  ||
+      /*(!strncasecmp (extension, ".smil", 5)) || caught by ".smi" */
+        (!strncasecmp (extension, ".pls", 4))  ||
+        (!strncasecmp (extension, ".sfv", 4))  ||
+        (!strncasecmp (extension, ".xml", 4))  ||
+        (!strncasecmp (extension, ".tox", 4))  ||
+        (!strncasecmp (extension, ".fxd", 4)))
+      return 1;
+    if ((!strncasecmp (extension, ".m3u", 4)) && (extension[4] != '8'))
+      return 1;
+  }
+  return 0;
+}
+ 
 static char *_read_file(const char *filename, int *size) {
   struct stat  st;
   char        *buf = NULL;
   int          fd, bytes_read;
-  char        *extension;
 
   if((!filename) || (!strlen(filename))) {
     fprintf(stderr, "%s(): Empty or NULL filename.\n", __XINE_FUNCTION__);
     return NULL;
   }
-  
-  extension = strrchr(filename, '.');
-  if(extension && 
-     ((!strncasecmp(extension, ".pls", 4)) || (!strncasecmp(extension, ".m3u", 4))  || 
-      (!strncasecmp(extension, ".sfv", 4)) || (!strncasecmp(extension, ".tox", 4))  ||
-      (!strncasecmp(extension, ".asx", 4)) || (!strncasecmp(extension, ".smi", 4))  || 
-      (!strncasecmp(extension, ".smil", 5)) || (!strncasecmp(extension, ".xml", 4)) ||
-      (!strncasecmp(extension, ".fxd", 4))) &&
-     is_downloadable((char *) filename))
+
+  if (mrl_look_like_playlist (filename) && is_downloadable ((char *)filename))
     return _download_file(filename, size);
   
   if(stat(filename, &st) < 0) {
@@ -2518,7 +2536,7 @@ int mediamark_get_entry_from_id(const char *ident) {
 void mediamark_insert_entry(int index, const char *mrl, const char *ident, 
 			    const char *sub, int start, int end, int av_offset, int spu_offset) {
   gGui_t *gui = gGui;
-  char  autosub[XITK_PATH_MAX + XITK_NAME_MAX + 2];
+  char  autosub[2*XITK_PATH_MAX + XITK_NAME_MAX + 2];
   char  subpath[XITK_PATH_MAX + XITK_NAME_MAX + 2];
   DIR           *dir;
   struct dirent *dentry;
@@ -2967,26 +2985,6 @@ void mediamark_save_mediamarks(const char *filename) {
   free(fullfn);
 }
 
-int mrl_look_like_playlist(char *mrl) {
-  char *extension;
-  
-  extension = strrchr(mrl, '.');
-  if(extension && /* All known playlist ending */
-     ((!strncasecmp(extension, ".asx", 4))  ||
-      (!strncasecmp(extension, ".smi", 4))  ||
-      (!strncasecmp(extension, ".smil", 5)) ||
-      (!strncasecmp(extension, ".pls", 4))  ||
-      (!strncasecmp(extension, ".m3u", 4))  ||
-      (!strncasecmp(extension, ".sfv", 4))  ||
-      (!strncasecmp(extension, ".xml", 4))  ||
-      (!strncasecmp(extension, ".tox", 4))  ||
-      (!strncasecmp(extension, ".fxd", 4)))) {
-    return 1;
-  }
-  
-  return 0;
-}
- 
 int mrl_look_like_file(char *mrl) {
   
   if(mrl && strlen(mrl)) {
@@ -3081,11 +3079,11 @@ static void mmkeditor_exit(xitk_widget_t *w, void *data) {
     xitk_window_destroy_window(gui->imlib_data, mmkeditor.xwin);
     
     mmkeditor.xwin = NULL;
-    xitk_list_free((XITK_WIDGET_LIST_LIST(mmkeditor.widget_list)));
+    /* xitk_dlist_init (&mmkeditor.widget_list.list); */
     
-    XLockDisplay(gui->display);
+    gui->x_lock_display (gui->display);
     XFreeGC(gui->display, (XITK_WIDGET_LIST_GC(mmkeditor.widget_list)));
-    XUnlockDisplay(gui->display);
+    gui->x_unlock_display (gui->display);
     
     XITK_WIDGET_LIST_FREE(mmkeditor.widget_list);
     
@@ -3100,7 +3098,7 @@ static void mmkeditor_handle_event(XEvent *event, void *data) {
     if(xitk_get_key_pressed(event) == XK_Escape)
       mmkeditor_exit(NULL, NULL);
     else
-      gui_handle_event(event, data);
+      gui_handle_event (event, gGui);
   }
 }
 
@@ -3168,13 +3166,13 @@ static void mmkeditor_apply(xitk_widget_t *w, void *data) {
   if(mmkeditor.mmk) {
     
     mrl = atoa(xitk_inputtext_get_text(mmkeditor.mrl));
-    if(mrl && (!strlen(mrl)))
+    if (!mrl[0])
       mrl = strdup((*mmkeditor.mmk)->mrl);
     else
       mrl = strdup(mrl);
     
     ident = atoa(xitk_inputtext_get_text(mmkeditor.ident));
-    if(ident && (!strlen(ident)))
+    if (!ident[0])
       ident = strdup(mrl);
     else
       ident = strdup(ident);
@@ -3299,13 +3297,12 @@ void mmk_edit_mediamark(mediamark_t **mmk, apply_callback_t callback, void *data
   
   set_window_states_start((xitk_window_get_window(mmkeditor.xwin)));
   
-  XLockDisplay (gui->display);
+  gui->x_lock_display (gui->display);
   gc = XCreateGC(gui->display, 
 		 (xitk_window_get_window(mmkeditor.xwin)), None, None);
-  XUnlockDisplay (gui->display);
+  gui->x_unlock_display (gui->display);
   
   mmkeditor.widget_list = xitk_widget_list_new();
-  xitk_widget_list_set(mmkeditor.widget_list, WIDGET_LIST_LIST, (xitk_list_new()));
   xitk_widget_list_set(mmkeditor.widget_list, 
 		       WIDGET_LIST_WINDOW, (void *) (xitk_window_get_window(mmkeditor.xwin)));
   xitk_widget_list_set(mmkeditor.widget_list, WIDGET_LIST_GC, gc);
@@ -3318,10 +3315,10 @@ void mmk_edit_mediamark(mediamark_t **mmk, apply_callback_t callback, void *data
 
   xitk_window_get_window_size(mmkeditor.xwin, &width, &height);
   bg = xitk_image_create_xitk_pixmap(gui->imlib_data, width, height);
-  XLockDisplay (gui->display);
+  gui->x_lock_display (gui->display);
   XCopyArea(gui->display, (xitk_window_get_background(mmkeditor.xwin)), bg->pixmap,
 	    bg->gc, 0, 0, width, height, 0, 0);
-  XUnlockDisplay (gui->display);
+  gui->x_unlock_display (gui->display);
 
   x = 15;
   y = 34 - 6;
@@ -3334,11 +3331,9 @@ void mmk_edit_mediamark(mediamark_t **mmk, apply_callback_t callback, void *data
   inp.max_length        = 2048;
   inp.callback          = NULL;
   inp.userdata          = NULL;
-  xitk_list_append_content((XITK_WIDGET_LIST_LIST(mmkeditor.widget_list)),
-	   (mmkeditor.ident = 
-	    xitk_noskin_inputtext_create(mmkeditor.widget_list, &inp,
-					 x + 10, y + 16, w - 20, 20,
-					 "Black", "Black", fontname)));
+  mmkeditor.ident = xitk_noskin_inputtext_create (mmkeditor.widget_list, &inp,
+    x + 10, y + 16, w - 20, 20, "Black", "Black", fontname);
+  xitk_add_widget (mmkeditor.widget_list, mmkeditor.ident);
   xitk_set_widget_tips_default(mmkeditor.ident, _("Mediamark Identifier"));
   xitk_enable_and_show_widget(mmkeditor.ident);
 
@@ -3351,11 +3346,9 @@ void mmk_edit_mediamark(mediamark_t **mmk, apply_callback_t callback, void *data
   inp.max_length        = 2048;
   inp.callback          = NULL;
   inp.userdata          = NULL;
-  xitk_list_append_content((XITK_WIDGET_LIST_LIST(mmkeditor.widget_list)),
-	    (mmkeditor.mrl = 
-	     xitk_noskin_inputtext_create(mmkeditor.widget_list, &inp,
-					  x + 10, y + 16, w - 20, 20,
-					  "Black", "Black", fontname)));
+  mmkeditor.mrl = xitk_noskin_inputtext_create (mmkeditor.widget_list, &inp,
+    x + 10, y + 16, w - 20, 20, "Black", "Black", fontname);
+  xitk_add_widget (mmkeditor.widget_list, mmkeditor.mrl);
   xitk_set_widget_tips_default(mmkeditor.mrl, _("Mediamark Mrl"));
   xitk_enable_and_show_widget(mmkeditor.mrl);
 
@@ -3368,11 +3361,9 @@ void mmk_edit_mediamark(mediamark_t **mmk, apply_callback_t callback, void *data
   inp.max_length        = 2048;
   inp.callback          = NULL;
   inp.userdata          = NULL;
-  xitk_list_append_content((XITK_WIDGET_LIST_LIST(mmkeditor.widget_list)),
-	    (mmkeditor.sub = 
-	     xitk_noskin_inputtext_create(mmkeditor.widget_list, &inp,
-					  x + 10, y + 16, w - 20 - 100 - 10, 20,
-					  "Black", "Black", fontname)));
+  mmkeditor.sub = xitk_noskin_inputtext_create (mmkeditor.widget_list, &inp,
+    x + 10, y + 16, w - 20 - 100 - 10, 20, "Black", "Black", fontname);
+  xitk_add_widget (mmkeditor.widget_list, mmkeditor.sub);
   xitk_set_widget_tips_default(mmkeditor.sub, _("Subtitle File"));
   xitk_enable_and_show_widget(mmkeditor.sub);
 
@@ -3383,10 +3374,9 @@ void mmk_edit_mediamark(mediamark_t **mmk, apply_callback_t callback, void *data
   lb.state_callback    = NULL;
   lb.userdata          = NULL;
   lb.skin_element_name = NULL;
-  xitk_list_append_content((XITK_WIDGET_LIST_LIST(mmkeditor.widget_list)), 
-	   (b = xitk_noskin_labelbutton_create(mmkeditor.widget_list, &lb,
-					       x + 10 + w - 20 - 100, y + 16, 100, 20,
-					       "Black", "Black", "White", btnfontname)));
+  b =  xitk_noskin_labelbutton_create (mmkeditor.widget_list, &lb,
+    x + 10 + w - 20 - 100, y + 16, 100, 20, "Black", "Black", "White", btnfontname);
+  xitk_add_widget (mmkeditor.widget_list, b);
   xitk_set_widget_tips_default(b, _("Select a subtitle file to use together with the mrl."));
   xitk_enable_and_show_widget(b);
 
@@ -3401,10 +3391,9 @@ void mmk_edit_mediamark(mediamark_t **mmk, apply_callback_t callback, void *data
   ib.parent_wlist      = mmkeditor.widget_list;
   ib.callback          = NULL;
   ib.userdata          = NULL;
-  xitk_list_append_content((XITK_WIDGET_LIST_LIST(mmkeditor.widget_list)),
-	    (mmkeditor.start = 
-	     xitk_noskin_intbox_create(mmkeditor.widget_list, &ib, 
-				       x + 30, y + 16, w - 60, 20, NULL, NULL, NULL)));
+  mmkeditor.start = xitk_noskin_intbox_create (mmkeditor.widget_list, &ib,
+    x + 30, y + 16, w - 60, 20, NULL, NULL, NULL);
+  xitk_add_widget (mmkeditor.widget_list, mmkeditor.start);
   xitk_set_widget_tips_default(mmkeditor.start, _("Mediamark start time (secs)."));
   xitk_enable_and_show_widget(mmkeditor.start);
 
@@ -3418,10 +3407,9 @@ void mmk_edit_mediamark(mediamark_t **mmk, apply_callback_t callback, void *data
   ib.parent_wlist      = mmkeditor.widget_list;
   ib.callback          = NULL;
   ib.userdata          = NULL;
-  xitk_list_append_content((XITK_WIDGET_LIST_LIST(mmkeditor.widget_list)),
-	    (mmkeditor.end = 
-	     xitk_noskin_intbox_create(mmkeditor.widget_list, &ib, 
-				       x + 30, y + 16, w - 60, 20, NULL, NULL, NULL)));
+  mmkeditor.end = xitk_noskin_intbox_create (mmkeditor.widget_list, &ib,
+    x + 30, y + 16, w - 60, 20, NULL, NULL, NULL);
+  xitk_add_widget (mmkeditor.widget_list, mmkeditor.end);
   xitk_set_widget_tips_default(mmkeditor.end, _("Mediamark end time (secs)."));
   xitk_enable_and_show_widget(mmkeditor.end);
 
@@ -3435,10 +3423,9 @@ void mmk_edit_mediamark(mediamark_t **mmk, apply_callback_t callback, void *data
   ib.parent_wlist      = mmkeditor.widget_list;
   ib.callback          = NULL;
   ib.userdata          = NULL;
-  xitk_list_append_content((XITK_WIDGET_LIST_LIST(mmkeditor.widget_list)),
-	    (mmkeditor.av_offset = 
-	     xitk_noskin_intbox_create(mmkeditor.widget_list, &ib, 
-				       x + 30, y + 16, w - 60, 20, NULL, NULL, NULL)));
+  mmkeditor.av_offset = xitk_noskin_intbox_create (mmkeditor.widget_list, &ib,
+    x + 30, y + 16, w - 60, 20, NULL, NULL, NULL);
+  xitk_add_widget (mmkeditor.widget_list, mmkeditor.av_offset);
   xitk_set_widget_tips_default(mmkeditor.av_offset, _("Offset of Audio and Video."));
   xitk_enable_and_show_widget(mmkeditor.av_offset);
 
@@ -3452,10 +3439,9 @@ void mmk_edit_mediamark(mediamark_t **mmk, apply_callback_t callback, void *data
   ib.parent_wlist      = mmkeditor.widget_list;
   ib.callback          = NULL;
   ib.userdata          = NULL;
-  xitk_list_append_content((XITK_WIDGET_LIST_LIST(mmkeditor.widget_list)),
-	    (mmkeditor.spu_offset = 
-	     xitk_noskin_intbox_create(mmkeditor.widget_list, &ib, 
-				       x + 30, y + 16, w - 60, 20, NULL, NULL, NULL)));
+  mmkeditor.spu_offset = xitk_noskin_intbox_create (mmkeditor.widget_list, &ib,
+    x + 30, y + 16, w - 60, 20, NULL, NULL, NULL);
+  xitk_add_widget (mmkeditor.widget_list, mmkeditor.spu_offset);
   xitk_set_widget_tips_default(mmkeditor.spu_offset, _("Subpicture offset."));
   xitk_enable_and_show_widget(mmkeditor.spu_offset);
 
@@ -3468,10 +3454,9 @@ void mmk_edit_mediamark(mediamark_t **mmk, apply_callback_t callback, void *data
   lb.state_callback    = NULL;
   lb.userdata          = NULL;
   lb.skin_element_name = NULL;
-  xitk_list_append_content((XITK_WIDGET_LIST_LIST(mmkeditor.widget_list)), 
-	   (b = xitk_noskin_labelbutton_create(mmkeditor.widget_list, 
-					       &lb, x, y, 100, 23,
-					       "Black", "Black", "White", btnfontname)));
+  b =  xitk_noskin_labelbutton_create (mmkeditor.widget_list,
+    &lb, x, y, 100, 23, "Black", "Black", "White", btnfontname);
+  xitk_add_widget (mmkeditor.widget_list, b);
   xitk_set_widget_tips_default(b, _("Apply the changes and close the window."));
   xitk_enable_and_show_widget(b);
 
@@ -3483,10 +3468,9 @@ void mmk_edit_mediamark(mediamark_t **mmk, apply_callback_t callback, void *data
   lb.state_callback    = NULL;
   lb.userdata          = NULL;
   lb.skin_element_name = NULL;
-  xitk_list_append_content((XITK_WIDGET_LIST_LIST(mmkeditor.widget_list)), 
-	   (b = xitk_noskin_labelbutton_create(mmkeditor.widget_list, 
-					       &lb, x, y, 100, 23,
-					       "Black", "Black", "White", btnfontname)));
+  b =  xitk_noskin_labelbutton_create (mmkeditor.widget_list,
+    &lb, x, y, 100, 23, "Black", "Black", "White", btnfontname);
+  xitk_add_widget (mmkeditor.widget_list, b);
   xitk_set_widget_tips_default(b, _("Apply the changes to the playlist."));
   xitk_enable_and_show_widget(b);
 
@@ -3498,13 +3482,12 @@ void mmk_edit_mediamark(mediamark_t **mmk, apply_callback_t callback, void *data
   lb.state_callback    = NULL;
   lb.userdata          = NULL;
   lb.skin_element_name = NULL;
-  xitk_list_append_content((XITK_WIDGET_LIST_LIST(mmkeditor.widget_list)), 
-	   (b = xitk_noskin_labelbutton_create(mmkeditor.widget_list, 
-					       &lb, x, y, 100, 23,
-					       "Black", "Black", "White", btnfontname)));
+  b =  xitk_noskin_labelbutton_create (mmkeditor.widget_list,
+    &lb, x, y, 100, 23, "Black", "Black", "White", btnfontname);
+  xitk_add_widget (mmkeditor.widget_list, b);
   xitk_set_widget_tips_default(b, _("Discard changes and dismiss the window."));
   xitk_enable_and_show_widget(b);
-  mmk_editor_show_tips(panel_get_tips_enable(), panel_get_tips_timeout());
+  mmk_editor_show_tips (panel_get_tips_enable (gui->panel), panel_get_tips_timeout (gui->panel));
 
   xitk_window_change_background(gui->imlib_data, mmkeditor.xwin, bg->pixmap, width, height);
   xitk_image_destroy_xitk_pixmap(bg);
@@ -3525,3 +3508,4 @@ void mmk_edit_mediamark(mediamark_t **mmk, apply_callback_t callback, void *data
 
   try_to_set_input_focus(xitk_window_get_window(mmkeditor.xwin));
 }
+
