@@ -1,6 +1,6 @@
 #warning IMPLEMENT POST SUPPORT
 /*
- * Copyright (C) 2000-2017 the xine project
+ * Copyright (C) 2000-2019 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -1135,7 +1135,7 @@ static void session_single_shot(session_t *session, int num_commands, char *comm
 
 static void show_version(void) {
   printf("This is %s - xine's remote control v%s.\n"
-	 "(c) 2000-2003 The xine Team.\n", PROGNAME, PROGVERSION);
+	 "(c) 2000-2019 The xine Team.\n", PROGNAME, PROGVERSION);
 }
 
 static void show_usage(void) {
@@ -1617,17 +1617,21 @@ static void handle_xine_error(client_info_t *client_info) {
     break;
     
   case XINE_ERROR_NO_INPUT_PLUGIN:
+    pthread_mutex_lock (&gui->mmk_mutex);
     sock_write(client_info->socket,
 	       "xine engine error:\n"
 	       "There is no available input plugin available to handle '%s'.\n",
 	       gui->mmk.mrl);
+    pthread_mutex_unlock (&gui->mmk_mutex);
     break;
     
   case XINE_ERROR_NO_DEMUX_PLUGIN:
+    pthread_mutex_lock (&gui->mmk_mutex);
     sock_write(client_info->socket, 
 	       "xine engine error:\n"
 	       "There is no available demuxer plugin to handle '%s'.\n", 
 	       gui->mmk.mrl);
+    pthread_mutex_unlock (&gui->mmk_mutex);
     break;
     
   default:
@@ -1855,7 +1859,7 @@ static void do_mrl(const commands_t *cmd, client_info_t *client_info) {
 	gui->ignore_next = 1;
 	xine_stop (gui->stream);
 	gui->ignore_next = 0;
-	gui_playlist_start_next();
+        gui_playlist_start_next (gui);
       }
       else if(is_arg_contain(client_info, 1, "prev")) {
 	gui->ignore_next = 1;
@@ -1863,8 +1867,10 @@ static void do_mrl(const commands_t *cmd, client_info_t *client_info) {
 	gui->playlist.cur--;
 	if ((gui->playlist.cur >= 0) && (gui->playlist.cur < gui->playlist.num)) {
 	  gui_set_current_mmk(mediamark_get_current_mmk());
+          pthread_mutex_lock (&gui->mmk_mutex);
 	  (void) gui_xine_open_and_play(gui->mmk.mrl, gui->mmk.sub, 0, 
 					gui->mmk.start, gui->mmk.av_offset, gui->mmk.spu_offset, 1);
+          pthread_mutex_unlock (&gui->mmk_mutex);
   
 	} 
 	else {
@@ -1892,6 +1898,7 @@ static void do_mrl(const commands_t *cmd, client_info_t *client_info) {
 	  xine_stop(gui->stream);
 	  gui->ignore_next = 0;
 	}
+        pthread_mutex_lock (&gui->mmk_mutex);
 	gui_set_current_mmk(gui->playlist.mmk[gui->playlist.num - 1]);
 	if(!(xine_open(gui->stream, gui->mmk.mrl) 
 	     && xine_play (gui->stream, 0, gui->mmk.start))) {
@@ -1900,6 +1907,7 @@ static void do_mrl(const commands_t *cmd, client_info_t *client_info) {
 	}
 	else
 	  gui->logo_mode = 0;
+        pthread_mutex_unlock (&gui->mmk_mutex);
       }
     }
   }
@@ -1918,10 +1926,12 @@ static void do_playlist(const commands_t *cmd, client_info_t *client_info) {
 	int i;
 
 	if(gui->playlist.num) {
+          pthread_mutex_lock (&gui->mmk_mutex);
 	  for(i = 0; i < gui->playlist.num; i++) {
 	    sock_write(client_info->socket, "%2s %5d %s\n", 
 		       (i == gui->playlist.cur) ? "*>" : "", i, gui->playlist.mmk[i]->mrl);
 	  }
+          pthread_mutex_unlock (&gui->mmk_mutex);
 	}
 	else
 	  sock_write(client_info->socket, "empty playlist.");
@@ -1951,7 +1961,7 @@ static void do_playlist(const commands_t *cmd, client_info_t *client_info) {
 
 	    gui->playlist.cur = entry;
 	    gui_set_current_mmk(mediamark_get_current_mmk());
-	    gui_play(NULL, NULL);
+            gui_play (NULL, gui);
 	  }
 	}
       }
@@ -1987,16 +1997,16 @@ static void do_playlist(const commands_t *cmd, client_info_t *client_info) {
       }
       else if(is_arg_contain(client_info, 1, "delete")) {
 	int i;
-	
+
 	if((is_arg_contain(client_info, 2, "all")) || 
 	   (is_arg_contain(client_info, 2, "*"))) {
-	  
+  
 	  mediamark_free_mediamarks();
-	  
+  
 	  if(xine_get_status(gui->stream) != XINE_STATUS_STOP)
-	    gui_stop(NULL, NULL);
-	  
-	  enable_playback_controls(0);
+            gui_stop (NULL, gui);
+  
+          enable_playback_controls (gui->panel, 0);
 	}
 	else {
 	  int j;
@@ -2010,6 +2020,7 @@ static void do_playlist(const commands_t *cmd, client_info_t *client_info) {
 	    
 	    mediamark_free_entry(j);
 	    
+            pthread_mutex_lock (&gui->mmk_mutex);
 	    for(i = j; i < gui->playlist.num; i++)
 	      gui->playlist.mmk[i] = gui->playlist.mmk[i + 1];
 	    
@@ -2018,6 +2029,7 @@ static void do_playlist(const commands_t *cmd, client_info_t *client_info) {
 	    gui->playlist.mmk[gui->playlist.num + 1] = NULL;
 	    
 	    gui->playlist.cur = 0;
+            pthread_mutex_unlock (&gui->mmk_mutex);
 	  }
 	}
 	
@@ -2028,8 +2040,8 @@ static void do_playlist(const commands_t *cmd, client_info_t *client_info) {
 	  gui_set_current_mmk(mediamark_get_current_mmk());
 	else {
 	  
-	  if(is_playback_widgets_enabled())
-	    enable_playback_controls(0);
+          if (is_playback_widgets_enabled (gui->panel))
+            enable_playback_controls (gui->panel, 0);
 	  
 	  if(xine_get_status(gui->stream) != XINE_STATUS_STOP)
 	    gui_stop(NULL, NULL);
@@ -2045,12 +2057,14 @@ static void do_play(const commands_t *cmd, client_info_t *client_info) {
   gGui_t *gui = gGui;
 
   if (xine_get_status (gui->stream) != XINE_STATUS_PLAY) {
+    pthread_mutex_lock (&gui->mmk_mutex);
     if(!(xine_open(gui->stream, gui->mmk.mrl) && xine_play (gui->stream, 0, gui->mmk.start))) {
       handle_xine_error(client_info);
       gui_display_logo();
     }
     else
       gui->logo_mode = 0;
+    pthread_mutex_unlock (&gui->mmk_mutex);
   } 
   else {
     xine_set_param(gui->stream, XINE_PARAM_SPEED, XINE_SPEED_NORMAL);
@@ -2084,14 +2098,14 @@ static void do_exit(const commands_t *cmd, client_info_t *client_info) {
 static void do_fullscreen(const commands_t *cmd, client_info_t *client_info) {
   action_id_t action = ACTID_TOGGLE_FULLSCREEN;
 
-  gui_execute_action_id(action);
+  gui_execute_action_id (gGui, action);
 }
 
 #ifdef HAVE_XINERAMA
 static void do_xinerama_fullscreen(const commands_t *cmd, client_info_t *client_info) {
   action_id_t action = ACTID_TOGGLE_XINERAMA_FULLSCREEN;
 
-  gui_execute_action_id(action);
+  gui_execute_action_id (gGui, action);
 }
 #endif
 
@@ -2358,17 +2372,17 @@ static void do_gui(const commands_t *cmd, client_info_t *client_info) {
       int flushing = 0;
       
       if(is_arg_contain(client_info, 1, "hide")) {
-	if(panel_is_visible()) {
-	  panel_toggle_visibility(NULL, NULL);
+        if (panel_is_visible (gui->panel)) {
+          panel_toggle_visibility (NULL, gui->panel);
 	  flushing++;
 	}
       }
       else if(is_arg_contain(client_info, 1, "output")) {
-	gui_toggle_visibility(NULL, NULL);
+        gui_toggle_visibility (NULL, gui);
 	flushing++;
       }
       else if(is_arg_contain(client_info, 1, "panel")) {
-	panel_toggle_visibility(NULL, NULL);
+	panel_toggle_visibility (NULL, gui->panel);
 	flushing++;
       }
       else if(is_arg_contain(client_info, 1, "playlist")) {
@@ -2406,9 +2420,9 @@ static void do_gui(const commands_t *cmd, client_info_t *client_info) {
 
       /* Flush event when xine !play */
       if(flushing && ((xine_get_status(gui->stream) != XINE_STATUS_PLAY))) {
-	XLockDisplay(gui->display);
+	gui->x_lock_display (gui->display);
 	XSync(gui->display, False);
-	XUnlockDisplay(gui->display);
+	gui->x_unlock_display (gui->display);
       }
     }
   }
@@ -2565,7 +2579,7 @@ static void do_seek(const commands_t *cmd, client_info_t *client_info) {
 }
 
 static void do_halt(const commands_t *cmd, client_info_t *client_info) {
-  gui_exit(NULL, NULL);
+  gui_exit (NULL, gGui);
 }
 
 static void network_messenger(void *data, char *message) {
@@ -2576,8 +2590,10 @@ static void network_messenger(void *data, char *message) {
 
 static void do_snap(const commands_t *cmd, client_info_t *client_info) {
   gGui_t *gui = gGui;
+  pthread_mutex_lock (&gui->mmk_mutex);
   create_snapshot(gui->mmk.mrl, 
 		  network_messenger, network_messenger, (void *)(intptr_t)client_info->socket);
+  pthread_mutex_unlock (&gui->mmk_mutex);
 }
 
 /*
