@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2017 the xine project
+ * Copyright (C) 2000-2019 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -66,9 +66,9 @@
  
 typedef struct {
   xine_post_t                 *post;
-  xine_post_api_t             *api;
-  xine_post_api_descr_t       *descr;
-  xine_post_api_parameter_t   *param;
+  const xine_post_api_t             *api;
+  const xine_post_api_descr_t       *descr;
+  const xine_post_api_parameter_t   *param;
   char                        *param_data;
 
   int                          x;
@@ -145,7 +145,7 @@ static void _applugin_close_help(xitk_widget_t *, void *);
 static void _pplugin_close_help(_pp_wrapper_t *pp_wrapper, xitk_widget_t *, void *);
 
 static void post_deinterlace_plugin_cb(void *data, xine_cfg_entry_t *cfg) {
-  gGui_t *gui = gGui;
+  gGui_t *gui = data;
   post_element_t **posts = NULL;
   int              num, i;
   
@@ -173,7 +173,7 @@ static void post_deinterlace_plugin_cb(void *data, xine_cfg_entry_t *cfg) {
 }
 
 static void post_audio_plugin_cb(void *data, xine_cfg_entry_t *cfg) {
-  gGui_t *gui = gGui;
+  gGui_t *gui = data;
   gui->visual_anim.post_plugin_num = cfg->num_value;  
   post_rewire_visual_anim();
 }
@@ -215,7 +215,7 @@ void post_init(void) {
 				    _("Post audio plugin to used when playing streams without video"),
 				    CONFIG_LEVEL_BEG,
 				    post_audio_plugin_cb,
-				    CONFIG_NO_DATA);
+				    gGui);
 	
 	gui->visual_anim.post_output_element.post = 
 	  xine_post_init(__xineui_global_xine_instance,
@@ -543,28 +543,15 @@ static void _pplugin_paint_widgets(_pp_wrapper_t *pp_wrapper) {
 static void _pplugin_send_expose(_pp_wrapper_t *pp_wrapper) {
   gGui_t *gui = gGui;
   if(pp_wrapper->pplugin) {
-    XLockDisplay(gui->display);
+    gui->x_lock_display (gui->display);
     XClearWindow(gui->display, (XITK_WIDGET_LIST_WINDOW(pp_wrapper->pplugin->widget_list)));
-    XUnlockDisplay(gui->display);
+    gui->x_unlock_display (gui->display);
   }
 }
 
-static void _pplugin_destroy_widget(_pp_wrapper_t *pp_wrapper, xitk_widget_t *w) {
-  xitk_widget_t *cw;
-
-  cw = (xitk_widget_t *) xitk_list_first_content((XITK_WIDGET_LIST_LIST(pp_wrapper->pplugin->widget_list)));
-  while(cw) {
-    
-    if(cw == w) {
-      xitk_hide_widget(cw);
-      xitk_destroy_widget(cw);
-      cw = NULL;
-      xitk_list_delete_current((XITK_WIDGET_LIST_LIST(pp_wrapper->pplugin->widget_list)));
-      return;
-    }
-    
-    cw = (xitk_widget_t *) xitk_list_next_content((XITK_WIDGET_LIST_LIST(pp_wrapper->pplugin->widget_list)));
-  }
+static void _pplugin_destroy_widget (xitk_widget_t **w) {
+  xitk_destroy_widget (*w);
+  *w = NULL;
 }
 
 static void _pplugin_update_parameter(post_object_t *pobj) {
@@ -618,7 +605,7 @@ static void _pplugin_set_param_double(xitk_widget_t *w, void *data, double value
   }
 }
 
-static void _pplugin_set_param_char(xitk_widget_t *w, void *data, char *text) {
+static void _pplugin_set_param_char(xitk_widget_t *w, void *data, const char *text) {
   post_object_t *pobj = (post_object_t *) data;
 
   if(pobj->readonly)
@@ -675,9 +662,9 @@ static void _pplugin_add_parameter_widget(_pp_wrapper_t *pp_wrapper, post_object
     lb.label               = buffer;
     lb.callback            = NULL;
     lb.userdata            = NULL;
-    xitk_list_append_content((XITK_WIDGET_LIST_LIST(pp_wrapper->pplugin->widget_list)), 
-     (pobj->comment = xitk_noskin_label_create(pp_wrapper->pplugin->widget_list, &lb,
-					       0, 0, FRAME_WIDTH - (26 + 6 + 6), 20, hboldfontname)));
+    pobj->comment =  xitk_noskin_label_create (pp_wrapper->pplugin->widget_list, &lb,
+      0, 0, FRAME_WIDTH - (26 + 6 + 6), 20, hboldfontname);
+    xitk_add_widget (pp_wrapper->pplugin->widget_list, pobj->comment);
 
     switch(pobj->param->type) {
     case POST_PARAM_TYPE_INT:
@@ -693,10 +680,9 @@ static void _pplugin_add_parameter_widget(_pp_wrapper_t *pp_wrapper, post_object
 	  cmb.parent_wkey       = &pp_wrapper->pplugin->widget_key;
 	  cmb.callback          = _pplugin_set_param_int;
 	  cmb.userdata          = pobj;
-	  xitk_list_append_content(XITK_WIDGET_LIST_LIST(pp_wrapper->pplugin->widget_list), 
-	   (pobj->value = xitk_noskin_combo_create(pp_wrapper->pplugin->widget_list, &cmb,
-						   0, 0, 365, NULL, NULL)));
-
+          pobj->value = xitk_noskin_combo_create (pp_wrapper->pplugin->widget_list, &cmb,
+            0, 0, 365, NULL, NULL);
+          xitk_add_widget (pp_wrapper->pplugin->widget_list, pobj->value);
 	  xitk_combo_set_select(pobj->value, *(int *)(pobj->param_data + pobj->param->offset));
 	}
 	else {
@@ -709,9 +695,9 @@ static void _pplugin_add_parameter_widget(_pp_wrapper_t *pp_wrapper, post_object
 	  ib.parent_wlist      = pp_wrapper->pplugin->widget_list;
 	  ib.callback          = _pplugin_set_param_int;
 	  ib.userdata          = pobj;
-	  xitk_list_append_content((XITK_WIDGET_LIST_LIST(pp_wrapper->pplugin->widget_list)),
-	   (pobj->value = xitk_noskin_intbox_create(pp_wrapper->pplugin->widget_list, &ib,
-						    0, 0, 50, 20, NULL, NULL, NULL)));
+          pobj->value =  xitk_noskin_intbox_create (pp_wrapper->pplugin->widget_list, &ib,
+            0, 0, 50, 20, NULL, NULL, NULL);
+          xitk_add_widget (pp_wrapper->pplugin->widget_list, pobj->value);
 	}
       }
       break;
@@ -727,9 +713,9 @@ static void _pplugin_add_parameter_widget(_pp_wrapper_t *pp_wrapper, post_object
 	ib.parent_wlist      = pp_wrapper->pplugin->widget_list;
 	ib.callback          = _pplugin_set_param_double;
 	ib.userdata          = pobj;
-	xitk_list_append_content((XITK_WIDGET_LIST_LIST(pp_wrapper->pplugin->widget_list)),
-	 (pobj->value = xitk_noskin_doublebox_create(pp_wrapper->pplugin->widget_list, &ib,
-						     0, 0, 100, 20, NULL, NULL, NULL)));
+        pobj->value =  xitk_noskin_doublebox_create (pp_wrapper->pplugin->widget_list, &ib,
+          0, 0, 100, 20, NULL, NULL, NULL);
+        xitk_add_widget (pp_wrapper->pplugin->widget_list, pobj->value);
       }
       break;
       
@@ -744,10 +730,9 @@ static void _pplugin_add_parameter_widget(_pp_wrapper_t *pp_wrapper, post_object
 	inp.max_length        = 256;
 	inp.callback          = _pplugin_set_param_char;
 	inp.userdata          = pobj;
-	xitk_list_append_content((XITK_WIDGET_LIST_LIST(pp_wrapper->pplugin->widget_list)),
-	 (pobj->value = xitk_noskin_inputtext_create(pp_wrapper->pplugin->widget_list, &inp,
-						     0, 0, 365, 20,
-						     "Black", "Black", fontname)));
+        pobj->value =  xitk_noskin_inputtext_create (pp_wrapper->pplugin->widget_list, &inp,
+          0, 0, 365, 20, "Black", "Black", fontname);
+        xitk_add_widget (pp_wrapper->pplugin->widget_list, pobj->value);
       }
       break;
       
@@ -763,10 +748,9 @@ static void _pplugin_add_parameter_widget(_pp_wrapper_t *pp_wrapper, post_object
 	cmb.parent_wkey       = &pp_wrapper->pplugin->widget_key;
 	cmb.callback          = _pplugin_set_param_stringlist;
 	cmb.userdata          = pobj;
-	xitk_list_append_content(XITK_WIDGET_LIST_LIST(pp_wrapper->pplugin->widget_list), 
-	 (pobj->value = xitk_noskin_combo_create(pp_wrapper->pplugin->widget_list, &cmb,
-						 0, 0, 365, NULL, NULL)));
-	
+        pobj->value = xitk_noskin_combo_create (pp_wrapper->pplugin->widget_list, &cmb,
+          0, 0, 365, NULL, NULL);
+        xitk_add_widget (pp_wrapper->pplugin->widget_list, pobj->value);
 	xitk_combo_set_select(pobj->value, *(int *)(pobj->param_data + pobj->param->offset));
       }
       break;
@@ -779,9 +763,8 @@ static void _pplugin_add_parameter_widget(_pp_wrapper_t *pp_wrapper, post_object
 	cb.skin_element_name = NULL;
 	cb.callback          = _pplugin_set_param_bool;
 	cb.userdata          = pobj;
-	xitk_list_append_content((XITK_WIDGET_LIST_LIST(pp_wrapper->pplugin->widget_list)),
-	  (pobj->value = xitk_noskin_checkbox_create(pp_wrapper->pplugin->widget_list, &cb,
-						     0, 0, 12, 12)));
+        pobj->value =  xitk_noskin_checkbox_create (pp_wrapper->pplugin->widget_list, &cb, 0, 0, 12, 12);
+        xitk_add_widget (pp_wrapper->pplugin->widget_list, pobj->value);
 	xitk_checkbox_set_state(pobj->value, 
 				(*(int *)(pobj->param_data + pobj->param->offset)));  
       }
@@ -795,11 +778,8 @@ static void _pplugin_change_parameter(_pp_wrapper_t *pp_wrapper, xitk_widget_t *
 
   if(pobj) {
 
-    if(pobj->value)
-      _pplugin_destroy_widget(pp_wrapper, pobj->value);
-    
-    if(pobj->comment)
-      _pplugin_destroy_widget(pp_wrapper, pobj->comment);
+    _pplugin_destroy_widget (&pobj->value);
+    _pplugin_destroy_widget (&pobj->comment);
 
     if(pobj->descr) {
       pobj->param    = pobj->descr->parameter;
@@ -844,8 +824,7 @@ static void _pplugin_destroy_only_obj(_pp_wrapper_t *pp_wrapper, post_object_t *
 
     if(pobj->properties) {
       
-      _pplugin_destroy_widget(pp_wrapper, pobj->properties);
-      pobj->properties = NULL;
+      _pplugin_destroy_widget (&pobj->properties);
       
       VFREE(pobj->param_data);
       pobj->param_data = NULL;
@@ -863,21 +842,13 @@ static void _pplugin_destroy_only_obj(_pp_wrapper_t *pp_wrapper, post_object_t *
       }
     }
     
-    if(pobj->comment) {
-      _pplugin_destroy_widget(pp_wrapper, pobj->comment);
-      pobj->comment = NULL;
-    }
-    
-    if(pobj->value) {
-      _pplugin_destroy_widget(pp_wrapper, pobj->value);
-      pobj->value = NULL;
-    }
+    _pplugin_destroy_widget (&pobj->comment);
+    _pplugin_destroy_widget (&pobj->value);
 
     if(pobj->api && pobj->help) {
       if(pp_wrapper->pplugin->help_running)
 	_pplugin_close_help(pp_wrapper, NULL,NULL);
-      _pplugin_destroy_widget(pp_wrapper, pobj->help);
-      pobj->help = NULL;
+      _pplugin_destroy_widget (&pobj->help);
     }
     
   }
@@ -887,12 +858,12 @@ static int __pplugin_retrieve_parameters(post_object_t *pobj) {
   xine_post_in_t             *input_api;
   
   if((input_api = (xine_post_in_t *) xine_post_input(pobj->post, "parameters"))) {
-    xine_post_api_t            *post_api;
-    xine_post_api_descr_t      *api_descr;
-    xine_post_api_parameter_t  *parm;
+    const xine_post_api_t            *post_api;
+    const xine_post_api_descr_t      *api_descr;
+    const xine_post_api_parameter_t  *parm;
     int                         pnum = 0;
     
-    post_api = (xine_post_api_t *) input_api->data;
+    post_api = (const xine_post_api_t *) input_api->data;
     
     api_descr = post_api->get_param_descr();
     
@@ -931,11 +902,11 @@ static void _pplugin_close_help(_pp_wrapper_t *pp_wrapper, xitk_widget_t *w, voi
   xitk_window_destroy_window(gui->imlib_data, pp_wrapper->pplugin->helpwin);
   
   pp_wrapper->pplugin->helpwin = NULL;
-  xitk_list_free((XITK_WIDGET_LIST_LIST(pp_wrapper->pplugin->help_widget_list)));
+  /* xitk_dlist_init (&pp_wrapper->pplugin->help_widget_list->list); */
     
-  XLockDisplay(gui->display);
+  gui->x_lock_display (gui->display);
   XFreeGC(gui->display, (XITK_WIDGET_LIST_GC(pp_wrapper->pplugin->help_widget_list)));
-  XUnlockDisplay(gui->display);
+  gui->x_unlock_display (gui->display);
     
   XITK_WIDGET_LIST_FREE(pp_wrapper->pplugin->help_widget_list);
 }
@@ -954,7 +925,7 @@ static void _pplugin_help_handle_event(_pp_wrapper_t *pp_wrapper, XEvent *event,
     if(xitk_get_key_pressed(event) == XK_Escape)
       _pplugin_close_help(pp_wrapper, NULL, NULL);
     else
-      gui_handle_event(event, data);
+      gui_handle_event (event, gGui);
   }
 }
 
@@ -1006,13 +977,12 @@ static void _pplugin_show_help(_pp_wrapper_t *pp_wrapper, xitk_widget_t *w, void
 
     set_window_states_start((xitk_window_get_window(pp_wrapper->pplugin->helpwin)));
 
-    XLockDisplay (gui->display);
+    gui->x_lock_display (gui->display);
     gc = XCreateGC(gui->display, 
   		 (xitk_window_get_window(pp_wrapper->pplugin->helpwin)), None, None);
-    XUnlockDisplay (gui->display);
+    gui->x_unlock_display (gui->display);
     
     pp_wrapper->pplugin->help_widget_list = xitk_widget_list_new();
-    xitk_widget_list_set(pp_wrapper->pplugin->help_widget_list, WIDGET_LIST_LIST, (xitk_list_new()));
     xitk_widget_list_set(pp_wrapper->pplugin->help_widget_list, 
                          WIDGET_LIST_WINDOW, (void *) (xitk_window_get_window(pp_wrapper->pplugin->helpwin)));
     xitk_widget_list_set(pp_wrapper->pplugin->help_widget_list, WIDGET_LIST_GC, gc);
@@ -1020,10 +990,10 @@ static void _pplugin_show_help(_pp_wrapper_t *pp_wrapper, xitk_widget_t *w, void
     xitk_window_get_window_size(pp_wrapper->pplugin->helpwin, &width, &height);
     bg = xitk_image_create_xitk_pixmap(gui->imlib_data, width, height);
   
-    XLockDisplay (gui->display);
+    gui->x_lock_display (gui->display);
     XCopyArea(gui->display, (xitk_window_get_background(pp_wrapper->pplugin->helpwin)), bg->pixmap,
   	    bg->gc, 0, 0, width, height, 0, 0);
-    XUnlockDisplay (gui->display);
+    gui->x_unlock_display (gui->display);
   
     XITK_WIDGET_INIT(&lb, gui->imlib_data);
     lb.button_type       = CLICK_BUTTON;
@@ -1033,10 +1003,10 @@ static void _pplugin_show_help(_pp_wrapper_t *pp_wrapper, xitk_widget_t *w, void
     lb.state_callback    = NULL;
     lb.userdata          = NULL;
     lb.skin_element_name = NULL;
-    xitk_list_append_content((XITK_WIDGET_LIST_LIST(pp_wrapper->pplugin->help_widget_list)), 
-     (w = xitk_noskin_labelbutton_create(pp_wrapper->pplugin->help_widget_list, 
-				       &lb, HELP_WINDOW_WIDTH - (100 + 15), HELP_WINDOW_HEIGHT - (23 + 15), 100, 23,
-  				       "Black", "Black", "White", btnfontname)));
+    w =  xitk_noskin_labelbutton_create (pp_wrapper->pplugin->help_widget_list,
+      &lb, HELP_WINDOW_WIDTH - (100 + 15), HELP_WINDOW_HEIGHT - (23 + 15), 100, 23,
+      "Black", "Black", "White", btnfontname);
+    xitk_add_widget (pp_wrapper->pplugin->help_widget_list, w);
     xitk_enable_and_show_widget(w);
   
   
@@ -1052,12 +1022,10 @@ static void _pplugin_show_help(_pp_wrapper_t *pp_wrapper, xitk_widget_t *w, void
     br.dbl_click_callback            = NULL;
     br.parent_wlist                  = pp_wrapper->pplugin->help_widget_list;
     br.userdata                      = NULL;
-    xitk_list_append_content((XITK_WIDGET_LIST_LIST(pp_wrapper->pplugin->help_widget_list)), 
-     (pp_wrapper->pplugin->help_browser = 
-      xitk_noskin_browser_create(pp_wrapper->pplugin->help_widget_list, &br,
-			       (XITK_WIDGET_LIST_GC(pp_wrapper->pplugin->help_widget_list)), 15, 34,
-  			       HELP_WINDOW_WIDTH - (30 + 16), 20,
-  			       16, br_fontname)));
+    pp_wrapper->pplugin->help_browser = xitk_noskin_browser_create (pp_wrapper->pplugin->help_widget_list,
+      &br, (XITK_WIDGET_LIST_GC(pp_wrapper->pplugin->help_widget_list)), 15, 34,
+      HELP_WINDOW_WIDTH - (30 + 16), 20, 16, br_fontname);
+    xitk_add_widget (pp_wrapper->pplugin->help_widget_list, pp_wrapper->pplugin->help_browser);
   }
 
   /* load text to the browser widget */
@@ -1170,11 +1138,9 @@ static void _pplugin_retrieve_parameters(_pp_wrapper_t *pp_wrapper, post_object_
     cmb.parent_wkey       = &pp_wrapper->pplugin->widget_key;
     cmb.callback          = (pp_wrapper == &vpp_wrapper) ? _vpplugin_change_parameter : _applugin_change_parameter;
     cmb.userdata          = pobj;
-    xitk_list_append_content(XITK_WIDGET_LIST_LIST(pp_wrapper->pplugin->widget_list), 
-     (pobj->properties = xitk_noskin_combo_create(pp_wrapper->pplugin->widget_list, 
-						  &cmb, 
-						  0, 0, 175, NULL, NULL)));
-    
+    pobj->properties = xitk_noskin_combo_create (pp_wrapper->pplugin->widget_list,
+      &cmb, 0, 0, 175, NULL, NULL);
+    xitk_add_widget (pp_wrapper->pplugin->widget_list, pobj->properties);
     xitk_combo_set_select(pobj->properties, 0);
     xitk_combo_callback_exec(pobj->properties);
 
@@ -1188,10 +1154,9 @@ static void _pplugin_retrieve_parameters(_pp_wrapper_t *pp_wrapper, post_object_
       lb.state_callback    = NULL;
       lb.userdata          = pobj;
       lb.skin_element_name = NULL;
-      xitk_list_append_content((XITK_WIDGET_LIST_LIST(pp_wrapper->pplugin->widget_list)), 
-			       (pobj->help = xitk_noskin_labelbutton_create(pp_wrapper->pplugin->widget_list, 
-									    &lb, 0, 0, 63, 20,
-									    "Black", "Black", "White", btnfontname)));
+      pobj->help =  xitk_noskin_labelbutton_create (pp_wrapper->pplugin->widget_list,
+        &lb, 0, 0, 63, 20, "Black", "Black", "White", btnfontname);
+      xitk_add_widget (pp_wrapper->pplugin->widget_list, pobj->help);
     }
   }
   else {
@@ -1204,10 +1169,9 @@ static void _pplugin_retrieve_parameters(_pp_wrapper_t *pp_wrapper, post_object_
     lb.label               = _("There is no parameter available for this plugin.");
     lb.callback            = NULL;
     lb.userdata            = NULL;
-    xitk_list_append_content((XITK_WIDGET_LIST_LIST(pp_wrapper->pplugin->widget_list)), 
-     (pobj->comment = xitk_noskin_label_create(pp_wrapper->pplugin->widget_list, &lb,
-					       0, 0, FRAME_WIDTH - (26 + 6 + 6), 20, hboldfontname)));
-    
+    pobj->comment =  xitk_noskin_label_create (pp_wrapper->pplugin->widget_list, &lb,
+      0, 0, FRAME_WIDTH - (26 + 6 + 6), 20, hboldfontname);
+    xitk_add_widget (pp_wrapper->pplugin->widget_list, pobj->comment);
     pobj->properties = NULL;
   }
 }
@@ -1239,10 +1203,10 @@ static void _pplugin_select_filter(_pp_wrapper_t *pp_wrapper, xitk_widget_t *w,
           xine_post_dispose (__xineui_global_xine_instance, p[n]->post);
           p[n]->post = NULL;
         }
-        _pplugin_destroy_widget (pp_wrapper, p[n]->plugins);
-        _pplugin_destroy_widget (pp_wrapper, p[n]->frame);
-        _pplugin_destroy_widget (pp_wrapper, p[n]->up);
-        _pplugin_destroy_widget (pp_wrapper, p[n]->down);
+        _pplugin_destroy_widget (&p[n]->plugins);
+        _pplugin_destroy_widget (&p[n]->frame);
+        _pplugin_destroy_widget (&p[n]->up);
+        _pplugin_destroy_widget (&p[n]->down);
         VFREE (p[n]);
         p[n] = NULL;
         n++;
@@ -1356,13 +1320,13 @@ static post_object_t *_pplugin_create_filter_object (_pp_wrapper_t *pp_wrapper)
 
   image = xitk_image_create_image(gui->imlib_data, FRAME_WIDTH + 1, FRAME_HEIGHT + 1);
 
-  XLockDisplay(gui->display);
+  gui->x_lock_display (gui->display);
   XSetForeground(gui->display, (XITK_WIDGET_LIST_GC(pp_wrapper->pplugin->widget_list)),
 		 xitk_get_pixel_color_gray(gui->imlib_data));
   XFillRectangle(gui->display, image->image->pixmap,
 		 (XITK_WIDGET_LIST_GC(pp_wrapper->pplugin->widget_list)),
 		 0, 0, image->width, image->height);
-  XUnlockDisplay(gui->display);
+  gui->x_unlock_display (gui->display);
 
   /* Some decorations */
   draw_outter_frame(gui->imlib_data, image->image, NULL, NULL,
@@ -1386,8 +1350,9 @@ static post_object_t *_pplugin_create_filter_object (_pp_wrapper_t *pp_wrapper)
 
   XITK_WIDGET_INIT(&im, gui->imlib_data);
   im.skin_element_name = NULL;
-  xitk_list_append_content((XITK_WIDGET_LIST_LIST(pp_wrapper->pplugin->widget_list)),
-   (pobj->frame = xitk_noskin_image_create(pp_wrapper->pplugin->widget_list, &im, image, pobj->x - 5, pobj->y - 5)));
+  pobj->frame =  xitk_noskin_image_create (pp_wrapper->pplugin->widget_list, &im, image, pobj->x - 5, pobj->y - 5);
+  xitk_add_widget (pp_wrapper->pplugin->widget_list, pobj->frame);
+
   DISABLE_ME(pobj->frame);
   
   XITK_WIDGET_INIT(&cmb, gui->imlib_data);
@@ -1398,11 +1363,10 @@ static post_object_t *_pplugin_create_filter_object (_pp_wrapper_t *pp_wrapper)
   cmb.parent_wkey       = &pp_wrapper->pplugin->widget_key;
   cmb.callback          = (pp_wrapper == &vpp_wrapper) ? _vpplugin_select_filter : _applugin_select_filter;
   cmb.userdata          = pobj;
-  xitk_list_append_content(XITK_WIDGET_LIST_LIST(pp_wrapper->pplugin->widget_list), 
-   (pobj->plugins = xitk_noskin_combo_create(pp_wrapper->pplugin->widget_list, 
-					     &cmb, 0, 0, 175, NULL, NULL)));
+  pobj->plugins = xitk_noskin_combo_create (pp_wrapper->pplugin->widget_list,
+    &cmb, 0, 0, 175, NULL, NULL);
+  xitk_add_widget (pp_wrapper->pplugin->widget_list, pobj->plugins);
   DISABLE_ME(pobj->plugins);
-  
   xitk_combo_set_select(pobj->plugins, 0);
 
   {
@@ -1412,9 +1376,8 @@ static post_object_t *_pplugin_create_filter_object (_pp_wrapper_t *pp_wrapper)
     b.skin_element_name = NULL;
     b.callback          = (pp_wrapper == &vpp_wrapper) ? _vpplugin_move_up : _applugin_move_up;
     b.userdata          = pobj;
-    xitk_list_append_content(XITK_WIDGET_LIST_LIST(pp_wrapper->pplugin->widget_list), 
-     (pobj->up = xitk_noskin_button_create(pp_wrapper->pplugin->widget_list, &b, 
-					   0, 0, 17, 17)));
+    pobj->up = xitk_noskin_button_create (pp_wrapper->pplugin->widget_list, &b, 0, 0, 17, 17);
+    xitk_add_widget (pp_wrapper->pplugin->widget_list, pobj->up);
     DISABLE_ME(pobj->up);
 
     if((bimage = xitk_get_widget_foreground_skin(pobj->up)) != NULL)
@@ -1423,9 +1386,8 @@ static post_object_t *_pplugin_create_filter_object (_pp_wrapper_t *pp_wrapper)
     b.skin_element_name = NULL;
     b.callback          = (pp_wrapper == &vpp_wrapper) ? _vpplugin_move_down : _applugin_move_down;
     b.userdata          = pobj;
-    xitk_list_append_content(XITK_WIDGET_LIST_LIST(pp_wrapper->pplugin->widget_list), 
-     (pobj->down = xitk_noskin_button_create(pp_wrapper->pplugin->widget_list, &b, 
-					     0, 0, 17, 17)));
+    pobj->down = xitk_noskin_button_create (pp_wrapper->pplugin->widget_list, &b, 0, 0, 17, 17);
+    xitk_add_widget (pp_wrapper->pplugin->widget_list, pobj->down);
     DISABLE_ME(pobj->down);
 
     if((bimage = xitk_get_widget_foreground_skin(pobj->down)) != NULL)
@@ -1636,10 +1598,10 @@ static void pplugin_exit(_pp_wrapper_t *pp_wrapper, xitk_widget_t *w, void *data
       post_object_t **p = pp_wrapper->pplugin->post_objects;
       while (*p) {
         _pplugin_destroy_only_obj (pp_wrapper, *p);
-        _pplugin_destroy_widget (pp_wrapper, (*p)->plugins);
-        _pplugin_destroy_widget (pp_wrapper, (*p)->frame);
-        _pplugin_destroy_widget (pp_wrapper, (*p)->up);
-        _pplugin_destroy_widget (pp_wrapper, (*p)->down);
+        _pplugin_destroy_widget (&(*p)->plugins);
+        _pplugin_destroy_widget (&(*p)->frame);
+        _pplugin_destroy_widget (&(*p)->up);
+        _pplugin_destroy_widget (&(*p)->down);
         VFREE (*p);
         *p = NULL;
         p++;
@@ -1658,11 +1620,11 @@ static void pplugin_exit(_pp_wrapper_t *pp_wrapper, xitk_widget_t *w, void *data
     xitk_window_destroy_window(gui->imlib_data, pp_wrapper->pplugin->xwin);
 
     pp_wrapper->pplugin->xwin = NULL;
-    xitk_list_free((XITK_WIDGET_LIST_LIST(pp_wrapper->pplugin->widget_list)));
+    /* xitk_dlist_init (&pp_wrapper->pplugin->widget_list->list); */
     
-    XLockDisplay(gui->display);
+    gui->x_lock_display (gui->display);
     XFreeGC(gui->display, (XITK_WIDGET_LIST_GC(pp_wrapper->pplugin->widget_list)));
-    XUnlockDisplay(gui->display);
+    gui->x_unlock_display (gui->display);
     
     XITK_WIDGET_LIST_FREE(pp_wrapper->pplugin->widget_list);
    
@@ -1750,7 +1712,7 @@ static void _pplugin_handle_event(_pp_wrapper_t *pp_wrapper, XEvent *event, void
 	pplugin_exit(pp_wrapper, NULL, NULL);
 	return;
       }
-      gui_handle_event(event, data);
+      gui_handle_event (event, gGui);
     }
     break;
   }
@@ -1969,13 +1931,12 @@ static void pplugin_panel(_pp_wrapper_t *pp_wrapper) {
   
   set_window_states_start((xitk_window_get_window(pp_wrapper->pplugin->xwin)));
 
-  XLockDisplay (gui->display);
+  gui->x_lock_display (gui->display);
   gc = XCreateGC(gui->display, 
 		 (xitk_window_get_window(pp_wrapper->pplugin->xwin)), None, None);
-  XUnlockDisplay (gui->display);
+  gui->x_unlock_display (gui->display);
   
   pp_wrapper->pplugin->widget_list = xitk_widget_list_new();
-  xitk_widget_list_set(pp_wrapper->pplugin->widget_list, WIDGET_LIST_LIST, (xitk_list_new()));
   xitk_widget_list_set(pp_wrapper->pplugin->widget_list, 
 		       WIDGET_LIST_WINDOW, (void *) (xitk_window_get_window(pp_wrapper->pplugin->xwin)));
   xitk_widget_list_set(pp_wrapper->pplugin->widget_list, WIDGET_LIST_GC, gc);
@@ -1987,10 +1948,10 @@ static void pplugin_panel(_pp_wrapper_t *pp_wrapper) {
   xitk_window_get_window_size(pp_wrapper->pplugin->xwin, &width, &height);
   bg = xitk_image_create_xitk_pixmap(gui->imlib_data, width, height);
 
-  XLockDisplay (gui->display);
+  gui->x_lock_display (gui->display);
   XCopyArea(gui->display, (xitk_window_get_background(pp_wrapper->pplugin->xwin)), bg->pixmap,
 	    bg->gc, 0, 0, width, height, 0, 0);
-  XUnlockDisplay (gui->display);
+  gui->x_unlock_display (gui->display);
   
   XITK_WIDGET_INIT(&sl, gui->imlib_data);
   
@@ -2002,11 +1963,9 @@ static void pplugin_panel(_pp_wrapper_t *pp_wrapper) {
   sl.userdata                 = NULL;
   sl.motion_callback          = (pp_wrapper == &vpp_wrapper) ? _vpplugin_nextprev : _applugin_nextprev;
   sl.motion_userdata          = NULL;
-  xitk_list_append_content((XITK_WIDGET_LIST_LIST(pp_wrapper->pplugin->widget_list)),
-   (pp_wrapper->pplugin->slider = xitk_noskin_slider_create(pp_wrapper->pplugin->widget_list, &sl,
-							    (WINDOW_WIDTH - (16 + 15)), 34,
-							    16, (MAX_DISPLAY_FILTERS * (FRAME_HEIGHT + 4) - 4),
-							    XITK_VSLIDER)));
+  pp_wrapper->pplugin->slider =  xitk_noskin_slider_create (pp_wrapper->pplugin->widget_list, &sl,
+    (WINDOW_WIDTH - (16 + 15)), 34, 16, (MAX_DISPLAY_FILTERS * (FRAME_HEIGHT + 4) - 4), XITK_VSLIDER);
+  xitk_add_widget (pp_wrapper->pplugin->widget_list, pp_wrapper->pplugin->slider);
 
   y = WINDOW_HEIGHT - (23 + 15);
   x = 15;
@@ -2018,10 +1977,9 @@ static void pplugin_panel(_pp_wrapper_t *pp_wrapper) {
   lb.state_callback    = (pp_wrapper == &vpp_wrapper) ? _vpplugin_enability : _applugin_enability;
   lb.userdata          = NULL;
   lb.skin_element_name = NULL;
-  xitk_list_append_content((XITK_WIDGET_LIST_LIST(pp_wrapper->pplugin->widget_list)), 
-   (pp_wrapper->pplugin->enable = xitk_noskin_labelbutton_create(pp_wrapper->pplugin->widget_list, 
-								 &lb, x, y, 100, 23,
-								 "Black", "Black", "White", btnfontname)));
+  pp_wrapper->pplugin->enable =  xitk_noskin_labelbutton_create (pp_wrapper->pplugin->widget_list,
+    &lb, x, y, 100, 23, "Black", "Black", "White", btnfontname);
+  xitk_add_widget (pp_wrapper->pplugin->widget_list, pp_wrapper->pplugin->enable);
 
   xitk_labelbutton_set_state(pp_wrapper->pplugin->enable, (pp_wrapper == &vpp_wrapper) ? gui->post_video_enable : gui->post_audio_enable);
   xitk_enable_and_show_widget(pp_wrapper->pplugin->enable);
@@ -2036,10 +1994,9 @@ static void pplugin_panel(_pp_wrapper_t *pp_wrapper) {
   lb.state_callback    = NULL;
   lb.userdata          = NULL;
   lb.skin_element_name = NULL;
-  xitk_list_append_content((XITK_WIDGET_LIST_LIST(pp_wrapper->pplugin->widget_list)), 
-   (w = xitk_noskin_labelbutton_create(pp_wrapper->pplugin->widget_list, 
-				       &lb, x, y, 100, 23,
-				       "Black", "Black", "White", btnfontname)));
+  w =  xitk_noskin_labelbutton_create (pp_wrapper->pplugin->widget_list,
+    &lb, x, y, 100, 23, "Black", "Black", "White", btnfontname);
+  xitk_add_widget (pp_wrapper->pplugin->widget_list, w);
   xitk_enable_and_show_widget(w);
   */
 
@@ -2052,10 +2009,9 @@ static void pplugin_panel(_pp_wrapper_t *pp_wrapper) {
   lb.state_callback    = NULL;
   lb.userdata          = NULL;
   lb.skin_element_name = NULL;
-  xitk_list_append_content((XITK_WIDGET_LIST_LIST(pp_wrapper->pplugin->widget_list)), 
-   (w = xitk_noskin_labelbutton_create(pp_wrapper->pplugin->widget_list, 
-				       &lb, x, y, 100, 23,
-				       "Black", "Black", "White", btnfontname)));
+  w =  xitk_noskin_labelbutton_create (pp_wrapper->pplugin->widget_list,
+    &lb, x, y, 100, 23, "Black", "Black", "White", btnfontname);
+  xitk_add_widget (pp_wrapper->pplugin->widget_list, w);
   xitk_enable_and_show_widget(w);
 
   _pplugin_get_plugins(pp_wrapper);
@@ -2329,7 +2285,7 @@ void post_deinterlace_init(const char *deinterlace_post) {
 					    "when deinterlace is used (plugin separator is ';')."),
 					  CONFIG_LEVEL_ADV,
 					  post_deinterlace_plugin_cb,
-					  CONFIG_NO_DATA);
+					  gGui);
   if((posts = pplugin_parse_and_load(0, (deinterlace_post && strlen(deinterlace_post)) ? 
 				     deinterlace_post : gui->deinterlace_plugin, &num))) {
     gui->deinterlace_elements     = posts;
