@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2009 the xine project
+ * Copyright (C) 2000-2017 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -72,8 +72,8 @@ int xitk_config_get_barstyle_feature(xitk_config_t *xtcf);
 int xitk_config_get_checkstyle_feature(xitk_config_t *xtcf);
 int xitk_config_get_shm_feature(xitk_config_t *xtcf);
 int xitk_config_get_cursors_feature(xitk_config_t *xtcf);
-char *xitk_config_get_system_font(xitk_config_t *xtcf);
-char *xitk_config_get_default_font(xitk_config_t *xtcf);
+const char *xitk_config_get_system_font(xitk_config_t *xtcf);
+const char *xitk_config_get_default_font(xitk_config_t *xtcf);
 int xitk_config_get_xmb_enability(xitk_config_t *xtcf);
 void xitk_config_set_xmb_enability(xitk_config_t *xtcf, int value);
 int xitk_config_get_black_color(xitk_config_t *xtcf);
