@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2009 the xine project
+ * Copyright (C) 2000-2019 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -46,25 +46,22 @@ typedef struct {
 } xitk_font_cache_item_t;
 
 typedef struct {
-  unsigned int             number;     /* number of instances of this font */
+  xitk_dnode_t             node;
   xitk_font_t             *font;       /* remembered loaded font */
+  unsigned int             number;     /* number of instances of this font */
 } xitk_font_list_item_t;
 
-typedef struct {
+struct xitk_font_cache_s {
   size_t                   n;
   size_t                   nlist;      /* number of fonts in the cache and in the list */
 
   unsigned int             life;       /* life counter for LRU */
-  xitk_list_t             *loaded;     /* list of loaded fonts */
+  xitk_dlist_t             loaded;     /* list of loaded fonts */
   pthread_mutex_t          mutex;      /* protecting mutex */
   xitk_font_cache_item_t   items[XITK_CACHE_SIZE]; /* cache */
 
-  pthread_mutex_t          xr_mutex;   /* protecting mutex */
   xitk_recode_t           *xr;         /* text recoding */
-} xitk_font_cache_t;
-
-/* global font cache */
-static xitk_font_cache_t cache;
+};
 
 #if defined(WITH_XMB) && !defined(WITH_XFT)
 /*
@@ -157,7 +154,7 @@ static const char * xitk_font_core_string_to_xft(char *new_name, size_t new_name
 /*
  * XCreateFontSet requires font name starting with '-'
  */
-static char *xitk_font_right_name(char *name) {
+static char *xitk_font_right_name(const char *name) {
   char *right_name;
 
   ABORT_IF_NULL(name);
@@ -182,7 +179,7 @@ static char *xitk_font_right_name(char *name) {
 /*
  * load the font and returns status
  */
-static int xitk_font_load_one(Display *display, char *font, xitk_font_t *xtfs) {
+static int xitk_font_load_one(Display *display, const char *font, xitk_font_t *xtfs) {
   int    ok;
 #ifdef WITH_XFT
 #else
@@ -194,10 +191,10 @@ static int xitk_font_load_one(Display *display, char *font, xitk_font_t *xtfs) {
 
   if(xitk_get_xmb_enability()) {
     right_name = xitk_font_right_name(font);
-    XLOCK(display);
+    XLOCK (xitk_x_lock_display, display);
     xtfs->fontset = XCreateFontSet(display, right_name, &missing, &count, &def);
     ok = !xitk_font_guess_error(xtfs->fontset, right_name, missing, count);
-    XUNLOCK(display);
+    XUNLOCK (xitk_x_unlock_display, display);
     free(right_name);
   }
   else
@@ -205,14 +202,14 @@ static int xitk_font_load_one(Display *display, char *font, xitk_font_t *xtfs) {
 #endif
   {
     char new_name[255];
-    XLOCK(display);
+    XLOCK (xitk_x_lock_display, display);
 #ifdef WITH_XFT
     xtfs->font = XftFontOpenName( display, DefaultScreen(display),
                                   xitk_font_core_string_to_xft(new_name, sizeof(new_name), font));
 #else
     xtfs->font = XLoadQueryFont(display, font);
 #endif
-    XUNLOCK(display);
+    XUNLOCK (xitk_x_unlock_display, display);
     ok = (xtfs->font != NULL);
   }
 
@@ -226,7 +223,7 @@ static int xitk_font_load_one(Display *display, char *font, xitk_font_t *xtfs) {
  * unload the font
  */
 static void xitk_font_unload_one(xitk_font_t *xtfs) {
-  XLOCK(xtfs->display);
+  XLOCK (xitk_x_lock_display, xtfs->display);
 #ifndef WITH_XFT 
 #ifdef WITH_XMB
   if(xitk_get_xmb_enability())
@@ -237,87 +234,91 @@ static void xitk_font_unload_one(xitk_font_t *xtfs) {
 #else
     XftFontClose( xtfs->display, xtfs->font );
 #endif
-  XUNLOCK(xtfs->display);
+  XUNLOCK (xitk_x_unlock_display, xtfs->display);
   free(xtfs->name);
 }
 
 /*
  * init font cache subsystem
  */
-void xitk_font_cache_init(void) {
-  
-  cache.n      = 0;
-  cache.life   = 0;
-  cache.loaded = xitk_list_new();	
+void xitk_font_cache_init (void) {
+  xitk_t *xitk = gXitk;
 
-  pthread_mutex_init(&cache.mutex, NULL);
-  pthread_mutex_init(&cache.xr_mutex, NULL);
-  
+  xitk->font_cache = malloc (sizeof (*xitk->font_cache));
+
+  if (xitk->font_cache) {
+    xitk->font_cache->n    = 0;
+    xitk->font_cache->life = 0;
+    xitk_dlist_init (&xitk->font_cache->loaded);
+    pthread_mutex_init (&xitk->font_cache->mutex, NULL);
 #ifdef WITH_XFT
-  cache.xr = xitk_recode_init(NULL, "UTF-8");
+    xitk->font_cache->xr = xitk_recode_init (NULL, "UTF-8", 1);
 #else
-  cache.xr = NULL;
+    xitk->font_cache->xr = NULL;
 #endif
+  }
 }
 
 /*
  * destroy font cache subsystem
  */
 void xitk_font_cache_done(void) {
+  xitk_t *xitk = gXitk;
   size_t       i;
   xitk_font_t *xtfs;
 
-  for (i = 0; i < cache.n; i++) {
-    xtfs = cache.items[i].font;
+  for (i = 0; i < xitk->font_cache->n; i++) {
+    xtfs = xitk->font_cache->items[i].font;
 
     xitk_font_unload_one(xtfs);
  
     free(xtfs);
   }
-  cache.n = 0;
+  xitk->font_cache->n = 0;
 
-#ifdef DEBUG
+#ifdef XITK_DEBUG
   {
-    xitk_font_list_item_t *item;
-    
-    if(!xitk_list_is_empty(cache.loaded)) {
-      XITK_WARNING("%s(): list of loaded font isn't empty:\n", __FUNCTION__);
-      item = xitk_list_first_content(cache.loaded);
-      while(item) {
-	printf("  %s (%u)\n", item->font->name, item->number);
-	item = xitk_list_next_content(cache.loaded);
-      }
+    xitk_font_list_item_t *item = (xitk_font_list_item_t *)cached.loaded.head.next;
+    if (item->node.next) {
+      XITK_WARNING ("%s(): list of loaded font isn't empty:\n", __FUNCTION__);
+      do {
+        printf ("  %s (%u)\n", item->font->name, item->number);
+        item = (xitk_font_list_item_t *)item->node.next;
+      } while (item->node.next);
     }
   }
 #endif
 
-  xitk_list_free(cache.loaded);
-  pthread_mutex_destroy(&cache.mutex);
-  pthread_mutex_destroy(&cache.xr_mutex);
+  xitk_dlist_clear (&xitk->font_cache->loaded);
+  pthread_mutex_destroy(&xitk->font_cache->mutex);
 
-  xitk_recode_done(cache.xr);
+  xitk_recode_done(xitk->font_cache->xr);
+
+  free (xitk->font_cache);
+  xitk->font_cache = NULL;
 }
 
 /* 
  * get index of oldest item in the cache 
  */
 static size_t xitk_cache_get_oldest(void) {
+  xitk_t *xitk = gXitk;
   size_t          i, oldest;
   unsigned long   lru;
 
-  if(!cache.n) {
+  if(!xitk->font_cache->n) {
     fprintf(stderr, "%s(%d): cache.n == 0. Aborting.\n", __FUNCTION__, __LINE__);
     abort();
   }
   
   i      = 0;
   oldest = 0;
-  lru    = cache.items[0].lru;
+  lru    = xitk->font_cache->items[0].lru;
 
-  for(i = 1; i < cache.n; i++) {
-    if(cache.items[i].lru < lru) {
+  for(i = 1; i < xitk->font_cache->n; i++) {
+    if(xitk->font_cache->items[i].lru < lru) {
       oldest = i;
-      lru = cache.items[i].lru;
+      lru = xitk->font_cache->items[i].lru;
     }
   }
 
@@ -328,30 +329,32 @@ static size_t xitk_cache_get_oldest(void) {
  * place item into cache and adjust all counters for LRU 
  */
 static void xitk_cache_insert_final(size_t pos, xitk_font_t *font) {
+  xitk_t *xitk = gXitk;
   size_t i;
 
-  cache.items[pos].lru  = cache.life;
-  cache.items[pos].font = font;
+  xitk->font_cache->items[pos].lru  = xitk->font_cache->life;
+  xitk->font_cache->items[pos].font = font;
 
-  if(cache.life + 1 == 0) {
-    cache.life >>= 1;
-    for(i = 0; i < cache.n; i++) 
-      cache.items[i].lru >>= 1;
+  if(xitk->font_cache->life + 1 == 0) {
+    xitk->font_cache->life >>= 1;
+    for(i = 0; i < xitk->font_cache->n; i++) 
+      xitk->font_cache->items[i].lru >>= 1;
   } 
   else 
-    cache.life++;
+    xitk->font_cache->life++;
 }
 
 /* 
  * insert item into sorted cache, it shifts part of cache to move gap
  */
 static void xitk_cache_insert_into(size_t gap, int step, xitk_font_t *font) {
+  xitk_t *xitk = gXitk;
   size_t i;
 
   i = gap;
-  while (((step == -1 && i > 0) || (step == 1 && i < cache.n - 1)) 
-	 && strcmp(font->name, cache.items[i + step].font->name) * step > 0) {
-    cache.items[i] = cache.items[i + step];
+  while (((step == -1 && i > 0) || (step == 1 && i < xitk->font_cache->n - 1)) 
+	 && strcmp(font->name, xitk->font_cache->items[i + step].font->name) * step > 0) {
+    xitk->font_cache->items[i] = xitk->font_cache->items[i + step];
     i += step;
   }
   xitk_cache_insert_final(i, font);
@@ -361,24 +364,25 @@ static void xitk_cache_insert_into(size_t gap, int step, xitk_font_t *font) {
  * add new item into cache, if it's full the oldest item will be released
  */
 static void xitk_cache_add_item(xitk_font_t *font) {
+  xitk_t *xitk = gXitk;
   size_t       oldest;
   int          cmp;
   xitk_font_t *xtfs;
 	
-  if(cache.n < XITK_CACHE_SIZE) {
+  if(xitk->font_cache->n < XITK_CACHE_SIZE) {
     /* cache is not full */
-    cache.n++;
-    xitk_cache_insert_into(cache.n - 1, -1, font);
+    xitk->font_cache->n++;
+    xitk_cache_insert_into(xitk->font_cache->n - 1, -1, font);
   } 
   else {
     /* cache is full */
     /* free the oldest item */
     oldest = xitk_cache_get_oldest();
-    cmp    = strcmp(font->name, cache.items[oldest].font->name);
-    xtfs   = cache.items[oldest].font;
+    cmp    = strcmp(font->name, xitk->font_cache->items[oldest].font->name);
+    xtfs   = xitk->font_cache->items[oldest].font;
 
 #ifdef LOG
-    printf("xiTK: dropped \"%s\", list: %zu, cache: %zu\n", xtfs->name, cache.nlist, cache.n);
+    printf("xiTK: dropped \"%s\", list: %zu, cache: %zu\n", xtfs->name, xitk->font_cache->nlist, xitk->font_cache->n);
 #endif
 
     xitk_font_unload_one(xtfs);
@@ -403,13 +407,14 @@ static void xitk_cache_add_item(xitk_font_t *font) {
  * remove the item in 'pos' and returns its data
  */
 static xitk_font_t *xitk_cache_remove_item(size_t pos) {
+  xitk_t *xitk = gXitk;
   xitk_font_t *xtfs;
   size_t       i;
 
-  xtfs = cache.items[pos].font;
-  for (i = pos; i < cache.n - 1; i++)
-    cache.items[i] = cache.items[i + 1];
-  cache.n--;
+  xtfs = xitk->font_cache->items[pos].font;
+  for (i = pos; i < xitk->font_cache->n - 1; i++)
+    xitk->font_cache->items[i] = xitk->font_cache->items[i + 1];
+  xitk->font_cache->n--;
 
   return xtfs;
 }
@@ -417,21 +422,22 @@ static xitk_font_t *xitk_cache_remove_item(size_t pos) {
 /* 
  * search the font of given display in the cache, remove it from the cache
  */
-static xitk_font_t *xitk_cache_take_item(Display *display, char *name) {
+static xitk_font_t *xitk_cache_take_item(Display *display, const char *name) {
+  xitk_t *xitk = gXitk;
   int     left, right;
   size_t  i, j;
   int     cmp;
 
-  if(!cache.n)
+  if(!xitk->font_cache->n)
     return NULL;
 
   /* binary search in the cache */
   left  = 0;
-  right = cache.n - 1;
+  right = xitk->font_cache->n - 1;
 
   while(right >= left) {
     i = (left + right) >> 1;
-    cmp = strcmp(name, cache.items[i].font->name);
+    cmp = strcmp(name, xitk->font_cache->items[i].font->name);
 
     if(cmp < 0)
       right = i - 1;
@@ -442,24 +448,24 @@ static xitk_font_t *xitk_cache_take_item(Display *display, char *name) {
       /* forward */
       j = i;
       while(1) {
-	if(cache.items[j].font->display == display)
+	if(xitk->font_cache->items[j].font->display == display)
 	  return xitk_cache_remove_item(j);
 
 	if(!j) 
 	  break;
 
-	cmp = strcmp(name, cache.items[--j].font->name);
+	cmp = strcmp(name, xitk->font_cache->items[--j].font->name);
       }
       /* backward */
       j = i + 1;
 
-      while(j < cache.n) {
-	cmp = strcmp(name, cache.items[j].font->name);
+      while(j < xitk->font_cache->n) {
+	cmp = strcmp(name, xitk->font_cache->items[j].font->name);
 
 	if(cmp != 0)
 	  return NULL;
 
-	if(cache.items[j].font->display == display)
+	if(xitk->font_cache->items[j].font->display == display)
 	  return xitk_cache_remove_item(j);
 
 	j++;
@@ -477,50 +483,35 @@ static xitk_font_t *xitk_cache_take_item(Display *display, char *name) {
 /*
  * search the font in the list of loaded fonts
  */
-static xitk_font_list_item_t *cache_get_from_list(Display *display, char *name) {
-  xitk_font_list_item_t *item;
-
-  item = xitk_list_first_content(cache.loaded);
-  if(!item)
-    return NULL;
-
-  do {
-    if((strcmp(name, item->font->name) == 0) && (display == item->font->display)) 
+static xitk_font_list_item_t *cache_get_from_list(Display *display, const char *name) {
+  xitk_t *xitk = gXitk;
+  xitk_font_list_item_t *item = (xitk_font_list_item_t *)xitk->font_cache->loaded.head.next;
+  while (item->node.next) {
+    if ((strcmp(name, item->font->name) == 0) && (display == item->font->display))
       return item;
-
-    item = xitk_list_next_content(cache.loaded);
-  } while(item);
-
+    item = (xitk_font_list_item_t *)item->node.next;
+  }
   return NULL;
 }
 
-static char *xitk_cache_recode(const char *text)
-{
-  char *result;
-  pthread_mutex_lock(&cache.xr_mutex);
-  result = xitk_recode(cache.xr, text);
-  pthread_mutex_unlock(&cache.xr_mutex);
-  return result;
-}
-
-
 /*
  *
  */
-xitk_font_t *xitk_font_load_font(Display *display, char *font) {
+xitk_font_t *xitk_font_load_font(Display *display, const char *font) {
+  xitk_t *xitk = gXitk;
   xitk_font_t           *xtfs;
   xitk_font_list_item_t *list_item;
   
   ABORT_IF_NULL(display);
   ABORT_IF_NULL(font);
-  pthread_mutex_lock(&cache.mutex);
+  pthread_mutex_lock(&xitk->font_cache->mutex);
   
   /* quick search in the cache of unloaded fonts */
   if((xtfs = xitk_cache_take_item(display, font)) == NULL) {
     /* search in the list of loaded fonts */
     if((list_item = cache_get_from_list(display, font)) != NULL) {
       list_item->number++;
-      pthread_mutex_unlock(&cache.mutex);
+      pthread_mutex_unlock(&xitk->font_cache->mutex);
       return list_item->font;
     }
   }
@@ -530,14 +521,14 @@ xitk_font_t *xitk_font_load_font(Display *display, char *font) {
     xtfs = (xitk_font_t *) xitk_xmalloc(sizeof(xitk_font_t));
 
     if(!xitk_font_load_one(display, font, xtfs)) {
-      char *fdname = xitk_get_default_font();
+      const char *fdname = xitk_get_default_font();
 
       if(!fdname || !xitk_font_load_one(display, fdname, xtfs)) {
-        char *fsname = xitk_get_system_font();
+        const char *fsname = xitk_get_system_font();
  	if(!xitk_font_load_one(display, fsname, xtfs)) {
 	  XITK_WARNING("loading font \"%s\" failed, default and system fonts \"%s\" and \"%s\" failed too\n", font, fdname, fsname);
 	  free(xtfs);
-	  pthread_mutex_unlock(&cache.mutex);
+	  pthread_mutex_unlock(&xitk->font_cache->mutex);
 
 	  /* Maybe broken XMB support */
 	  if(xitk_get_xmb_enability()) {
@@ -559,7 +550,7 @@ xitk_font_t *xitk_font_load_font(Display *display, char *font) {
     xtfs->display = display;
 
 #ifdef LOG
-    printf("xiTK: loaded new \"%s\", list: %zu, cache: %zu\n", xtfs->name, cache.nlist, cache.n);
+    printf("xiTK: loaded new \"%s\", list: %zu, cache: %zu\n", xtfs->name, xitk->font_cache->nlist, xitk->font_cache->n);
 #endif
   }
 
@@ -567,10 +558,10 @@ xitk_font_t *xitk_font_load_font(Display *display, char *font) {
   list_item         = xitk_xmalloc(sizeof(xitk_font_list_item_t));
   list_item->font   = xtfs;
   list_item->number = 1;
-  xitk_list_append_content(cache.loaded, list_item);
-  cache.nlist++;
+  xitk_dlist_add_tail (&xitk->font_cache->loaded, &list_item->node);
+  xitk->font_cache->nlist++;
 
-  pthread_mutex_unlock(&cache.mutex);
+  pthread_mutex_unlock(&xitk->font_cache->mutex);
 
   return xtfs;
 }
@@ -579,6 +570,7 @@ xitk_font_t *xitk_font_load_font(Display *display, char *font) {
  *
  */
 void xitk_font_unload_font(xitk_font_t *xtfs) {
+  xitk_t *xitk = gXitk;
   xitk_font_list_item_t *item;
   
   ABORT_IF_NULL(xtfs);
@@ -592,20 +584,20 @@ void xitk_font_unload_font(xitk_font_t *xtfs) {
 #endif
     ABORT_IF_NULL(xtfs->font);
 
-  pthread_mutex_lock(&cache.mutex);
+  pthread_mutex_lock(&xitk->font_cache->mutex);
   /* search the font in the list */
   item = cache_get_from_list(xtfs->display, xtfs->name);
   /* it must be there */
   ABORT_IF_NULL(item);
 
   if(--item->number == 0) {
-    xitk_list_delete_current(cache.loaded);
-    cache.nlist--;
+    xitk_dnode_remove (&item->node);
+    xitk->font_cache->nlist--;
     xitk_cache_add_item(item->font);
     free(item);
   }
 
-  pthread_mutex_unlock(&cache.mutex);
+  pthread_mutex_unlock(&xitk->font_cache->mutex);
 }
 
 /*
@@ -615,13 +607,14 @@ void xitk_font_draw_string(xitk_font_t *xtfs, Pixmap pix, GC gc,
 			   int x, int y, const char *text, 
 			   size_t nbytes) {
 
+  xitk_t *xitk = gXitk;
 #ifdef DEBUG
   if (nbytes > strlen(text) + 1) {
     XITK_WARNING("draw: %zu > %zu\n", nbytes, strlen(text));
   }
 #endif
   
-  XLOCK(xtfs->display);  
+  XLOCK (xitk_x_lock_display, xtfs->display);
 #ifndef WITH_XFT
 # ifdef WITH_XMB
   if(xitk_get_xmb_enability())
@@ -639,7 +632,16 @@ void xitk_font_draw_string(xitk_font_t *xtfs, Pixmap pix, GC gc,
     XRenderColor  xr_color;
     XftColor      xcolor;
     XftDraw      *xft_draw;
-    char         *encoded_text = xitk_cache_recode(text);
+    char          buf[2048];
+    xitk_recode_string_t rs;
+
+    rs.src = text;
+    rs.ssize = strlen (text);
+    if ((size_t)nbytes < rs.ssize)
+      rs.ssize = nbytes;
+    rs.buf = buf;
+    rs.bsize = sizeof (buf);
+    xitk_recode2_do (xitk->font_cache->xr, &rs);
 
     XGetGCValues(xtfs->display, gc, GCForeground, &gc_color);
     paint_color.pixel = gc_color.foreground;
@@ -650,14 +652,14 @@ void xitk_font_draw_string(xitk_font_t *xtfs, Pixmap pix, GC gc,
     xr_color.alpha = (short)-1;
     xft_draw       = XftDrawCreate(xtfs->display, pix, visual, colormap);
     XftColorAllocValue(xtfs->display, visual, colormap, &xr_color, &xcolor);
-    XftDrawStringUtf8(xft_draw, &xcolor, xtfs->font, x, y, (FcChar8*)encoded_text, strlen(encoded_text));
+    XftDrawStringUtf8(xft_draw, &xcolor, xtfs->font, x, y, (FcChar8 *)rs.res, rs.rsize);
     XftColorFree(xtfs->display, visual, colormap, &xcolor);
     XftDrawDestroy(xft_draw);
 
-    free(encoded_text);
+    xitk_recode2_done (xitk->font_cache->xr, &rs);
   }
 #endif
-  XUNLOCK(xtfs->display);
+  XUNLOCK (xitk_x_unlock_display, xtfs->display);
 }
 
 #ifndef WITH_XFT
@@ -721,14 +723,14 @@ int xitk_font_get_text_width(xitk_font_t *xtfs, const char *c, int nbytes) {
     {
       ABORT_IF_NULL(xtfs->font);
       
-      XLOCK(xtfs->display);
+      XLOCK (xitk_x_lock_display, xtfs->display);
       
       if(xitk_font_is_font_8(xtfs))
 	width = XTextWidth (xtfs->font, c, nbytes);
       else
 	width = XTextWidth16 (xtfs->font, (XChar2b *)c, nbytes);
       
-      XUNLOCK(xtfs->display);
+      XUNLOCK (xitk_x_unlock_display, xtfs->display);
     }
 #endif
  
@@ -745,7 +747,7 @@ int xitk_font_get_string_length(xitk_font_t *xtfs, const char *c) {
 /*
  *
  */
-int xitk_font_get_char_width(xitk_font_t *xtfs, char *c, int maxnbytes, int *nbytes) {
+int xitk_font_get_char_width(xitk_font_t *xtfs, const char *c, int maxnbytes, int *nbytes) {
 #ifndef WITH_XFT
   unsigned int  ch = (*c & 0xff);
   int           width;
@@ -830,7 +832,7 @@ int xitk_font_get_string_height(xitk_font_t *xtfs, const char *c) {
 /*
  *
  */
-int xitk_font_get_char_height(xitk_font_t *xtfs, char *c, int maxnbytes, int *nbytes) {
+int xitk_font_get_char_height(xitk_font_t *xtfs, const char *c, int maxnbytes, int *nbytes) {
 # ifdef WITH_XMB
   mbstate_t state;
   size_t    n;
@@ -857,13 +859,13 @@ int xitk_font_get_char_height(xitk_font_t *xtfs, char *c, int maxnbytes, int *nb
  */
 void xitk_font_text_extent(xitk_font_t *xtfs, const char *c, int nbytes,
 			   int *lbearing, int *rbearing, int *width, int *ascent, int *descent) {
+  xitk_t *xitk = gXitk;
 #ifndef WITH_XFT  
   XCharStruct ov;
   int         dir;
   int         fascent, fdescent;
 #else
   XGlyphInfo  xft_extents;
-  char       *foo_text, *encoded_text;
 #endif
 #if defined(WITH_XMB) && !defined(WITH_XFT)
   XRectangle  ink, logic;
@@ -884,7 +886,7 @@ void xitk_font_text_extent(xitk_font_t *xtfs, const char *c, int nbytes,
     
 #ifdef DEBUG
   if ((size_t)nbytes > strlen(c) + 1) {
-    XITK_WARNING("extent: %d > %d\n", nbytes, strlen(c));
+    XITK_WARNING("extent: %d > %zu\n", nbytes, strlen(c));
   }
 #endif
 
@@ -894,19 +896,26 @@ void xitk_font_text_extent(xitk_font_t *xtfs, const char *c, int nbytes,
   ABORT_IF_NULL(c);
 
   /* recode right part of string */
-  if ((size_t)nbytes > strlen(c))
-    nbytes = strlen(c);
+  {
+    char buf[2048];
+    xitk_recode_string_t rs;
 
-  foo_text = strdup(c);
-  foo_text[nbytes] = '\0';
-  encoded_text = xitk_cache_recode(foo_text);
-  nbytes = strlen(encoded_text);
-  free(foo_text);
-  
-  XLOCK(xtfs->display);
-  XftTextExtentsUtf8(xtfs->display, xtfs->font, (FcChar8*)encoded_text, nbytes, &xft_extents);
-  XUNLOCK(xtfs->display);
-  free(encoded_text);
+    rs.src = c;
+    rs.ssize = strlen (c);
+    if ((size_t)nbytes < rs.ssize)
+      rs.ssize = nbytes;
+    rs.buf = buf;
+    rs.bsize = sizeof (buf);
+    xitk_recode2_do (xitk->font_cache->xr, &rs);
+
+    /* FIXME: XftTextExtentsUtf8 () seems to be non reentrant - at least when called
+     * with same font. We probably need to mutex it when XLockDisplay () is turned off. */
+    XLOCK (xitk_x_lock_display, xtfs->display);
+    XftTextExtentsUtf8 (xtfs->display, xtfs->font, (FcChar8 *)rs.res, rs.rsize, &xft_extents);
+    XUNLOCK (xitk_x_unlock_display, xtfs->display);
+
+    xitk_recode2_done (xitk->font_cache->xr, &rs);
+  }
 
   if (width) *width       = xft_extents.xOff;
   /* Since Xft fonts have more top and bottom space rows than Xmb/Xcore fonts, */
@@ -925,9 +934,9 @@ void xitk_font_text_extent(xitk_font_t *xtfs, const char *c, int nbytes,
     ABORT_IF_NULL(xtfs->fontset);
     ABORT_IF_NULL(c);
     
-    XLOCK(xtfs->display);
+    XLOCK (xitk_x_lock_display, xtfs->display);
     XmbTextExtents(xtfs->fontset, c, nbytes, &ink, &logic);
-    XUNLOCK(xtfs->display);
+    XUNLOCK (xitk_x_unlock_display, xtfs->display);
     if (!logic.width || !logic.height) {
       /* XmbTextExtents() has problems, try char-per-char counting */
       mbstate_t state;
@@ -936,7 +945,7 @@ void xitk_font_text_extent(xitk_font_t *xtfs, const char *c, int nbytes,
       
       memset(&state, '\0', sizeof(mbstate_t));
       
-      XLOCK(xtfs->display);
+      XLOCK (xitk_x_lock_display, xtfs->display);
       while (i < nbytes) {
 	n = mbrtowc(NULL, c + i, nbytes - i, &state);
 	if (n == (size_t)-1 || n == (size_t)-2 || i + n > nbytes) n = 1;
@@ -948,7 +957,7 @@ void xitk_font_text_extent(xitk_font_t *xtfs, const char *c, int nbytes,
 	i += n;
       }
       rb = width - logic.width + ink.x + ink.width; /* right bearing of string */
-      XUNLOCK(xtfs->display);
+      XUNLOCK (xitk_x_unlock_display, xtfs->display);
       
       if (!height || !width) {
 	/* char-per-char counting fails too */
@@ -978,12 +987,12 @@ void xitk_font_text_extent(xitk_font_t *xtfs, const char *c, int nbytes,
       ABORT_IF_NULL(xtfs->display);
       ABORT_IF_NULL(c);
       
-      XLOCK(xtfs->display);
+      XLOCK (xitk_x_lock_display, xtfs->display);
       if(xitk_font_is_font_8(xtfs))
 	XTextExtents(xtfs->font, c, nbytes, &dir, &fascent, &fdescent, &ov);
       else
 	XTextExtents16(xtfs->font, (XChar2b *)c, (nbytes / 2), &dir, &fascent, &fdescent, &ov);
-      XUNLOCK(xtfs->display);
+      XUNLOCK (xitk_x_unlock_display, xtfs->display);
       
       if(lbearing)
 	*lbearing = ov.lbearing;
@@ -1068,15 +1077,17 @@ void xitk_font_set_font(xitk_font_t *xtfs, GC gc) {
   ABORT_IF_NULL(xtfs);
   ABORT_IF_NULL(xtfs->display);
 #ifndef WITH_XFT
-#ifdef WITH_XMB
+# ifdef WITH_XMB
   if(xitk_get_xmb_enability())
     ;
   else
-#endif
+# endif
     {
-      XLOCK(xtfs->display);
+      XLOCK (xitk_x_lock_display, xtfs->display);
       XSetFont(xtfs->display, gc, xitk_font_get_font_id(xtfs));
-      XUNLOCK(xtfs->display);
+      XUNLOCK (xitk_x_unlock_display, xtfs->display);
     }
+#else
+  (void)gc;
 #endif
 }
