@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2009 the xine project
+ * Copyright (C) 2000-2019 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -23,10 +23,10 @@
 
 #include "_xitk.h"
 
-#define DEFAULT_FONT_10      "-*-helvetica-medium-r-*-*-10-*-*-*-*-*-*-*"
+#define DEFAULT_FONT_10      "-*-helvetica-medium-r-*-*-11-*-*-*-*-*-*-*"
 #define DEFAULT_FONT_12      "-*-helvetica-medium-r-*-*-12-*-*-*-*-*-*-*"
 #define DEFAULT_FONT_14      "-*-helvetica-medium-r-*-*-14-*-*-*-*-*-*-*"
-#define DEFAULT_BOLD_FONT_10 "-*-helvetica-bold-r-*-*-10-*-*-*-*-*-*-*"
+#define DEFAULT_BOLD_FONT_10 "-*-helvetica-bold-r-*-*-11-*-*-*-*-*-*-*"
 #define DEFAULT_BOLD_FONT_12 "-*-helvetica-bold-r-*-*-12-*-*-*-*-*-*-*"
 #define DEFAULT_BOLD_FONT_14 "-*-helvetica-bold-r-*-*-14-*-*-*-*-*-*-*"
 #define FONT_HEIGHT_MODEL    "azertyuiopqsdfghjklmwxcvbnAZERTYUIOPQSDFGHJKLMWXCVBN&(-_)=~#{[|`\\^@]}%"
@@ -44,7 +44,7 @@ void xitk_font_cache_done(void);
 /*
  * load font from font name. Return NULL on failure.
  */
-xitk_font_t *xitk_font_load_font(Display *display, char *font);
+xitk_font_t *xitk_font_load_font(Display *display, const char *font);
 
 /*
  * Draw the string with given font.
@@ -72,7 +72,7 @@ int xitk_font_get_string_length(xitk_font_t *xtfs, const char *c);
  * Return the char width (in pixel),
  * if nbytes != NULL returns there size of multibyte character in bytes.
  */
-int xitk_font_get_char_width(xitk_font_t *xtfs, char *c, int maxnbytes, int *nbytes);
+int xitk_font_get_char_width(xitk_font_t *xtfs, const char *c, int maxnbytes, int *nbytes);
 
 /*
  * Return the height (in pixel) of string, with length length.
@@ -88,7 +88,7 @@ int xitk_font_get_string_height(xitk_font_t *xtfs, const char *c);
 /*
  * Return char height (in pixel).
  */
-int xitk_font_get_char_height(xitk_font_t *xtfs, char *c, int maxnbytes, int *nbytes);
+int xitk_font_get_char_height(xitk_font_t *xtfs, const char *c, int maxnbytes, int *nbytes);
 
 /*
  * Get text extents of string length length.
