@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2014 the xine project
+ * Copyright (C) 2000-2019 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -152,9 +152,9 @@ int xitk_get_keysym_and_buf(XEvent *event, KeySym *ksym, char kbuf[], int kblen)
   if(event) {
     XKeyEvent  pkeyev = event->xkey;
     
-    XLOCK(pkeyev.display);
+    XLOCK (xitk_x_lock_display, pkeyev.display);
     len = XLookupString(&pkeyev, kbuf, kblen, ksym, NULL);
-    XUNLOCK(pkeyev.display);
+    XUNLOCK (xitk_x_unlock_display, pkeyev.display);
   }
   return len;
 }
@@ -297,17 +297,17 @@ static void create_labelofinputtext(xitk_widget_t *w,
       xcolor.green = color->green<<8;
     }
     
-    XLOCK(private_data->imlibdata->x.disp);
+    XLOCK (private_data->imlibdata->x.x_lock_display, private_data->imlibdata->x.disp);
     XAllocColor(private_data->imlibdata->x.disp, Imlib_get_colormap(private_data->imlibdata), &xcolor);
-    XUNLOCK(private_data->imlibdata->x.disp);
+    XUNLOCK (private_data->imlibdata->x.x_unlock_display, private_data->imlibdata->x.disp);
     
     fg = xcolor.pixel;
     
   }
   
-  XLOCK(private_data->imlibdata->x.disp);
+  XLOCK (private_data->imlibdata->x.x_lock_display, private_data->imlibdata->x.disp);
   XSetForeground(private_data->imlibdata->x.disp, gc, fg);
-  XUNLOCK(private_data->imlibdata->x.disp);
+  XUNLOCK (private_data->imlibdata->x.x_unlock_display, private_data->imlibdata->x.disp);
   
   if(label && strlen(label)) {
     if(private_data->cursor_pos >= 0) {
@@ -345,11 +345,11 @@ static void create_labelofinputtext(xitk_widget_t *w,
   /*  Put text in the right place */
   if(private_data->skin_element_name) {
     if(label && strlen(label)) {
-      XLOCK(private_data->imlibdata->x.disp);
+      XLOCK (private_data->imlibdata->x.x_lock_display, private_data->imlibdata->x.disp);
       xitk_font_draw_string(fs, pix, gc, 
 			    2, ((ysize+asc+des+yoff)>>1)-des, 
 			    plabel, strlen(plabel));
-      XUNLOCK(private_data->imlibdata->x.disp);
+      XUNLOCK (private_data->imlibdata->x.x_unlock_display, private_data->imlibdata->x.disp);
     }
   }
   else {
@@ -357,29 +357,29 @@ static void create_labelofinputtext(xitk_widget_t *w,
     xitk_pixmap_t     *tpix;
     GC                 lgc;
     
-    XLOCK(private_data->imlibdata->x.disp);
+    XLOCK (private_data->imlibdata->x.x_lock_display, private_data->imlibdata->x.disp);
     XGetWindowAttributes(private_data->imlibdata->x.disp, win, &attr);
     
     lgc = XCreateGC(private_data->imlibdata->x.disp, win, None, None);
     XCopyGC(private_data->imlibdata->x.disp, gc, (1 << GCLastBit) - 1, lgc);
-    XUNLOCK(private_data->imlibdata->x.disp);
+    XUNLOCK (private_data->imlibdata->x.x_unlock_display, private_data->imlibdata->x.disp);
     
     tpix = xitk_image_create_xitk_pixmap(private_data->imlibdata, xsize, ysize);
     
-    XLOCK(private_data->imlibdata->x.disp);
+    XLOCK (private_data->imlibdata->x.x_lock_display, private_data->imlibdata->x.disp);
     XCopyArea (private_data->imlibdata->x.disp, pix, tpix->pixmap, lgc, 0, 0, xsize, ysize, 0, 0);
     
     if(label && strlen(label))
       xitk_font_draw_string(fs, tpix->pixmap, lgc, 3 , ((ysize+asc+des+yoff)>>1)-des, plabel, strlen(plabel));
     
     XCopyArea (private_data->imlibdata->x.disp, tpix->pixmap, pix, lgc, 0, 0, xsize - 1, ysize, 0, 0);
-    XUNLOCK(private_data->imlibdata->x.disp);
+    XUNLOCK (private_data->imlibdata->x.x_unlock_display, private_data->imlibdata->x.disp);
     
     xitk_image_destroy_xitk_pixmap(tpix);
 
-    XLOCK(private_data->imlibdata->x.disp);
+    XLOCK (private_data->imlibdata->x.x_lock_display, private_data->imlibdata->x.disp);
     XFreeGC(private_data->imlibdata->x.disp, lgc);
-    XUNLOCK(private_data->imlibdata->x.disp);
+    XUNLOCK (private_data->imlibdata->x.x_unlock_display, private_data->imlibdata->x.disp);
   }
 
   /* Draw cursor pointer */
@@ -393,7 +393,7 @@ static void create_labelofinputtext(xitk_widget_t *w,
       width += 2;
 #endif
     
-    XLOCK(private_data->imlibdata->x.disp);
+    XLOCK (private_data->imlibdata->x.x_lock_display, private_data->imlibdata->x.disp);
     XDrawLine(private_data->imlibdata->x.disp, pix, gc,
 	      width + 1, 2, width + 3, 2);
     
@@ -402,7 +402,7 @@ static void create_labelofinputtext(xitk_widget_t *w,
     
     XDrawLine(private_data->imlibdata->x.disp, pix, gc, 
 	      width + 1, ysize - 3, width + 3, ysize - 3);
-    XUNLOCK(private_data->imlibdata->x.disp);
+    XUNLOCK (private_data->imlibdata->x.x_unlock_display, private_data->imlibdata->x.disp);
   }
   
   if(label && strlen(label))  
@@ -432,29 +432,29 @@ static void paint_inputtext(xitk_widget_t *w) {
 	 && (xitk_is_mouse_over_widget(private_data->imlibdata->x.disp, w->wl->win, w)))
 	_cursor_focus(private_data, w->wl->win, 1);
 
-      XLOCK(private_data->imlibdata->x.disp);
+      XLOCK (private_data->imlibdata->x.x_lock_display, private_data->imlibdata->x.disp);
       XGetWindowAttributes(private_data->imlibdata->x.disp, w->wl->win, &attr);
-      XUNLOCK(private_data->imlibdata->x.disp);
+      XUNLOCK (private_data->imlibdata->x.x_unlock_display, private_data->imlibdata->x.disp);
 
       skin = private_data->skin;
       
-      XLOCK(private_data->imlibdata->x.disp);
+      XLOCK (private_data->imlibdata->x.x_lock_display, private_data->imlibdata->x.disp);
       lgc = XCreateGC(private_data->imlibdata->x.disp, w->wl->win, None, None);
       XCopyGC(private_data->imlibdata->x.disp, w->wl->gc, (1 << GCLastBit) - 1, lgc);
-      XUNLOCK(private_data->imlibdata->x.disp);
+      XUNLOCK (private_data->imlibdata->x.x_unlock_display, private_data->imlibdata->x.disp);
       
       if (skin->mask) {
-	XLOCK(private_data->imlibdata->x.disp);
+        XLOCK (private_data->imlibdata->x.x_lock_display, private_data->imlibdata->x.disp);
 	XSetClipOrigin(private_data->imlibdata->x.disp, lgc, w->x, w->y);
 	XSetClipMask(private_data->imlibdata->x.disp, lgc, skin->mask->pixmap);
-	XUNLOCK(private_data->imlibdata->x.disp);
+        XUNLOCK (private_data->imlibdata->x.x_unlock_display, private_data->imlibdata->x.disp);
       }
       
       button_width = skin->width / 2;
       
       btn = xitk_image_create_xitk_pixmap(private_data->imlibdata, button_width, skin->height);
       
-      XLOCK(private_data->imlibdata->x.disp);
+      XLOCK (private_data->imlibdata->x.x_lock_display, private_data->imlibdata->x.disp);
       if((w->have_focus == FOCUS_RECEIVED) || (private_data->have_focus == FOCUS_MOUSE_IN)) {
 	state = FOCUS;
 	XCopyArea (private_data->imlibdata->x.disp, skin->image->pixmap,
@@ -467,21 +467,21 @@ static void paint_inputtext(xitk_widget_t *w) {
 		   btn->pixmap, w->wl->gc, 0, 0,
 		   button_width, skin->height, 0, 0);
       }
-      XUNLOCK(private_data->imlibdata->x.disp);
+      XUNLOCK (private_data->imlibdata->x.x_unlock_display, private_data->imlibdata->x.disp);
       
       create_labelofinputtext(w, w->wl->win, w->wl->gc, btn->pixmap, 
 			      button_width, skin->height, private_data->text, state);
       
-      XLOCK(private_data->imlibdata->x.disp);
+      XLOCK (private_data->imlibdata->x.x_lock_display, private_data->imlibdata->x.disp);
       XCopyArea (private_data->imlibdata->x.disp, btn->pixmap, w->wl->win, lgc, 0, 0,
 		 button_width, skin->height, w->x, w->y);
-      XUNLOCK(private_data->imlibdata->x.disp);
+      XUNLOCK (private_data->imlibdata->x.x_unlock_display, private_data->imlibdata->x.disp);
 
       xitk_image_destroy_xitk_pixmap(btn);
 
-      XLOCK(private_data->imlibdata->x.disp);
+      XLOCK (private_data->imlibdata->x.x_lock_display, private_data->imlibdata->x.disp);
       XFreeGC(private_data->imlibdata->x.disp, lgc);
-      XUNLOCK(private_data->imlibdata->x.disp);
+      XUNLOCK (private_data->imlibdata->x.x_unlock_display, private_data->imlibdata->x.disp);
     }
     else {
       if(private_data->cursor_focus)
@@ -1087,7 +1087,7 @@ char *xitk_inputtext_get_text(xitk_widget_t *w) {
 /*
  * Change and redisplay the text of widget.
  */
-void xitk_inputtext_change_text(xitk_widget_t *w, char *text) {
+void xitk_inputtext_change_text(xitk_widget_t *w, const char *text) {
   inputtext_private_data_t *private_data;
   
   if(w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_INPUTTEXT)) {
@@ -1110,10 +1110,10 @@ void xitk_inputtext_change_text(xitk_widget_t *w, char *text) {
 static xitk_widget_t *_xitk_inputtext_create (xitk_widget_list_t *wl,
 					      xitk_skin_config_t *skonfig, 
 					      xitk_inputtext_widget_t *it,
-					      int x, int y, char *skin_element_name,
+                                              int x, int y, const char *skin_element_name,
 					      xitk_image_t *skin,
-					      char *fontname,
-					      char *ncolor, char *fcolor,
+                                              const char *fontname,
+                                              const char *ncolor, const char *fcolor,
 					      int visible, int enable) {
   xitk_widget_t             *mywidget;
   inputtext_private_data_t  *private_data;
@@ -1199,7 +1199,7 @@ xitk_widget_t *xitk_inputtext_create (xitk_widget_list_t *wl,
 xitk_widget_t *xitk_noskin_inputtext_create (xitk_widget_list_t *wl,
 					     xitk_inputtext_widget_t *it,
 					     int x, int y, int width, int height,
-					     char *ncolor, char *fcolor, char *fontname) {
+                                             const char *ncolor, const char *fcolor, const char *fontname) {
   xitk_image_t *i;
 
   XITK_CHECK_CONSTITENCY(it);
