@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2009 the xine project
+ * Copyright (C) 2000-2019 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -32,6 +32,33 @@
 #define FOCUS     2
 #define NORMAL    3
 
+static size_t _strlcpy (char *d, const char *s, size_t l) {
+  size_t n;
+  if (!s)
+    s = "";
+  n = strlen (s);
+  if (l > n + 1)
+    l = n + 1;
+  memcpy (d, s, l);
+  d[l - 1] = 0;
+  return n;
+}
+
+static void _set_label (lbutton_private_data_t *private_data, const char *label) {
+  size_t n;
+  if (private_data->label != private_data->lbuf)
+    free (private_data->label);
+  if (!label)
+    label = "";
+  n = strlen (label) + 1;
+  if (n <= sizeof (private_data->lbuf)) {
+    memcpy (private_data->lbuf, label, n);
+    private_data->label = private_data->lbuf;
+  } else {
+    private_data->label = strdup (label);
+  }
+}
+
 /*
  *
  */
@@ -41,16 +68,14 @@ static void notify_destroy(xitk_widget_t *w) {
   if(w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_LABELBUTTON)) {
     private_data = (lbutton_private_data_t *) w->private_data;
 
-    if(!private_data->skin_element_name)
+    if (private_data->skin_element_name[0] == '\x01')
       xitk_image_free_image(private_data->imlibdata, &(private_data->skin));
 
-    XITK_FREE(private_data->label);
+    if (private_data->label != private_data->lbuf) {
+      XITK_FREE (private_data->label);
+    }
     XITK_FREE(private_data->shortcut_label);
     XITK_FREE(private_data->shortcut_font);
-    XITK_FREE(private_data->skin_element_name);
-    XITK_FREE(private_data->normcolor);
-    XITK_FREE(private_data->focuscolor);
-    XITK_FREE(private_data->clickcolor);
     XITK_FREE(private_data->fontname);
     XITK_FREE(private_data);
   }
@@ -131,8 +156,8 @@ static void create_labelofbutton(xitk_widget_t *lb,
     yoff = 1;
   }
 
-  if((private_data->skin_element_name != NULL) ||
-     ((private_data->skin_element_name == NULL) && ((fg = xitk_get_black_color()) == (unsigned int)-1))) {
+  if ((private_data->skin_element_name[0] & ~1)
+    || (!(private_data->skin_element_name[0] & ~1) && ((fg = xitk_get_black_color()) == (unsigned int)-1))) {
     
     /*  Some colors configurations */
     switch(state) {
@@ -173,17 +198,17 @@ static void create_labelofbutton(xitk_widget_t *lb,
       xcolor.green = color->green<<8;
     }
     
-    XLOCK(private_data->imlibdata->x.disp);
+    XLOCK (private_data->imlibdata->x.x_lock_display, private_data->imlibdata->x.disp);
     XAllocColor(private_data->imlibdata->x.disp,
     Imlib_get_colormap(private_data->imlibdata), &xcolor);
-    XUNLOCK(private_data->imlibdata->x.disp);
+    XUNLOCK (private_data->imlibdata->x.x_unlock_display, private_data->imlibdata->x.disp);
 
     fg = xcolor.pixel;
   }
   
-  XLOCK(private_data->imlibdata->x.disp);
+  XLOCK (private_data->imlibdata->x.x_lock_display, private_data->imlibdata->x.disp);
   XSetForeground(private_data->imlibdata->x.disp, gc, fg);
-  XUNLOCK(private_data->imlibdata->x.disp);
+  XUNLOCK (private_data->imlibdata->x.x_unlock_display, private_data->imlibdata->x.disp);
 
   origin = ((ysize+asc+des+yoff)>>1)-des;
   
@@ -242,22 +267,22 @@ static void paint_labelbutton (xitk_widget_t *w) {
 
     if(w->visible == 1) {
       
-      XLOCK(private_data->imlibdata->x.disp);
+      XLOCK (private_data->imlibdata->x.x_lock_display, private_data->imlibdata->x.disp);
       XGetWindowAttributes(private_data->imlibdata->x.disp, w->wl->win, &attr);
-      XUNLOCK(private_data->imlibdata->x.disp);
+      XUNLOCK (private_data->imlibdata->x.x_unlock_display, private_data->imlibdata->x.disp);
       
       skin = private_data->skin;
       
-      XLOCK(private_data->imlibdata->x.disp);
+      XLOCK (private_data->imlibdata->x.x_lock_display, private_data->imlibdata->x.disp);
       lgc = XCreateGC(private_data->imlibdata->x.disp, w->wl->win, None, None);
       XCopyGC(private_data->imlibdata->x.disp, w->wl->gc, (1 << GCLastBit) - 1, lgc);
-      XUNLOCK(private_data->imlibdata->x.disp);
+      XUNLOCK (private_data->imlibdata->x.x_unlock_display, private_data->imlibdata->x.disp);
 	    
       if (skin->mask) {
-	XLOCK(private_data->imlibdata->x.disp);
+        XLOCK (private_data->imlibdata->x.x_lock_display, private_data->imlibdata->x.disp);
 	XSetClipOrigin(private_data->imlibdata->x.disp, lgc, w->x, w->y);
 	XSetClipMask(private_data->imlibdata->x.disp, lgc, skin->mask->pixmap);
-	XUNLOCK(private_data->imlibdata->x.disp);
+        XUNLOCK (private_data->imlibdata->x.x_unlock_display, private_data->imlibdata->x.disp);
       }
       
       button_width = skin->width / 3;
@@ -266,35 +291,35 @@ static void paint_labelbutton (xitk_widget_t *w) {
       if ((private_data->focus == FOCUS_RECEIVED) || (private_data->focus == FOCUS_MOUSE_IN)) {
 	if (private_data->bClicked) {
 	  state = CLICK;
-	  XLOCK(private_data->imlibdata->x.disp);
+          XLOCK (private_data->imlibdata->x.x_lock_display, private_data->imlibdata->x.disp);
 	  XCopyArea (private_data->imlibdata->x.disp, skin->image->pixmap, 
 		     btn->pixmap, w->wl->gc, 2*button_width, 0,
 		     button_width, skin->height, 0, 0);
-	  XUNLOCK(private_data->imlibdata->x.disp);
+          XUNLOCK (private_data->imlibdata->x.x_unlock_display, private_data->imlibdata->x.disp);
 	}
 	else {
 	  if(!private_data->bState || private_data->bType == CLICK_BUTTON) {
 	    state = FOCUS;
-	    XLOCK(private_data->imlibdata->x.disp);
+            XLOCK (private_data->imlibdata->x.x_lock_display, private_data->imlibdata->x.disp);
 	    XCopyArea (private_data->imlibdata->x.disp, skin->image->pixmap,
 		       btn->pixmap, w->wl->gc, button_width, 0,
 		       button_width, skin->height, 0, 0);
-	    XUNLOCK(private_data->imlibdata->x.disp);
+            XUNLOCK (private_data->imlibdata->x.x_unlock_display, private_data->imlibdata->x.disp);
 	  }
 	  else {
 	    if(private_data->bType == RADIO_BUTTON) {
 	      state = CLICK;
-	      XLOCK(private_data->imlibdata->x.disp);
+              XLOCK (private_data->imlibdata->x.x_lock_display, private_data->imlibdata->x.disp);
 	      XCopyArea (private_data->imlibdata->x.disp, skin->image->pixmap,
 			 btn->pixmap, w->wl->gc, 2*button_width, 0,
 			 button_width, skin->height, 0, 0);
-	      XUNLOCK(private_data->imlibdata->x.disp);
+              XUNLOCK (private_data->imlibdata->x.x_unlock_display, private_data->imlibdata->x.disp);
 	    }
 	  }
 	}
       }
       else {
-	XLOCK(private_data->imlibdata->x.disp);
+        XLOCK (private_data->imlibdata->x.x_lock_display, private_data->imlibdata->x.disp);
 	if(private_data->bState && private_data->bType == RADIO_BUTTON) {
 	  if(private_data->bOldState == 1 && private_data->bClicked == 1) {
 	    state = NORMAL;
@@ -313,7 +338,7 @@ static void paint_labelbutton (xitk_widget_t *w) {
 	  XCopyArea (private_data->imlibdata->x.disp, skin->image->pixmap, btn->pixmap, 
 		     w->wl->gc, 0, 0, button_width, skin->height, 0, 0);
 	}
-	XUNLOCK(private_data->imlibdata->x.disp);
+        XUNLOCK (private_data->imlibdata->x.x_unlock_display, private_data->imlibdata->x.disp);
       }
       
       if(private_data->label_visible) {
@@ -322,16 +347,16 @@ static void paint_labelbutton (xitk_widget_t *w) {
 			     private_data->label, private_data->shortcut_label, private_data->shortcut_pos, state);
       }
       
-      XLOCK(private_data->imlibdata->x.disp);
+      XLOCK (private_data->imlibdata->x.x_lock_display, private_data->imlibdata->x.disp);
       XCopyArea (private_data->imlibdata->x.disp, btn->pixmap, w->wl->win, lgc, 0, 0,
 		 button_width, skin->height, w->x, w->y);
-      XUNLOCK(private_data->imlibdata->x.disp);
+      XUNLOCK (private_data->imlibdata->x.x_unlock_display, private_data->imlibdata->x.disp);
 
       xitk_image_destroy_xitk_pixmap(btn);
 
-      XLOCK(private_data->imlibdata->x.disp);
+      XLOCK (private_data->imlibdata->x.x_lock_display, private_data->imlibdata->x.disp);
       XFreeGC(private_data->imlibdata->x.disp, lgc);
-      XUNLOCK(private_data->imlibdata->x.disp);
+      XUNLOCK (private_data->imlibdata->x.x_unlock_display, private_data->imlibdata->x.disp);
     }
   }
 }
@@ -380,17 +405,13 @@ static int notify_click_labelbutton (xitk_widget_t *w, int button, int bUp, int
 /*
  * Changing button caption
  */
-int xitk_labelbutton_change_label(xitk_widget_t *w, char *newlabel) {
+int xitk_labelbutton_change_label (xitk_widget_t *w, const char *newlabel) {
   lbutton_private_data_t *private_data;
   
   if (w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_LABELBUTTON)) {
     private_data = (lbutton_private_data_t *) w->private_data;
-
-    if((private_data->label = (char *) realloc(private_data->label, strlen(newlabel)+1)) != NULL)
-      strcpy(private_data->label, newlabel);
-    
+    _set_label (private_data, newlabel);
     paint_labelbutton(w);
-    
     return 1;
   }
 
@@ -400,7 +421,7 @@ int xitk_labelbutton_change_label(xitk_widget_t *w, char *newlabel) {
 /*
  * Return the current button label
  */
-char *xitk_labelbutton_get_label(xitk_widget_t *w) {
+const char *xitk_labelbutton_get_label(xitk_widget_t *w) {
   lbutton_private_data_t *private_data;
 
   if (w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_LABELBUTTON)) {
@@ -414,7 +435,7 @@ char *xitk_labelbutton_get_label(xitk_widget_t *w) {
 /*
  * Changing button caption
  */
-int xitk_labelbutton_change_shortcut_label(xitk_widget_t *w, char *newlabel, int pos, char *newfont) {
+int xitk_labelbutton_change_shortcut_label(xitk_widget_t *w, const char *newlabel, int pos, const char *newfont) {
   lbutton_private_data_t *private_data;
 
   if (w && (((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_LABELBUTTON) && 
@@ -446,7 +467,7 @@ int xitk_labelbutton_change_shortcut_label(xitk_widget_t *w, char *newlabel, int
 /*
  * Return the current button label
  */
-char *xitk_labelbutton_get_shortcut_label(xitk_widget_t *w) {
+const char *xitk_labelbutton_get_shortcut_label(xitk_widget_t *w) {
   lbutton_private_data_t *private_data;
 
   if (w && (((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_LABELBUTTON) &&
@@ -481,38 +502,29 @@ static void notify_change_skin(xitk_widget_t *w, xitk_skin_config_t *skonfig) {
   if (w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_LABELBUTTON)) {
     private_data = (lbutton_private_data_t *) w->private_data;
     
-    if(private_data->skin_element_name) {
+    if (private_data->skin_element_name[0] & ~1) {
+      const xitk_skin_element_info_t *info;
       
       xitk_skin_lock(skonfig);
-      
-      private_data->skin          = xitk_skin_get_image(skonfig, 
-							xitk_skin_get_skin_filename(skonfig, private_data->skin_element_name));
-      XITK_FREE(private_data->normcolor);
-      private_data->normcolor     = strdup(xitk_skin_get_label_color(skonfig, 
-							     private_data->skin_element_name));
-      XITK_FREE(private_data->focuscolor);
-      private_data->focuscolor    = strdup(xitk_skin_get_label_color_focus(skonfig,
-							   private_data->skin_element_name));
-      XITK_FREE(private_data->clickcolor);
-      private_data->clickcolor    = strdup(xitk_skin_get_label_color_click(skonfig, 
-							   private_data->skin_element_name));
-      XITK_FREE(private_data->fontname);
-      private_data->fontname      = strdup(xitk_skin_get_label_fontname(skonfig, 
-									private_data->skin_element_name));
-      private_data->label_visible = xitk_skin_get_label_printable(skonfig, private_data->skin_element_name);
-      
-      private_data->label_static  = xitk_skin_get_label_staticity(skonfig, private_data->skin_element_name);
-
-      private_data->align         = xitk_skin_get_label_alignment(skonfig, private_data->skin_element_name);
-
-      w->x                    = xitk_skin_get_coord_x(skonfig, private_data->skin_element_name);
-      w->y                    = xitk_skin_get_coord_y(skonfig, private_data->skin_element_name);
-      w->width                = private_data->skin->width/3;
-      w->height               = private_data->skin->height;
-      w->visible              = (xitk_skin_get_visibility(skonfig, private_data->skin_element_name)) ? 1 : -1;
-      w->enable               = xitk_skin_get_enability(skonfig, private_data->skin_element_name);
-      
-      xitk_skin_unlock(skonfig);
+      info = xitk_skin_get_info (skonfig, private_data->skin_element_name);
+      XITK_FREE (private_data->fontname);
+      if (info) {
+        private_data->skin = info->pixmap_img;
+        _strlcpy (private_data->normcolor,  info->label_color,       sizeof (private_data->normcolor));
+        _strlcpy (private_data->focuscolor, info->label_color_focus, sizeof (private_data->focuscolor));
+        _strlcpy (private_data->clickcolor, info->label_color_click, sizeof (private_data->clickcolor));
+        private_data->fontname      = strdup (info->label_fontname);
+        private_data->label_visible = info->label_printable;
+        private_data->label_static  = info->label_staticity;
+        private_data->align         = info->label_alignment;
+        w->x      = info->x;
+        w->y      = info->y;
+        w->width  = private_data->skin->width / 3;
+        w->height = private_data->skin->height;
+        w->visible = info->visibility ? 1 : -1;
+        w->enable = info->enability;
+      }
+      xitk_skin_unlock (skonfig);
 
       xitk_set_widget_pos(w, w->x, w->y);
     }
@@ -701,24 +713,21 @@ void xitk_labelbutton_callback_exec(xitk_widget_t *w) {
 /*
  * Create the labeled button
  */
-static xitk_widget_t *_xitk_labelbutton_create (xitk_widget_list_t *wl,
-						xitk_skin_config_t *skonfig, 
-						xitk_labelbutton_widget_t *b,
-						int x, int y, 
-						char *skin_element_name, xitk_image_t *skin,
-						char *ncolor, char *fcolor, char *ccolor,
-						char *fontname, 
-						int label_visible, int label_static,
-						int visible, int enable) {
+xitk_widget_t *xitk_info_labelbutton_create (xitk_widget_list_t *wl,
+  const xitk_labelbutton_widget_t *b, const xitk_skin_element_info_t *info) {
   xitk_widget_t           *mywidget;
   lbutton_private_data_t *private_data;
   
   mywidget = (xitk_widget_t *) xitk_xmalloc (sizeof(xitk_widget_t));
-  
+  if (!mywidget)
+    return NULL;
   private_data = (lbutton_private_data_t *) xitk_xmalloc (sizeof (lbutton_private_data_t));
+  if (!private_data) {
+    free (mywidget);
+    return NULL;
+  }
 
   private_data->imlibdata         = b->imlibdata;
-  private_data->skin_element_name = (skin_element_name == NULL) ? NULL : strdup(b->skin_element_name);
 
   private_data->bWidget           = mywidget;
   private_data->bType             = b->button_type;
@@ -730,34 +739,43 @@ static xitk_widget_t *_xitk_labelbutton_create (xitk_widget_list_t *wl,
   private_data->callback          = b->callback;
   private_data->state_callback    = b->state_callback;
   private_data->userdata          = b->userdata;
-  private_data->label             = strdup((b->label)?b->label:"");
-  private_data->shortcut_label    = strdup("");
-  private_data->shortcut_font     = strdup(fontname);
-  private_data->shortcut_pos      = -1;
-  private_data->label_visible     = label_visible;
-  private_data->label_static      = label_static;
 
-  private_data->skin              = skin;
-  private_data->normcolor         = strdup(ncolor);
-  private_data->focuscolor        = strdup(fcolor);
-  private_data->clickcolor        = strdup(ccolor);
-  private_data->fontname          = strdup(fontname);
+  private_data->label = private_data->lbuf;
+  _set_label (private_data, b->label);
+
+  private_data->shortcut_label    = strdup("");
+  private_data->shortcut_font     = strdup (info->label_fontname);
+  private_data->shortcut_pos      = -1;
+  private_data->label_visible     = info->label_printable;
+  private_data->label_static      = info->label_staticity;
+
+  private_data->skin              = info->pixmap_img;
+
+  if (info->pixmap_name && (info->pixmap_name[0] == '\x01') && (info->pixmap_name[1] == 0)) {
+    private_data->skin_element_name[0] = '\x01';
+    private_data->skin_element_name[1] = 0;
+  } else {
+    _strlcpy (private_data->skin_element_name, b->skin_element_name, sizeof (private_data->skin_element_name));
+  }
+  _strlcpy (private_data->normcolor, info->label_color, sizeof (private_data->normcolor));
+  _strlcpy (private_data->focuscolor, info->label_color_focus, sizeof (private_data->focuscolor));
+  _strlcpy (private_data->clickcolor, info->label_color_click, sizeof (private_data->clickcolor));
+  private_data->fontname          = strdup (info->label_fontname);
 
   private_data->label_offset      = 0;
-
-  private_data->align             = skin_element_name ? (xitk_skin_get_label_alignment(skonfig, skin_element_name)) : b->align;
+  private_data->align             = info->label_alignment;
 
   mywidget->private_data          = private_data;
 
   mywidget->wl                    = wl;
 
-  mywidget->enable                = enable;
+  mywidget->enable                = info->enability;
   mywidget->running               = 1;
-  mywidget->visible               = visible;
+  mywidget->visible               = info->visibility;
   mywidget->have_focus            = FOCUS_LOST;
   mywidget->imlibdata             = private_data->imlibdata;
-  mywidget->x                     = x;
-  mywidget->y                     = y;
+  mywidget->x                     = info->x;
+  mywidget->y                     = info->y;
   mywidget->width                 = private_data->skin->width/3;
   mywidget->height                = private_data->skin->height;
   mywidget->type                  = WIDGET_TYPE_LABELBUTTON | WIDGET_FOCUSABLE | WIDGET_CLICKABLE | WIDGET_KEYABLE;
@@ -772,39 +790,53 @@ static xitk_widget_t *_xitk_labelbutton_create (xitk_widget_list_t *wl,
  *
  */
 xitk_widget_t *xitk_labelbutton_create (xitk_widget_list_t *wl,
-					xitk_skin_config_t *skonfig, xitk_labelbutton_widget_t *b) {
+  xitk_skin_config_t *skonfig, const xitk_labelbutton_widget_t *b) {
+  const xitk_skin_element_info_t *pinfo;
+  xitk_skin_element_info_t info;
 
-  XITK_CHECK_CONSTITENCY(b);
+  XITK_CHECK_CONSTITENCY (b);
+  pinfo = xitk_skin_get_info (skonfig, b->skin_element_name);
+  if (pinfo) {
+    if (!pinfo->visibility) {
+      info = *pinfo;
+      info.visibility = 1;
+      pinfo = &info;
+    }
+  } else {
+    memset (&info, 0, sizeof (info));
+    info.label_alignment = b->align;
+    info.visibility = -1;
+    pinfo = &info;
+  }
 
-  return _xitk_labelbutton_create(wl, skonfig, b, 
-		  (xitk_skin_get_coord_x(skonfig, b->skin_element_name)),
-		  (xitk_skin_get_coord_y(skonfig, b->skin_element_name)),
-		   b->skin_element_name,
-		  (xitk_skin_get_image(skonfig, xitk_skin_get_skin_filename(skonfig, b->skin_element_name))),
-		  (xitk_skin_get_label_color(skonfig, b->skin_element_name)),
-		  (xitk_skin_get_label_color_focus(skonfig, b->skin_element_name)),
-		  (xitk_skin_get_label_color_click(skonfig, b->skin_element_name)),
-		  (xitk_skin_get_label_fontname(skonfig, b->skin_element_name)),
-		  (xitk_skin_get_label_printable(skonfig, b->skin_element_name)),
-		  (xitk_skin_get_label_staticity(skonfig, b->skin_element_name)),
-		  ((xitk_skin_get_visibility(skonfig, b->skin_element_name)) ? 1 : -1),
-		  (xitk_skin_get_enability(skonfig, b->skin_element_name)));
+  return xitk_info_labelbutton_create (wl, b, pinfo);
 }
 
 /*
  *
  */
 xitk_widget_t *xitk_noskin_labelbutton_create (xitk_widget_list_t *wl,
-					       xitk_labelbutton_widget_t *b,
-					       int x, int y, int width, int height,
-					       char *ncolor, char *fcolor, char *ccolor, 
-					       char *fname) {
-  xitk_image_t  *i;
+  const xitk_labelbutton_widget_t *b, int x, int y, int width, int height,
+  const char *ncolor, const char *fcolor, const char *ccolor, const char *fname) {
+  xitk_skin_element_info_t info;
   
   XITK_CHECK_CONSTITENCY(b);
+  info.x                 = x;
+  info.y                 = y;
+  info.label_alignment   = b->align;
+  info.label_printable   = 1;
+  info.label_staticity   = 0;
+  info.visibility        = 0;
+  info.enability         = 0;
+  info.label_color       = (char *)ncolor;
+  info.label_color_focus = (char *)fcolor;
+  info.label_color_click = (char *)ccolor;
+  info.label_fontname    = (char *)fname;
+  info.pixmap_name       = (char *)"\x01";
+  info.pixmap_img        = xitk_image_create_image (b->imlibdata, width * 3, height);
+  if (info.pixmap_img)
+    draw_bevel_three_state (b->imlibdata, info.pixmap_img);
 
-  i = xitk_image_create_image(b->imlibdata, width * 3, height);
-  draw_bevel_three_state(b->imlibdata, i);
-
-  return _xitk_labelbutton_create(wl, NULL, b, x, y, NULL, i, ncolor, fcolor, ccolor, fname, 1, 0, 0, 0);
+  return xitk_info_labelbutton_create (wl, b, &info);
 }
+
