@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2017 the xine project
+ * Copyright (C) 2000-2019 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -85,6 +85,100 @@
                                       (w)->width  = 0;    \
                                     } while(0)
 
+typedef struct xitk_dnode_st {
+  struct xitk_dnode_st *next, *prev;
+} xitk_dnode_t;
+
+typedef struct {
+  xitk_dnode_t head, tail;
+} xitk_dlist_t;
+
+static inline void xitk_dlist_init (xitk_dlist_t *list) {
+  list->head.next = &list->tail;
+  list->head.prev = NULL;
+  list->tail.next = NULL;
+  list->tail.prev = &list->head;
+}
+
+static inline void xitk_dnode_init (xitk_dnode_t *node) {
+  node->next = node->prev = NULL;
+}
+
+static inline void xitk_dnode_remove (xitk_dnode_t *node) {
+  if (node->next) {
+    node->prev->next = node->next;
+    node->next->prev = node->prev;
+    node->next = node->prev = NULL;
+  }
+}
+
+static inline void xitk_dlist_add_tail (xitk_dlist_t *list, xitk_dnode_t *node) {
+#ifdef XITK_DEBUG
+  if (node->next)
+    printf ("xitk_dlist_add_tail: node %p already added (next = %p, prev = %p).\n",
+        (void *)node, (void *)node->next, (void *)node->prev);
+#endif
+  node->prev = list->tail.prev;
+  list->tail.prev->next = node;
+  list->tail.prev = node;
+  node->next = &list->tail;
+}
+
+static inline int xitk_dlist_clear (xitk_dlist_t *list) {
+  int n = 0;
+  xitk_dnode_t *node = list->head.next;
+  list->head.next = &list->tail;
+  list->head.prev = NULL;
+  list->tail.next = NULL;
+  list->tail.prev = &list->head;
+  while (1) {
+    xitk_dnode_t *next = node->next;
+    if (!next)
+      break;
+    node->next = node->prev = NULL;
+    node = next;
+    n++;
+  }
+  return n;
+}
+
+#if 0 /* yet unused */
+
+static inline int xitk_dlist_is_empty (xitk_dlist_t *list) {
+  return list->tail.prev == &list->head;
+}
+
+static inline xitk_dnode_t *xitk_dlist_remove_head (xitk_dlist_t *list) {
+  xitk_dnode_t *node = list->head.next;
+  if (!node->next)
+    return NULL;
+  node->prev->next = node->next;
+  node->next->prev = node->prev;
+  node->next = node->prev = NULL;
+  return node;
+}
+  
+static inline xitk_dnode_t *xitk_dlist_remove_tail (xitk_dlist_t *list) {
+  xitk_dnode_t *node = list->tail.prev;
+  if (!node->prev)
+    return NULL;
+  node->prev->next = node->next;
+  node->next->prev = node->prev;
+  node->next = node->prev = NULL;
+  return node;
+}
+
+static inline void xitk_dlist_add_head (xitk_dlist_t *list, xitk_dnode_t *node) {
+  node->next = list->head.next;
+  list->head.next->prev = node;
+  list->head.next = node;
+  node->prev = &list->head;
+}
+
+#endif
+
+typedef struct xitk_s xitk_t;
+
 typedef struct xitk_widget_s xitk_widget_t;
 typedef struct xitk_menu_entry_s xitk_menu_entry_t;
 typedef struct xitk_widget_list_s xitk_widget_list_t;
@@ -93,13 +187,15 @@ typedef struct xitk_font_s xitk_font_t;
 typedef struct xitk_pixmap_s xitk_pixmap_t;
 typedef struct xitk_window_s xitk_window_t;
 
+void xitk_add_widget (xitk_widget_list_t *wl, xitk_widget_t *wi);
+
 typedef void (*xitk_startup_callback_t)(void *);
 typedef void (*xitk_simple_callback_t)(xitk_widget_t *, void *);
 typedef void (*xitk_menu_callback_t)(xitk_widget_t *, xitk_menu_entry_t *, void *);
 typedef void (*xitk_state_callback_t)(xitk_widget_t *, void *, int);
 typedef void (*xitk_state_double_callback_t)(xitk_widget_t *, void *, double);
-typedef void (*xitk_string_callback_t)(xitk_widget_t *, void *, char *);
-typedef void (*xitk_dnd_callback_t) (char *filename);
+typedef void (*xitk_string_callback_t)(xitk_widget_t *, void *, const char *);
+typedef void (*xitk_dnd_callback_t) (const char *filename);
 typedef void (*xitk_pixmap_destroyer_t)(xitk_pixmap_t *);
 #ifdef NEED_MRLBROWSER
 typedef void (*xitk_mrl_callback_t)(xitk_widget_t *, void *, xine_mrl_t *);
@@ -121,7 +217,7 @@ typedef void (*widget_event_callback_t)(XEvent *event, void *data);
  * and, of course, only if there was a window
  * specified at register time.
  */
-typedef void (*widget_newpos_callback_t)(int, int, int, int);
+typedef void (*widget_newpos_callback_t)(void *data, int x, int y, int width, int height);
 
 /*
  * Signal handler callback function.
@@ -138,6 +234,14 @@ typedef void (*xitk_signal_callback_t)(int, void *);
 typedef int xitk_register_key_t;
 
 
+typedef struct {
+  int x, y;
+} xitk_point_t;
+
+typedef struct {
+  int first, last;
+} xitk_range_t;
+
 struct xitk_pixmap_s {
   ImlibData                        *imlibdata;
   XImage                           *xim;
@@ -153,9 +257,24 @@ struct xitk_pixmap_s {
   xitk_pixmap_destroyer_t           destroy;
 };
 
+typedef struct {
+  int                               width;
+  int                               height;
+  int                               chars_per_row;
+  int                               chars_total;
+  int                               char_width;
+  int                               char_height;
+  xitk_point_t                      space;
+  xitk_point_t                      asterisk;
+  xitk_point_t                      unknown;
+#define XITK_MAX_UNICODE_RANGES 16
+  xitk_range_t                      unicode_ranges[XITK_MAX_UNICODE_RANGES + 1];
+} xitk_pix_font_t;
+
 typedef struct {
   xitk_pixmap_t                    *image;
   xitk_pixmap_t                    *mask;
+  xitk_pix_font_t                  *pix_font;
   int                               width;
   int                               height;
 } xitk_image_t;
@@ -167,19 +286,6 @@ typedef struct {
   char                              colorname[20];
 } xitk_color_names_t;
 
-typedef struct xitk_node_s {
-  struct xitk_node_s               *next;
-  struct xitk_node_s               *prev;
-  void                             *content;
-} xitk_node_t;
-
-
-typedef struct {
-  xitk_node_t                      *first;
-  xitk_node_t                      *last;
-  xitk_node_t                      *cur;
-} xitk_list_t;
-
 #define MWM_HINTS_DECORATIONS       (1L << 1)
 #define PROP_MWM_HINTS_ELEMENTS     5
 typedef struct _mwmhints {
@@ -205,6 +311,30 @@ typedef struct {
   int                               width;
 } window_info_t;
 
+typedef struct {
+  /* all */
+  int x, y;
+  int visibility, enability;
+  char *pixmap_name;
+  xitk_image_t *pixmap_img;
+  /* button list */
+  int max_buttons, direction;
+  /* browser */
+  int browser_entries;
+  /* label */
+  int label_length, label_alignment, label_printable, label_staticity;
+  int label_animation, label_animation_step;
+  unsigned long int label_animation_timer;
+  char *label_color, *label_color_focus, *label_color_click, *label_fontname;
+  char *label_pixmap_font_name;
+  char *label_pixmap_font_format;
+  xitk_image_t *label_pixmap_font_img;
+  /* slider */
+  int slider_type, slider_radius;
+  char *slider_pixmap_pad_name;
+  xitk_image_t *slider_pixmap_pad_img;
+} xitk_skin_element_info_t;
+
 /*
  *  1 <widget group >
  *   1 <The groupped widget>
@@ -422,8 +552,6 @@ typedef enum {
  */
 #define WIDGET_LIST_GC              1
 #define WIDGET_LIST_WINDOW          2
-#define WIDGET_LIST_LIST            3
-#define XITK_WIDGET_LIST_LIST(wl)   (xitk_list_t *) xitk_widget_list_get(wl, WIDGET_LIST_LIST)
 #define XITK_WIDGET_LIST_WINDOW(wl) (Window) xitk_widget_list_get(wl, WIDGET_LIST_WINDOW)
 #define XITK_WIDGET_LIST_GC(wl)     (GC) xitk_widget_list_get(wl, WIDGET_LIST_GC)
 #define XITK_WIDGET_LIST_FREE(wl)   xitk_widget_list_defferred_destroy(wl)
@@ -566,16 +694,19 @@ typedef struct {
   } origin;
   
   xitk_dnd_callback_t               dndcallback;
+  void                             *dnddata;
 
   struct {
     char                           *caption;
     char                           *skin_element_name;
     xitk_mrl_callback_t             callback;
+    void                           *data;
   } select;
 
   struct {
     char                           *skin_element_name;
     xitk_mrl_callback_t             callback;
+    void                           *data;
   } play;
 
   struct {
@@ -585,6 +716,7 @@ typedef struct {
 
   struct {
     xitk_simple_callback_t          callback;
+    void                           *data;
   } kill;
 
   const char *const                *ip_availables;
@@ -685,6 +817,7 @@ struct xitk_menu_entry_s {
   char                             *type;     /* NULL, <separator>, <branch>, <check>, <checked> */
   xitk_menu_callback_t              cb;
   void                             *user_data;
+  int                               user_id;
 };
 
 typedef struct {
@@ -728,7 +861,7 @@ void xitk_register_signal_handler(xitk_signal_callback_t sigcb, void *user_data)
  * dnd_cb: callback for dnd event.
  * wl:     widget_list handled internaly for xevent reactions.
  */
-xitk_register_key_t xitk_register_event_handler(char *name, Window window,
+xitk_register_key_t xitk_register_event_handler(const char *name, Window window,
 						widget_event_callback_t cb,
 						widget_newpos_callback_t pos_cb,
 						xitk_dnd_callback_t dnd_cb,
@@ -753,12 +886,13 @@ int xitk_get_window_info(xitk_register_key_t key, window_info_t *winf);
 /*
  * Initialization function, should be the first call to widget lib.
  */
-void xitk_init(Display *display, XColor black, int verbosity);
+void xitk_init(Display *display, XColor black, void (*x_lock_display) (Display *display),
+  void (*x_unlock_display) (Display *display), int verbosity);
 
 /*
  *
  */
-char *xitk_set_locale(void);
+const char *xitk_set_locale(void);
 
 /*
  *
@@ -807,8 +941,8 @@ void xitk_stop(void);
 /*
  * Some user settings values.
  */
-char *xitk_get_system_font(void);
-char *xitk_get_default_font(void);
+const char *xitk_get_system_font(void);
+const char *xitk_get_default_font(void);
 int xitk_get_xmb_enability(void);
 void xitk_set_xmb_enability(int value);
 int xitk_get_black_color(void);
@@ -842,7 +976,7 @@ int xitk_is_cursor_out_mask(Display *display, xitk_widget_t *w, Pixmap mask, int
 /**
  *
  */
-xitk_color_names_t *xitk_get_color_name(char *color);
+xitk_color_names_t *xitk_get_color_name(const char *color);
 
 /**
  * return pointer to the xitk_color_names struct.
@@ -1022,17 +1156,17 @@ xitk_image_t *xitk_get_widget_background_skin(xitk_widget_t *w);
 /**
  *
  */
-void xitk_set_widget_tips(xitk_widget_t *w, char *str);
+void xitk_set_widget_tips(xitk_widget_t *w, const char *str);
 
 /**
  *
  */
-void xitk_set_widget_tips_default(xitk_widget_t *w, char *str);
+void xitk_set_widget_tips_default(xitk_widget_t *w, const char *str);
 
 /**
  *
  */
-void xitk_set_widget_tips_and_timeout(xitk_widget_t *w, char *str, unsigned long timeout);
+void xitk_set_widget_tips_and_timeout(xitk_widget_t *w, const char *str, unsigned long timeout);
 
 /**
  *
@@ -1156,98 +1290,41 @@ void xitk_slider_make_backstep(xitk_widget_t *);
  */
 void xitk_slider_callback_exec(xitk_widget_t *);
 
-/*
- * *** Nodes ***
- */
-/**
- * Create a new list.
- */
-xitk_list_t *xitk_list_new (void);
-
-/**
- * Clear list.
- */
-void xitk_list_clear(xitk_list_t *l);
-
-/**
- * Freeing list.
- */
-void xitk_list_free(xitk_list_t *l);
-
-/**
- * Boolean, status of list.
- */
-int xitk_list_is_empty (xitk_list_t *l);
-
-/**
- * return content of first entry in list.
- */
-void *xitk_list_first_content (xitk_list_t *l);
-
-/**
- * return next content in list.
- */
-void *xitk_list_next_content (xitk_list_t *l);
-
-/**
- * Return last content of list.
- */
-void *xitk_list_last_content (xitk_list_t *l);
-
-/**
- * Return previous content of list.
- */
-void *xitk_list_prev_content (xitk_list_t *l);
-
-/**
- * Append content to list.
- */
-void xitk_list_append_content (xitk_list_t *l, void *content);
-
-/**
- * Insert content in list.
- */
-void xitk_list_insert_content (xitk_list_t *l, void *content);
-
-/**
- * Remove current content in list.
- */
-void xitk_list_delete_current (xitk_list_t *l);
-
-/**
- * Return current content
- */
-xitk_node_t *xitk_list_get_current(xitk_list_t *l);
-
 /*
  * *** Label Buttons
  */
 /**
  * Create a labeled button.
  */
-xitk_widget_t *xitk_labelbutton_create(xitk_widget_list_t *wl,
-				       xitk_skin_config_t *skonfig, xitk_labelbutton_widget_t *b);
+xitk_widget_t *xitk_labelbutton_create (xitk_widget_list_t *wl,
+  xitk_skin_config_t *skonfig, const xitk_labelbutton_widget_t *b);
+
+/**
+ *
+ */
+xitk_widget_t *xitk_info_labelbutton_create (xitk_widget_list_t *wl,
+  const xitk_labelbutton_widget_t *b, const xitk_skin_element_info_t *info);
 
 /**
  *
  */
 xitk_widget_t *xitk_noskin_labelbutton_create (xitk_widget_list_t *wl,
-					       xitk_labelbutton_widget_t *b,
-					       int x, int y, int width, int height,
-					       char *ncolor, char *fcolor, char *ccolor, 
-					       char *fname);
+  const xitk_labelbutton_widget_t *b,
+  int x, int y, int width, int height,
+  const char *ncolor, const char *fcolor, const char *ccolor,
+  const char *fname);
 
 /**
  * Change label of button 'widget'.
  */
-int xitk_labelbutton_change_label(xitk_widget_t *, char *);
-int xitk_labelbutton_change_shortcut_label(xitk_widget_t *, char *, int, char *);
+int xitk_labelbutton_change_label (xitk_widget_t *, const char *new_label);
+int xitk_labelbutton_change_shortcut_label(xitk_widget_t *, const char *, int, const char *);
 
 /**
  * Return label of button 'widget'.
  */
-char *xitk_labelbutton_get_label(xitk_widget_t *);
-char *xitk_labelbutton_get_shortcut_label(xitk_widget_t *);
+const char *xitk_labelbutton_get_label(xitk_widget_t *);
+const char *xitk_labelbutton_get_shortcut_label(xitk_widget_t *);
 
 /**
  * Get state of button 'widget'.
@@ -1288,15 +1365,15 @@ void xitk_labelbutton_callback_exec(xitk_widget_t *w);
 /**
  * Create a label widget.
  */
-xitk_widget_t *xitk_label_create(xitk_widget_list_t *wl,
-				 xitk_skin_config_t *skonfig, xitk_label_widget_t *l);
+xitk_widget_t *xitk_label_create (xitk_widget_list_t *wl,
+  xitk_skin_config_t *skonfig, const xitk_label_widget_t *l);
 
 /**
  *
  */
-xitk_widget_t *xitk_noskin_label_create(xitk_widget_list_t *wl,
-					xitk_label_widget_t *l,
-					int x, int y, int width, int height, char *fontname);
+xitk_widget_t *xitk_noskin_label_create (xitk_widget_list_t *wl,
+  const xitk_label_widget_t *l, int x, int y, int width, int height,
+  const char *fontname);
 
 /**
  * Change label of widget 'widget'.
@@ -1306,7 +1383,7 @@ int xitk_label_change_label(xitk_widget_t *l, const char *newlabel);
 /**
  * Get label.
  */
-char *xitk_label_get_label(xitk_widget_t *w);
+const char *xitk_label_get_label(xitk_widget_t *w);
 
 /*
  * *** Image
@@ -1315,7 +1392,8 @@ char *xitk_label_get_label(xitk_widget_t *w);
 /**
  * Load image and return a xitk_image_t data type.
  */
-xitk_image_t *xitk_image_load_image(ImlibData *idata, char *image);
+xitk_image_t *xitk_image_load_image(ImlibData *idata, const char *image);
+void xitk_image_set_pix_font (xitk_image_t *image, const char *format);
 
 /**
  * Create an image widget type.
@@ -1406,7 +1484,7 @@ xitk_widget_t *xitk_browser_create(xitk_widget_list_t *wl,
 xitk_widget_t *xitk_noskin_browser_create(xitk_widget_list_t *wl,
 					  xitk_browser_widget_t *br, GC gc, 
 					  int x, int y, 
-					  int itemw, int itemh, int slidw, char *fontname);
+					  int itemw, int itemh, int slidw, const char *fontname);
 
 /**
  * Redraw buttons/slider
@@ -1481,7 +1559,7 @@ void xitk_browser_set_alignment(xitk_widget_t *w, int align);
 /*
  * Jump to entry in list which match with the alphanum char key.
  */
-void xitk_browser_warp_jump(xitk_widget_t *w, char *key, int modifier);
+void xitk_browser_warp_jump(xitk_widget_t *w, const char *key, int modifier);
 
 xitk_widget_t *xitk_browser_get_browser(xitk_widget_t *w);
 
@@ -1577,7 +1655,7 @@ xitk_widget_t *xitk_inputtext_create (xitk_widget_list_t *wl,
 xitk_widget_t *xitk_noskin_inputtext_create (xitk_widget_list_t *wl,
 					     xitk_inputtext_widget_t *it,
 					     int x, int y, int width, int height,
-					     char *ncolor, char *fcolor, char *fontname);
+					     const char *ncolor, const char *fcolor, const char *fontname);
 /**
  * Return the text of widget.
  */
@@ -1586,7 +1664,7 @@ char *xitk_inputtext_get_text(xitk_widget_t *it);
 /**
  * Change and redisplay the text of widget.
  */
-void xitk_inputtext_change_text(xitk_widget_t *it, char *text);
+void xitk_inputtext_change_text(xitk_widget_t *it, const char *text);
 
 
 /*
@@ -1606,7 +1684,7 @@ void xitk_skin_free_config(xitk_skin_config_t *);
 /*
  * Load the skin configfile.
  */
-int xitk_skin_load_config(xitk_skin_config_t *, char *, char *);
+int xitk_skin_load_config(xitk_skin_config_t *, const char *, const char *);
 
 /*
  * Check skin version.
@@ -1655,17 +1733,17 @@ int xitk_skin_get_coord_y(xitk_skin_config_t *, const char *);
 /*
  *
  */
-char *xitk_skin_get_label_color(xitk_skin_config_t *, const char *);
+const char *xitk_skin_get_label_color(xitk_skin_config_t *, const char *);
 
 /*
  *
  */
-char *xitk_skin_get_label_color_focus(xitk_skin_config_t *, const char *);
+const char *xitk_skin_get_label_color_focus(xitk_skin_config_t *, const char *);
 
 /*
  *
  */
-char *xitk_skin_get_label_color_click(xitk_skin_config_t *, const char *);
+const char *xitk_skin_get_label_color_click(xitk_skin_config_t *, const char *);
 
 /*
  *
@@ -1695,7 +1773,7 @@ int xitk_skin_get_label_alignment(xitk_skin_config_t *, const char *);
 /*
  *
  */
-char *xitk_skin_get_label_fontname(xitk_skin_config_t *, const char *);
+const char *xitk_skin_get_label_fontname(xitk_skin_config_t *, const char *);
 
 /*
  *
@@ -1710,12 +1788,12 @@ int xitk_skin_get_label_staticity(xitk_skin_config_t *, const char *);
 /*
  *
  */
-char *xitk_skin_get_skin_filename(xitk_skin_config_t *, const char *);
+const char *xitk_skin_get_skin_filename(xitk_skin_config_t *, const char *);
 
 /*
  *
  */
-char *xitk_skin_get_slider_skin_filename(xitk_skin_config_t *, const char *);
+const char *xitk_skin_get_slider_skin_filename(xitk_skin_config_t *, const char *);
 
 /*
  *
@@ -1730,12 +1808,12 @@ int xitk_skin_get_slider_radius(xitk_skin_config_t *, const char *);
 /*
  *
  */
-char *xitk_skin_get_logo(xitk_skin_config_t *);
+const char *xitk_skin_get_logo(xitk_skin_config_t *);
 
 /*
  *
  */
-char *xitk_skin_get_animation(xitk_skin_config_t *);
+const char *xitk_skin_get_animation(xitk_skin_config_t *);
 
 /*
  *
@@ -1769,7 +1847,7 @@ void xitk_skin_unlock(xitk_skin_config_t *);
 /*
  *
  */
-xitk_font_t *xitk_font_load_font(Display *display, char *font);
+xitk_font_t *xitk_font_load_font(Display *display, const char *font);
 
 /*
  *
@@ -1796,7 +1874,7 @@ int xitk_font_get_string_length(xitk_font_t *xtfs, const char *c);
 /*
  *
  */
-int xitk_font_get_char_width(xitk_font_t *xtfs, char *c, int maxnbytes, int *nbytes);
+int xitk_font_get_char_width(xitk_font_t *xtfs, const char *c, int maxnbytes, int *nbytes);
 
 /*
  *
@@ -1811,7 +1889,7 @@ int xitk_font_get_string_height(xitk_font_t *xtfs, const char *c);
 /*
  *
  */
-int xitk_font_get_char_height(xitk_font_t *xtfs, char *c, int maxnbytes, int *nbytes);
+int xitk_font_get_char_height(xitk_font_t *xtfs, const char *c, int maxnbytes, int *nbytes);
 
 /*
  *
@@ -1943,13 +2021,13 @@ unsigned int xitk_get_pixel_color_warning_background(ImlibData *im);
  *
  */
 xitk_image_t *xitk_image_create_image_with_colors_from_string(ImlibData *im, 
-							      char *fontname, 
-							      int width, int align, char *str,
+                                                              const char *fontname,
+                                                              int width, int align, const char *str,
 							      unsigned int foreground,
 							      unsigned int background);
 xitk_image_t *xitk_image_create_image_from_string(ImlibData *im, 
-						  char *fontname, 
-						  int width, int align, char *str);
+                                                  const char *fontname,
+                                                  int width, int align, const char *str);
 xitk_image_t *xitk_image_create_image(ImlibData *im, int width, int height);
 
 xitk_pixmap_t *xitk_image_create_xitk_pixmap_with_depth(ImlibData *im, int width, int height, int depth);
@@ -2061,10 +2139,10 @@ void draw_rectangular_outter_box_light(ImlibData *im, xitk_pixmap_t *p,
 /**
  *
  */
-void draw_inner_frame(ImlibData *im, xitk_pixmap_t *p, char *title, char *fontname,
-		      int x, int y, int w, int h);
-void draw_outter_frame(ImlibData *im, xitk_pixmap_t *p, char *title, char *fontname,
-		       int x, int y, int w, int h);
+void draw_inner_frame(ImlibData *im, xitk_pixmap_t *p, const char *title, const char *fontname,
+                      int x, int y, int w, int h);
+void draw_outter_frame(ImlibData *im, xitk_pixmap_t *p, const char *title, const char *fontname,
+                       int x, int y, int w, int h);
 
 void draw_tab(ImlibData *im, xitk_image_t *p);
 
@@ -2094,7 +2172,7 @@ xitk_window_t *xitk_window_create_simple_window(ImlibData *im, int x, int y, int
 /**
  *
  */
-xitk_window_t *xitk_window_create_dialog_window(ImlibData *im, char *title, int x, int y, int width, int height);
+xitk_window_t *xitk_window_create_dialog_window(ImlibData *im, const char *title, int x, int y, int width, int height);
 
 
 /**
@@ -2167,12 +2245,12 @@ void xitk_window_get_window_position(ImlibData *im, xitk_window_t *w,
 /*
  *
  */
-void xitk_set_window_title(Display *display, Window window, char *title);
+void xitk_set_window_title(Display *display, Window window, const char *title);
 
 /*
  *
  */
-void xitk_window_set_window_title(ImlibData *im, xitk_window_t *w, char *title);
+void xitk_window_set_window_title(ImlibData *im, xitk_window_t *w, const char *title);
 
 /*
  *
@@ -2194,31 +2272,31 @@ Window xitk_get_desktop_root_window(Display *display, int screen, Window *client
  */
 int xitk_is_window_size(Display *display, Window window, int width, int height);
 
-xitk_window_t *xitk_window_dialog_button_free_with_width(ImlibData *im, char *title,
-							 int window_width, int align, char *message, ...)
+xitk_window_t *xitk_window_dialog_button_free_with_width(ImlibData *im, const char *title,
+                                                         int window_width, int align, const char *message, ...)
 							 __attribute__ ((format (printf, 5, 6)));
 
 /*
  *
  */
-xitk_window_t *xitk_window_dialog_one_button_with_width(ImlibData *im, char *title, char *button_label,
+xitk_window_t *xitk_window_dialog_one_button_with_width(ImlibData *im, const char *title, const char *button_label,
 							xitk_state_callback_t cb, void *userdata, 
-							int window_width, int align, char *message, ...)
+                                                        int window_width, int align, const char *message, ...)
 							__attribute__ ((format (printf, 8, 9)));
 
 /**
  *
  */
-xitk_window_t *xitk_window_dialog_ok_with_width(ImlibData *im, char *title,
+xitk_window_t *xitk_window_dialog_ok_with_width(ImlibData *im, const char *title,
 						xitk_state_callback_t cb, void *userdata, 
-						int window_width, int align, char *message, ...)
+                                                int window_width, int align, const char *message, ...)
 						__attribute__ ((format (printf, 7, 8)));
 
 /**
  *
  */
-xitk_window_t *xitk_window_dialog_ok(ImlibData *im, char *title,
-				     xitk_state_callback_t cb, void *userdata, int align, char *message, ...)
+xitk_window_t *xitk_window_dialog_ok(ImlibData *im, const char *title,
+                                     xitk_state_callback_t cb, void *userdata, int align, const char *message, ...)
 				     __attribute__ ((format (printf, 6, 7)));
 
 
@@ -2244,35 +2322,35 @@ xitk_window_t *xitk_window_dialog_ok(ImlibData *im, char *title,
 /* 
  *
  */
-xitk_window_t *xitk_window_dialog_three_buttons_with_width(ImlibData *im, char *title, 
-							   char *button1_label,
-							   char *button2_label,
-							   char *button3_label,
+xitk_window_t *xitk_window_dialog_three_buttons_with_width(ImlibData *im, const char *title, 
+                                                           const char *button1_label,
+                                                           const char *button2_label,
+                                                           const char *button3_label,
 							   xitk_state_callback_t cb1, 
 							   xitk_state_callback_t cb2, 
 							   xitk_state_callback_t cb3, 
 							   void *userdata, 
-							   int window_width, int align, char *message, ...)
+                                                           int window_width, int align, const char *message, ...)
 							   __attribute__ ((format (printf, 12, 13)));
 /**
  *
  */
-xitk_window_t *xitk_window_dialog_yesnocancel_with_width(ImlibData *im, char *title,
+xitk_window_t *xitk_window_dialog_yesnocancel_with_width(ImlibData *im, const char *title,
 							 xitk_state_callback_t ycb, 
 							 xitk_state_callback_t ncb, 
 							 xitk_state_callback_t ccb, 
 							 void *userdata, 
-							 int window_width, int align, char *message, ...)
+                                                         int window_width, int align, const char *message, ...)
 							 __attribute__ ((format (printf, 9, 10)));
 
 /**
  *
  */
-xitk_window_t *xitk_window_dialog_yesnocancel(ImlibData *im, char *title,
+xitk_window_t *xitk_window_dialog_yesnocancel(ImlibData *im, const char *title,
 					      xitk_state_callback_t ycb, 
 					      xitk_state_callback_t ncb, 
 					      xitk_state_callback_t ccb, 
-					      void *userdata, int align, char *message, ...)
+                                              void *userdata, int align, const char *message, ...)
 					      __attribute__ ((format (printf, 8, 9)));
 
 #ifdef __GNUC__
@@ -2286,14 +2364,15 @@ xitk_window_t *xitk_window_dialog_yesnocancel(ImlibData *im, char *title,
 /*
  *
  */
-xitk_window_t *xitk_window_dialog_checkbox_two_buttons_with_width(ImlibData *im, char *title,
-							 char *button1_label, char *button2_label,
-							 xitk_state_callback_t cb1, 
-							 xitk_state_callback_t cb2, 
-                                                         char *checkbox_label, int checkbox_state,
-                                                         xitk_state_callback_t cb3,
-							 void *userdata, 
-							 int window_width, int align, char *message, ...)
+xitk_window_t *xitk_window_dialog_checkbox_two_buttons_with_width(ImlibData *im, const char *title,
+                                                                  const char *button1_label,
+                                                                  const char *button2_label,
+                                                                  xitk_state_callback_t cb1,
+                                                                  xitk_state_callback_t cb2,
+                                                                  const char *checkbox_label, int checkbox_state,
+                                                                  xitk_state_callback_t cb3,
+                                                                  void *userdata,
+                                                                  int window_width, int align, const char *message, ...)
   __attribute__((__format__(__printf__, 13, 14)));
 
 #ifdef __GNUC__
@@ -2307,20 +2386,20 @@ xitk_window_t *xitk_window_dialog_checkbox_two_buttons_with_width(ImlibData *im,
 /**
  *
  */
-xitk_window_t *xitk_window_dialog_yesno_with_width(ImlibData *im, char *title,
+xitk_window_t *xitk_window_dialog_yesno_with_width(ImlibData *im, const char *title,
 						   xitk_state_callback_t ycb, 
 						   xitk_state_callback_t ncb, 
 						   void *userdata, 
-						   int window_width, int align, char *message, ...)
+                                                   int window_width, int align, const char *message, ...)
 						   __attribute__ ((format (printf, 8, 9)));
 
 /**
  *
  */
-xitk_window_t *xitk_window_dialog_yesno(ImlibData *im, char *title,
+xitk_window_t *xitk_window_dialog_yesno(ImlibData *im, const char *title,
 					xitk_state_callback_t ycb, 
 					xitk_state_callback_t ncb, 
-					void *userdata, int align, char *message, ...)
+                                        void *userdata, int align, const char *message, ...)
 					__attribute__ ((format (printf, 7, 8)));
 #ifdef __GNUC__
 #define xitk_window_dialog_yesno(im, title, ycb, ncb, userdata, align, message, args...) \
@@ -2337,9 +2416,9 @@ void xitk_window_dialog_set_modal(xitk_window_t *w);
 
 xitk_widget_t *xitk_noskin_tabs_create(xitk_widget_list_t *wl,
 				       xitk_tabs_widget_t *t, 
-				       int x, int y, int width, char *fontname);
+                                       int x, int y, int width, const char *fontname);
 int xitk_tabs_get_current_selected(xitk_widget_t *w);
-char *xitk_tabs_get_current_tab_selected(xitk_widget_t *w);
+const char *xitk_tabs_get_current_tab_selected(xitk_widget_t *w);
 void xitk_tabs_set_current_selected(xitk_widget_t *w, int select);
 
 xitk_widget_t *xitk_noskin_intbox_create(xitk_widget_list_t *wl,
